@@ -1,19 +1,19 @@
 /*
- * Copyright (c) 2007-2010 by The Broad Institute, Inc. and the Massachusetts Institute of Technology.
- * All Rights Reserved.
+ * Copyright (c) 2007-2011 by The Broad Institute of MIT and Harvard.  All Rights Reserved.
  *
- * This software is licensed under the terms of the GNU Lesser General Public License (LGPL), Version 2.1 which
- * is available at http://www.opensource.org/licenses/lgpl-2.1.php.
+ * This software is licensed under the terms of the GNU Lesser General Public License (LGPL),
+ * Version 2.1 which is available at http://www.opensource.org/licenses/lgpl-2.1.php.
  *
- * THE SOFTWARE IS PROVIDED "AS IS." THE BROAD AND MIT MAKE NO REPRESENTATIONS OR WARRANTIES OF
- * ANY KIND CONCERNING THE SOFTWARE, EXPRESS OR IMPLIED, INCLUDING, WITHOUT LIMITATION, WARRANTIES
- * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NONINFRINGEMENT, OR THE ABSENCE OF LATENT
- * OR OTHER DEFECTS, WHETHER OR NOT DISCOVERABLE.  IN NO EVENT SHALL THE BROAD OR MIT, OR THEIR
- * RESPECTIVE TRUSTEES, DIRECTORS, OFFICERS, EMPLOYEES, AND AFFILIATES BE LIABLE FOR ANY DAMAGES OF
- * ANY KIND, INCLUDING, WITHOUT LIMITATION, INCIDENTAL OR CONSEQUENTIAL DAMAGES, ECONOMIC
- * DAMAGES OR INJURY TO PROPERTY AND LOST PROFITS, REGARDLESS OF WHETHER THE BROAD OR MIT SHALL
- * BE ADVISED, SHALL HAVE OTHER REASON TO KNOW, OR IN FACT SHALL KNOW OF THE POSSIBILITY OF THE
- * FOREGOING.
+ * THE SOFTWARE IS PROVIDED "AS IS." THE BROAD AND MIT MAKE NO REPRESENTATIONS OR
+ * WARRANTES OF ANY KIND CONCERNING THE SOFTWARE, EXPRESS OR IMPLIED, INCLUDING,
+ * WITHOUT LIMITATION, WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
+ * PURPOSE, NONINFRINGEMENT, OR THE ABSENCE OF LATENT OR OTHER DEFECTS, WHETHER
+ * OR NOT DISCOVERABLE.  IN NO EVENT SHALL THE BROAD OR MIT, OR THEIR RESPECTIVE
+ * TRUSTEES, DIRECTORS, OFFICERS, EMPLOYEES, AND AFFILIATES BE LIABLE FOR ANY DAMAGES
+ * OF ANY KIND, INCLUDING, WITHOUT LIMITATION, INCIDENTAL OR CONSEQUENTIAL DAMAGES,
+ * ECONOMIC DAMAGES OR INJURY TO PROPERTY AND LOST PROFITS, REGARDLESS OF WHETHER
+ * THE BROAD OR MIT SHALL BE ADVISED, SHALL HAVE OTHER REASON TO KNOW, OR IN FACT
+ * SHALL KNOW OF THE POSSIBILITY OF THE FOREGOING.
  */
 /*
  * FeatureTileManager.java
@@ -28,16 +28,14 @@ package org.broad.igv.data;
 //~--- non-JDK imports --------------------------------------------------------
 
 import org.apache.log4j.Logger;
-import org.broad.igv.feature.Feature;
-import org.broad.igv.feature.FeatureUtils;
-import org.broad.igv.feature.GeneManager;
-import org.broad.igv.feature.LocusScore;
+import org.broad.igv.feature.*;
+import org.broad.igv.feature.genome.Genome;
+import org.broad.igv.ui.panel.FrameManager;
 import org.broad.igv.h5.DataAccessException;
 import org.broad.igv.h5.HDF5Reader;
 import org.broad.igv.h5.HDF5ReaderFactory;
 import org.broad.igv.h5.ObjectNotFoundException;
-import org.broad.igv.renderer.RendererFactory;
-import org.broad.igv.session.ViewContext;
+import org.broad.igv.session.RendererFactory;
 import org.broad.igv.track.TrackProperties;
 import org.broad.igv.track.TrackType;
 import org.broad.igv.track.WindowFunction;
@@ -67,7 +65,7 @@ public class HDFDataManager {
     /**
      * Cache of zoom level parameters.  Key is the zoom level
      */
-    private LRUCache<String, ZoomParameters> zoomParameterCache = new LRUCache(1000);
+    private LRUCache<String, ZoomParameters> zoomParameterCache = new LRUCache(this, 100);
     /**
      * Cache of rawMeansPath arrays -- shared across all instances.  Used for location arrays
      */
@@ -107,6 +105,7 @@ public class HDFDataManager {
     private WindowFunction windowFunction;
     private TrackProperties trackProperties;
     private ResourceLocator locator;
+    private Genome genome;
 
     /**
      * Creates a new instance of FeatureTileManager for a given track and chromosome
@@ -114,8 +113,13 @@ public class HDFDataManager {
      * @param locator
      * @throws DataAccessException
      */
-    public HDFDataManager(ResourceLocator locator) throws DataAccessException {
 
+    public HDFDataManager(ResourceLocator locator) throws DataAccessException {
+        this(locator, null);
+    }
+
+    public HDFDataManager(ResourceLocator locator, Genome genome) throws DataAccessException {
+        this.genome = genome;
         int rootGroup = -1;
         try {
             setGroupPaths();
@@ -497,14 +501,9 @@ public class HDFDataManager {
 
                     if (!Float.isNaN(value)) {
 
-                        // TODO - replace with statitstic, maybe based on confidence interval?
-                        // Note: Confidence in the "count" is by definition 100%
-                        float confidence = 1.0f;
-
                         summaryTile.addScore(j, new SummaryScore(
-                                (int) (tile.getStart(i) * locationUnit),
-                                (int) (tile.getEnd(i) * locationUnit), value,
-                                confidence));
+                                (tile.getStart(i) * locationUnit),
+                                (int) (tile.getEnd(i) * locationUnit), value));
                     }
 
                 }
@@ -667,10 +666,14 @@ public class HDFDataManager {
     private synchronized int getLongestFeature(String chr) {
 
         if (!longestFeatureCache.containsKey(chr)) {
-            String genomeId = ViewContext.getInstance().getGenomeId();
-            // Put a limit on the "longest feature" to prevent outliers from dictating
-            // huge data loads.  This is neccessary due to a flaw in the indexing scheme.
-            int maxLongestFeature = GeneManager.getGeneManager(genomeId).getLongestGeneLength(chr) + 100;
+            int maxLongestFeature = 1000;
+
+            if (genome != null) {
+                String genomeId = genome.getId();
+                // Put a limit on the "longest feature" to prevent outliers from dictating
+                // huge data loads.  This is neccessary due to a flaw in the indexing scheme.
+                maxLongestFeature = 5000;
+            }
             int longestFeature = 1000;
 
             String groupName = featureGroupPath + chr + "/raw";
@@ -706,7 +709,7 @@ public class HDFDataManager {
 
         // The binWidth is the width in base pairs of a single screen pixel
         // TODO -- binWidth should be passed in
-        double binWidth = ViewContext.getInstance().getScale();
+        double binWidth = FrameManager.getDefaultFrame().getScale();
 
         // Get the raw data as a list of tiles.
         List<DataTile2D> tiles = getRawData(chr, startLocation, endLocation);
@@ -741,7 +744,7 @@ public class HDFDataManager {
 
                         // NaN is treated as a missing value (as if its not there)
                         if (!Float.isNaN(val)) {
-                            SummaryScore ss = new SummaryScore(start, end, val, 1.0f);
+                            SummaryScore ss = new SummaryScore(start, end, val);
                             double pixelWidth = (end - start) / binWidth;
                             if (pixelWidth < 1) {
                                 scoresToBin.get(trackNumber).add(ss);
@@ -791,7 +794,7 @@ public class HDFDataManager {
                         // to add these.
                         if (!Float.isNaN(binValue)) {
                             tile.addScore(trackNumber,
-                                    new SummaryScore(binStart, binEnd, binValue, 1.0f));
+                                    new SummaryScore(binStart, binEnd, binValue));
                         }
                     }
 
@@ -815,7 +818,7 @@ public class HDFDataManager {
 
     private static List<FeatureBin> computeBins(int startLocation, int endLocation,
                                                 List<LocusScore> features) {
-        double binSize = ViewContext.getInstance().getScale();
+        double binSize = FrameManager.getDefaultFrame().getScale();
         int nBins = (int) Math.ceil((endLocation - startLocation) / binSize);
         double correctedBinSize = ((double) (endLocation - startLocation)) / nBins;
         return (new FeatureBinCalculator()).computeFeatureBins(features, nBins, correctedBinSize,
@@ -846,7 +849,6 @@ public class HDFDataManager {
                             // The only purpose of this is to reconsitute segmented data from
                             // cn files that were created from cbs (segmented) files.
                             previousScore.setEnd(score.getEnd());
-                            previousScore.setConfidence(1);
 
                         } else {
 
@@ -873,11 +875,11 @@ public class HDFDataManager {
 
         return joinedScores;
     }    /*
-     *    private void dumpFeatures(List<Feature> features) {
+     *    private void dumpFeatures(List<IGVFeature> features) {
      * PrintWriter pw = null;
      * try {
      * pw = new PrintWriter(new FileWriter("feature_dump.txt"));
-     * for (Feature f : features) {
+     * for (IGVFeature f : features) {
      * pw.println(f.getStart());
      * }
      * } catch (IOException ex) {
@@ -897,7 +899,7 @@ public class HDFDataManager {
      * @param endLocation
      * @return
      */
-    public List<Feature> getFeaturesForRange(String chr, int startLocation, int endLocation) {
+    public List<IGVFeature> getFeaturesForRange(String chr, int startLocation, int endLocation) {
         return null;
     }
 
@@ -1215,18 +1217,18 @@ public class HDFDataManager {
             try {
                 String altColorString = dataReader.readStringAttribute(group, "track.altColor");
                 getTrackProperties().setAltColor(
-                        ColorUtilities.convertRGBStringToColor(altColorString));
+                        ColorUtilities.stringToColor(altColorString));
             } catch (Exception e) {
             }
             try {
                 String midColorString = dataReader.readStringAttribute(group, "track.midColor");
                 getTrackProperties().setMidColor(
-                        ColorUtilities.convertRGBStringToColor(midColorString));
+                        ColorUtilities.stringToColor(midColorString));
             } catch (Exception e) {
             }
             try {
                 String colorString = dataReader.readStringAttribute(group, "track.color");
-                getTrackProperties().setColor(ColorUtilities.convertRGBStringToColor(colorString));
+                getTrackProperties().setColor(ColorUtilities.stringToColor(colorString));
             } catch (Exception e) {
             }
             try {
@@ -1261,7 +1263,7 @@ public class HDFDataManager {
             }
             try {
                 String drawMidValue = dataReader.readStringAttribute(group, "track.drawMidValue");
-                getTrackProperties().setDrawMidValue(Boolean.valueOf(drawMidValue));
+                getTrackProperties().setDrawYLine(Boolean.valueOf(drawMidValue));
             } catch (Exception e) {
             }
             try {
