@@ -1,33 +1,34 @@
 /*
- * Copyright (c) 2007-2010 by The Broad Institute, Inc. and the Massachusetts Institute of Technology.
- * All Rights Reserved.
+ * Copyright (c) 2007-2011 by The Broad Institute of MIT and Harvard.  All Rights Reserved.
  *
- * This software is licensed under the terms of the GNU Lesser General Public License (LGPL), Version 2.1 which
- * is available at http://www.opensource.org/licenses/lgpl-2.1.php.
+ * This software is licensed under the terms of the GNU Lesser General Public License (LGPL),
+ * Version 2.1 which is available at http://www.opensource.org/licenses/lgpl-2.1.php.
  *
- * THE SOFTWARE IS PROVIDED "AS IS." THE BROAD AND MIT MAKE NO REPRESENTATIONS OR WARRANTIES OF
- * ANY KIND CONCERNING THE SOFTWARE, EXPRESS OR IMPLIED, INCLUDING, WITHOUT LIMITATION, WARRANTIES
- * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NONINFRINGEMENT, OR THE ABSENCE OF LATENT
- * OR OTHER DEFECTS, WHETHER OR NOT DISCOVERABLE.  IN NO EVENT SHALL THE BROAD OR MIT, OR THEIR
- * RESPECTIVE TRUSTEES, DIRECTORS, OFFICERS, EMPLOYEES, AND AFFILIATES BE LIABLE FOR ANY DAMAGES OF
- * ANY KIND, INCLUDING, WITHOUT LIMITATION, INCIDENTAL OR CONSEQUENTIAL DAMAGES, ECONOMIC
- * DAMAGES OR INJURY TO PROPERTY AND LOST PROFITS, REGARDLESS OF WHETHER THE BROAD OR MIT SHALL
- * BE ADVISED, SHALL HAVE OTHER REASON TO KNOW, OR IN FACT SHALL KNOW OF THE POSSIBILITY OF THE
- * FOREGOING.
+ * THE SOFTWARE IS PROVIDED "AS IS." THE BROAD AND MIT MAKE NO REPRESENTATIONS OR
+ * WARRANTES OF ANY KIND CONCERNING THE SOFTWARE, EXPRESS OR IMPLIED, INCLUDING,
+ * WITHOUT LIMITATION, WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
+ * PURPOSE, NONINFRINGEMENT, OR THE ABSENCE OF LATENT OR OTHER DEFECTS, WHETHER
+ * OR NOT DISCOVERABLE.  IN NO EVENT SHALL THE BROAD OR MIT, OR THEIR RESPECTIVE
+ * TRUSTEES, DIRECTORS, OFFICERS, EMPLOYEES, AND AFFILIATES BE LIABLE FOR ANY DAMAGES
+ * OF ANY KIND, INCLUDING, WITHOUT LIMITATION, INCIDENTAL OR CONSEQUENTIAL DAMAGES,
+ * ECONOMIC DAMAGES OR INJURY TO PROPERTY AND LOST PROFITS, REGARDLESS OF WHETHER
+ * THE BROAD OR MIT SHALL BE ADVISED, SHALL HAVE OTHER REASON TO KNOW, OR IN FACT
+ * SHALL KNOW OF THE POSSIBILITY OF THE FOREGOING.
  */
 package org.broad.igv.data;
 
 //~--- non-JDK imports --------------------------------------------------------
 
+import org.broad.igv.feature.genome.GenomeManager;
+import org.broad.igv.track.TrackType;
+import org.broad.igv.ui.IGV;
 import org.broad.igv.util.collections.FloatArrayList;
 import org.broad.igv.util.collections.IntArrayList;
 import org.apache.log4j.Logger;
 import org.broad.igv.exceptions.ParserException;
-import org.broad.igv.feature.Genome;
-import org.broad.igv.feature.GenomeManager;
-import org.broad.igv.session.ViewContext;
+import org.broad.igv.feature.genome.Genome;
 import org.broad.igv.track.TrackProperties;
-import org.broad.igv.util.AsciiLineReader;
+import org.broad.tribble.readers.AsciiLineReader;
 import org.broad.igv.util.ParsingUtils;
 import org.broad.igv.util.ResourceLocator;
 
@@ -38,7 +39,7 @@ import java.util.Map;
 import java.util.Set;
 
 /**
- * Class description
+ * Parser for wiggle and "wiggle-like" formats.
  *
  * @author Enter your name here...
  * @version Enter version here..., 08/10/31
@@ -46,19 +47,23 @@ import java.util.Set;
 public class WiggleParser {
 
     private static Logger log = Logger.getLogger(WiggleParser.class);
+    private int chrColumn = 0;
+    private int startColumn = 1;
+    private int endColumn = 2;
+    private int dataColumn = 3;
 
     private enum Type {
 
-        FIXED, VARIABLE, BED
+        FIXED, VARIABLE, BED_GRAPH, CPG, EXPR
     }
 
-    ;
+    Genome genome;
 
     WiggleDataset dataset;
     /**
      * The type of wiggle locator (see UCSC documentation).
      */
-    private Type type = Type.BED;
+    private Type type = Type.BED_GRAPH;
 
     // State variables.  This is a serial type parser,  these variables are used to hold temporary
     // state.
@@ -77,27 +82,37 @@ public class WiggleParser {
     int estArraySize;
     Map<String, Integer> longestFeatureMap = new HashMap();
 
-
-    /**
-     * Constructs ...
-     *
-     * @param locator
-     * @param genomeId
-     */
-    public WiggleParser(ResourceLocator locator, String genomeId) {
-
-        this.resourceLocator = locator;
-        this.estArraySize = estArraySize(locator, genomeId);
-        dataset = new WiggleDataset(genomeId, locator.getTrackName());
+    public WiggleParser(ResourceLocator locator) {
+        this(locator, null);
     }
 
-    private int estArraySize(ResourceLocator locator, String genomeId) {
+    public WiggleParser(ResourceLocator locator, Genome genome) {
+        this.genome = genome;
+        this.resourceLocator = locator;
+        this.estArraySize = estArraySize(locator, genome);
+        dataset = new WiggleDataset(genome, locator.getTrackName());
+
+        if (locator.getPath().endsWith("CpG.txt")) {
+            type = Type.CPG;
+        } else if (locator.getPath().toLowerCase().endsWith(".expr")) {
+            //gene_id	bundle_id	chr	left	right	FPKM	FPKM_conf_lo	FPKM_conf_hi
+            type = Type.EXPR;
+            chrColumn = 2;
+            startColumn = 3;
+            endColumn = 4;
+            dataColumn = 5;
+            startBase = 1;
+            dataset.setType(TrackType.EXPR);
+        }
+    }
+
+    private int estArraySize(ResourceLocator locator, Genome genome) {
 
         int estLines = 100000;
         if (locator.getServerURL() == null) {
             estLines = ParsingUtils.estimateLineCount(locator.getPath());
         }
-        int nChromosomes = GenomeManager.getInstance().getGenome(genomeId).getChromosomeNames().size();
+        int nChromosomes = genome.getChromosomeNames().size();
         return Math.max(1000, (int) (estLines / nChromosomes));
 
     }
@@ -111,6 +126,10 @@ public class WiggleParser {
      * @return
      */
     public static boolean isWiggle(ResourceLocator file) {
+
+        if (file.getPath().endsWith("CpG.txt") || file.getPath().endsWith(".expr")) {
+            return true;
+        }
         AsciiLineReader reader = null;
         try {
             reader = ParsingUtils.openAsciiReader(file);
@@ -149,18 +168,22 @@ public class WiggleParser {
         try {
             reader = ParsingUtils.openAsciiReader(resourceLocator);
 
+            if (type == type.EXPR) {
+                reader.readLine(); // Skip header line
+            }
+
             int position = -1;
 
-            while ((nextLine = reader.readLine()) != null && (nextLine.trim().length() > 0)) {
+            while ((nextLine = reader.readLine()) != null) {
 
-                if (nextLine.startsWith("#") || nextLine.startsWith("data") || nextLine.startsWith("browser")) {
+                if (nextLine.startsWith("#") || nextLine.startsWith("data") || nextLine.startsWith("browser") || nextLine.trim().length() == 0) {
                     continue;
                     // Skip
                 }
 
 
-                if (nextLine.startsWith("track")) {
-                    type = Type.BED;
+                if (nextLine.startsWith("track") && type != Type.CPG) {
+                    type = Type.BED_GRAPH;
                     ParsingUtils.parseTrackLine(nextLine, dataset.getTrackProperties());
                     if (dataset.getTrackProperties().getBaseCoord() == TrackProperties.BaseCoord.ZERO) {
                         this.startBase = 0;
@@ -188,12 +211,46 @@ public class WiggleParser {
                         continue;
                     }
                     try {
-
-
-                        if (type.equals(Type.BED)) {
+                        if (type.equals(Type.CPG)) {
 
                             if (nTokens > 3) {
-                                chr = tokens[0].trim();
+                                chr = tokens[1].trim();
+                                if (!chr.equals(lastChr)) {
+                                    changedChromosome(dataset, lastChr);
+
+                                }
+                                lastChr = chr;
+
+                                int endPosition = -1;
+                                try {
+                                    endPosition = Integer.parseInt(tokens[2].trim());
+                                } catch (NumberFormatException numberFormatException) {
+                                    log.error("Column 2  is not a number");
+
+                                    throw new ParserException("Column 2 must be numeric." + " Found: " + tokens[1],
+                                            reader.getCurrentLineNumber(), nextLine);
+                                }
+                                int startPosition = endPosition - 1;
+
+                                if (startPosition < lastPosition) {
+                                    unsortedChromosomes.add(chr);
+                                }
+                                lastPosition = startPosition;
+
+                                startLocations.add(startPosition);
+                                endLocations.add(endPosition);
+
+                                float value = Float.parseFloat(tokens[4].trim());
+                                if (tokens[3].trim().equals("R")) {
+                                    value = -value;
+                                }
+
+                                data.add(value);
+                            }
+                        } else if (type.equals(Type.BED_GRAPH) || type.equals(Type.EXPR)) {
+
+                            if (nTokens > 3) {
+                                chr = tokens[chrColumn].trim();
                                 if (!chr.equals(lastChr)) {
                                     changedChromosome(dataset, lastChr);
 
@@ -202,11 +259,12 @@ public class WiggleParser {
 
                                 int startPosition = -1;
                                 try {
-                                    startPosition = Integer.parseInt(tokens[1].trim());
+                                    startPosition = Integer.parseInt(tokens[startColumn].trim());
                                 } catch (NumberFormatException numberFormatException) {
-                                    log.error("Column 2  is not a number");
+                                    log.error("Column " + (startColumn + 1) + "  is not a number");
 
-                                    throw new ParserException("Column 2 must be numeric." + " Found: " + tokens[1],
+                                    throw new ParserException("Column (startColumn + 1) must be numeric." + " Found: " +
+                                            tokens[startColumn],
                                             reader.getCurrentLineNumber(), nextLine);
                                 }
 
@@ -219,20 +277,20 @@ public class WiggleParser {
 
 
                                 try {
-                                    int endPosition = Integer.parseInt(tokens[2].trim());
+                                    int endPosition = Integer.parseInt(tokens[endColumn].trim());
                                     endLocations.add(endPosition);
                                     int length = endPosition - startPosition;
                                     updateLongestFeature(length);
                                 }
                                 catch (NumberFormatException numberFormatException) {
-                                    log.error("Column 3  is not a number");
+                                    log.error("Column " + (endColumn + 1) + " is not a number");
 
-                                    throw new ParserException("Column 3 must be numeric." + " Found: " + tokens[2],
+                                    throw new ParserException("Column " + (endColumn + 1) +
+                                            " must be numeric." + " Found: " + tokens[endColumn],
                                             reader.getCurrentLineNumber(), nextLine);
                                 }
 
-
-                                data.add(Float.parseFloat(tokens[3].trim()));
+                                data.add(Float.parseFloat(tokens[dataColumn].trim()));
                             }
                         } else if (type.equals(Type.VARIABLE)) {
                             if (nTokens > 1) {
@@ -338,13 +396,11 @@ public class WiggleParser {
     }
 
 
-
     private void changedChromosome(WiggleDataset dataset, String lastChr) {
 
-        Genome genome = ViewContext.getInstance().getGenome();
         if (startLocations != null && startLocations.size() > 0) {
 
-            String convertedChr = genome.getChromosomeAlias(lastChr);
+            String convertedChr = genome == null ? lastChr : genome.getChromosomeAlias(lastChr);
             dataset.addDataChunk(convertedChr, startLocations, endLocations, data);
             //sz = startLocations.size();
         }
