@@ -1,19 +1,19 @@
 /*
- * Copyright (c) 2007-2010 by The Broad Institute, Inc. and the Massachusetts Institute of Technology.
- * All Rights Reserved.
+ * Copyright (c) 2007-2011 by The Broad Institute of MIT and Harvard.  All Rights Reserved.
  *
- * This software is licensed under the terms of the GNU Lesser General Public License (LGPL), Version 2.1 which
- * is available at http://www.opensource.org/licenses/lgpl-2.1.php.
+ * This software is licensed under the terms of the GNU Lesser General Public License (LGPL),
+ * Version 2.1 which is available at http://www.opensource.org/licenses/lgpl-2.1.php.
  *
- * THE SOFTWARE IS PROVIDED "AS IS." THE BROAD AND MIT MAKE NO REPRESENTATIONS OR WARRANTIES OF
- * ANY KIND CONCERNING THE SOFTWARE, EXPRESS OR IMPLIED, INCLUDING, WITHOUT LIMITATION, WARRANTIES
- * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NONINFRINGEMENT, OR THE ABSENCE OF LATENT
- * OR OTHER DEFECTS, WHETHER OR NOT DISCOVERABLE.  IN NO EVENT SHALL THE BROAD OR MIT, OR THEIR
- * RESPECTIVE TRUSTEES, DIRECTORS, OFFICERS, EMPLOYEES, AND AFFILIATES BE LIABLE FOR ANY DAMAGES OF
- * ANY KIND, INCLUDING, WITHOUT LIMITATION, INCIDENTAL OR CONSEQUENTIAL DAMAGES, ECONOMIC
- * DAMAGES OR INJURY TO PROPERTY AND LOST PROFITS, REGARDLESS OF WHETHER THE BROAD OR MIT SHALL
- * BE ADVISED, SHALL HAVE OTHER REASON TO KNOW, OR IN FACT SHALL KNOW OF THE POSSIBILITY OF THE
- * FOREGOING.
+ * THE SOFTWARE IS PROVIDED "AS IS." THE BROAD AND MIT MAKE NO REPRESENTATIONS OR
+ * WARRANTES OF ANY KIND CONCERNING THE SOFTWARE, EXPRESS OR IMPLIED, INCLUDING,
+ * WITHOUT LIMITATION, WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
+ * PURPOSE, NONINFRINGEMENT, OR THE ABSENCE OF LATENT OR OTHER DEFECTS, WHETHER
+ * OR NOT DISCOVERABLE.  IN NO EVENT SHALL THE BROAD OR MIT, OR THEIR RESPECTIVE
+ * TRUSTEES, DIRECTORS, OFFICERS, EMPLOYEES, AND AFFILIATES BE LIABLE FOR ANY DAMAGES
+ * OF ANY KIND, INCLUDING, WITHOUT LIMITATION, INCIDENTAL OR CONSEQUENTIAL DAMAGES,
+ * ECONOMIC DAMAGES OR INJURY TO PROPERTY AND LOST PROFITS, REGARDLESS OF WHETHER
+ * THE BROAD OR MIT SHALL BE ADVISED, SHALL HAVE OTHER REASON TO KNOW, OR IN FACT
+ * SHALL KNOW OF THE POSSIBILITY OF THE FOREGOING.
  */
 package org.broad.igv.data.seg;
 
@@ -22,20 +22,25 @@ package org.broad.igv.data.seg;
 import org.apache.log4j.Logger;
 import org.broad.igv.exceptions.DataLoadException;
 import org.broad.igv.exceptions.ParserException;
-import org.broad.igv.feature.Genome;
-import org.broad.igv.session.ViewContext;
-import org.broad.igv.track.TrackProperties;
+import org.broad.igv.feature.genome.Genome;
+import org.broad.igv.feature.genome.GenomeManager;
 import org.broad.igv.track.TrackType;
-import org.broad.igv.util.AsciiLineReader;
+import org.broad.igv.ui.IGV;
+import org.broad.tribble.readers.AsciiLineReader;
 import org.broad.igv.util.ParsingUtils;
 import org.broad.igv.util.ResourceLocator;
 
-import java.util.HashSet;
-import java.util.Set;
+import java.awt.*;
+import java.util.ArrayList;
 
 /**
- * @author Enter your name here...
- * @version Enter version here..., 09/01/09
+ * Example
+ * CCLE_name	chrom	loc.start	loc.end	num.mark	seg.mean
+ * A2780_OVARY	1	51598	88465	17	1.0491
+ * A2780_OVARY	1	218569	55606710	30512	0.0248
+ * A2780_OVARY	1	55606801	61331401	4399	-0.0431
+ *
+ * @author jrobinso
  */
 public class SegmentFileParser implements SegFileParser {
 
@@ -52,7 +57,7 @@ public class SegmentFileParser implements SegFileParser {
     int chrColumn = 1;
     int startColumn = 2;
     int endColumn = 3;
-    int snpCountColumn = 4;    // Default value
+    //int snpCountColumn = 4;    // Default value
     int dataColumn = 5;        // Default value
     ResourceLocator locator;
 
@@ -74,7 +79,7 @@ public class SegmentFileParser implements SegFileParser {
      *
      * @return
      */
-    public void loadSegments(SegmentedAsciiDataSet dataset) {
+    public void loadSegments(SegmentedAsciiDataSet dataset, Genome genome) {
 
         if (birdsuite) {
             dataset.setTrackType(TrackType.CNV);
@@ -101,30 +106,27 @@ public class SegmentFileParser implements SegFileParser {
             if (birdsuite) {
                 //sample	sample_index	copy_number	chr	start	end	confidence
                 sampleColumn = 0;
+                dataColumn = 2;
                 chrColumn = 3;
                 startColumn = 4;
                 endColumn = 5;
-                snpCountColumn = 6;
-                dataColumn = 2;
+
             } else {
-                // The data value is always the last column
+                sampleColumn = 0;
+                chrColumn = 1;
+                startColumn = 2;
+                endColumn = 3;
                 dataColumn = headings.length - 1;
-                // We assume the snp count is next to last, but not 0-3
-                snpCountColumn = dataColumn - 1;
-                if (snpCountColumn < 4) {
-                    snpCountColumn = -1;
-                }
             }
-
+ 
             String[] tokens = new String[headings.length];
 
-            Genome genome = ViewContext.getInstance().getGenome();
-
             while ((nextLine = reader.readLine()) != null && (nextLine.trim().length() > 0)) {
 
                 int nTokens = ParsingUtils.split(nextLine, tokens, '\t');
                 if (nTokens > 4) {
 
+
                     int start;
                     int end;
                     try {
@@ -150,19 +152,30 @@ public class SegmentFileParser implements SegFileParser {
 
                     String trackId = new String(tokens[sampleColumn].trim());
 
-                    int snpCount = 0;
-                    if (snpCountColumn > 0) {
-                        try {
-                            snpCount = Integer.parseInt(tokens[snpCountColumn]);
-                        } catch (NumberFormatException numberFormatException) {
-
-                            // This is an expected condition, nothing needs done.
+                    StringBuffer desc = null;
+                    if (birdsuite) {
+                        desc = new StringBuffer();
+                        desc.append("<br>");
+                        desc.append(headings[6]);
+                        desc.append("=");
+                        desc.append(tokens[6]);
+                    } else {
+                        if (tokens.length > 4) {
+                            desc = new StringBuffer();
+                            for (int i = 4; i < headings.length - 1; i++) {
+                                desc.append("<br>");
+                                desc.append(headings[i]);
+                                desc.append(": ");
+                                desc.append(tokens[i]);
+                            }
                         }
                     }
 
+
                     try {
                         float value = Float.parseFloat(tokens[dataColumn]);
-                        dataset.addSegment(trackId, chr, start, end, value, snpCount);
+                        String description = desc == null ? null : desc.toString();
+                        dataset.addSegment(trackId, chr, start, end, value, description);
                     } catch (NumberFormatException numberFormatException) {
                         log.info("Skipping line: " + nextLine);
                     }
@@ -200,20 +213,20 @@ public class SegmentFileParser implements SegFileParser {
     private void parseComment(String comment, SegmentedAsciiDataSet dataset) {
 
         String tmp = comment.substring(1, comment.length());
-        if (tmp.startsWith("type")) {
-            String[] tokens = tmp.split("=");
-            if (tokens.length != 2) {
-                return;
-            }
+        String[] tokens = tmp.split("=");
+        if (tokens.length != 2) {
+            return;
+        }
+        String key = tokens[0].trim().toLowerCase();
+        if (key.equals("type")) {
+
             try {
                 dataset.setTrackType(TrackType.valueOf(tokens[1].trim().toUpperCase()));
             } catch (Exception exception) {
 
+                // Ignore
+
             }
-        } else if (tmp.startsWith("track")) {
-            TrackProperties props = new TrackProperties();
-            ParsingUtils.parseTrackLine(comment, props);
-            dataset.setTrackProperties(props);
         }
     }
 }
