@@ -1,32 +1,34 @@
 /*
- * Copyright (c) 2007-2010 by The Broad Institute, Inc. and the Massachusetts Institute of Technology.
- * All Rights Reserved.
+ * Copyright (c) 2007-2011 by The Broad Institute of MIT and Harvard.  All Rights Reserved.
  *
- * This software is licensed under the terms of the GNU Lesser General Public License (LGPL), Version 2.1 which
- * is available at http://www.opensource.org/licenses/lgpl-2.1.php.
+ * This software is licensed under the terms of the GNU Lesser General Public License (LGPL),
+ * Version 2.1 which is available at http://www.opensource.org/licenses/lgpl-2.1.php.
  *
- * THE SOFTWARE IS PROVIDED "AS IS." THE BROAD AND MIT MAKE NO REPRESENTATIONS OR WARRANTIES OF
- * ANY KIND CONCERNING THE SOFTWARE, EXPRESS OR IMPLIED, INCLUDING, WITHOUT LIMITATION, WARRANTIES
- * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NONINFRINGEMENT, OR THE ABSENCE OF LATENT
- * OR OTHER DEFECTS, WHETHER OR NOT DISCOVERABLE.  IN NO EVENT SHALL THE BROAD OR MIT, OR THEIR
- * RESPECTIVE TRUSTEES, DIRECTORS, OFFICERS, EMPLOYEES, AND AFFILIATES BE LIABLE FOR ANY DAMAGES OF
- * ANY KIND, INCLUDING, WITHOUT LIMITATION, INCIDENTAL OR CONSEQUENTIAL DAMAGES, ECONOMIC
- * DAMAGES OR INJURY TO PROPERTY AND LOST PROFITS, REGARDLESS OF WHETHER THE BROAD OR MIT SHALL
- * BE ADVISED, SHALL HAVE OTHER REASON TO KNOW, OR IN FACT SHALL KNOW OF THE POSSIBILITY OF THE
- * FOREGOING.
+ * THE SOFTWARE IS PROVIDED "AS IS." THE BROAD AND MIT MAKE NO REPRESENTATIONS OR
+ * WARRANTES OF ANY KIND CONCERNING THE SOFTWARE, EXPRESS OR IMPLIED, INCLUDING,
+ * WITHOUT LIMITATION, WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
+ * PURPOSE, NONINFRINGEMENT, OR THE ABSENCE OF LATENT OR OTHER DEFECTS, WHETHER
+ * OR NOT DISCOVERABLE.  IN NO EVENT SHALL THE BROAD OR MIT, OR THEIR RESPECTIVE
+ * TRUSTEES, DIRECTORS, OFFICERS, EMPLOYEES, AND AFFILIATES BE LIABLE FOR ANY DAMAGES
+ * OF ANY KIND, INCLUDING, WITHOUT LIMITATION, INCIDENTAL OR CONSEQUENTIAL DAMAGES,
+ * ECONOMIC DAMAGES OR INJURY TO PROPERTY AND LOST PROFITS, REGARDLESS OF WHETHER
+ * THE BROAD OR MIT SHALL BE ADVISED, SHALL HAVE OTHER REASON TO KNOW, OR IN FACT
+ * SHALL KNOW OF THE POSSIBILITY OF THE FOREGOING.
  */
 package org.broad.igv.feature;
 
 //~--- non-JDK imports --------------------------------------------------------
 
-import org.broad.igv.renderer.BasicFeatureRenderer;
+import org.apache.log4j.Logger;
+import org.broad.igv.feature.genome.Genome;
+import org.broad.igv.renderer.IGVFeatureRenderer;
 import org.broad.igv.renderer.GeneTrackRenderer;
 import org.broad.igv.track.FeatureCollectionSource;
 import org.broad.igv.track.FeatureTrack;
 import org.broad.igv.track.TrackProperties;
-import org.broad.igv.util.AsciiLineReader;
 import org.broad.igv.util.ParsingUtils;
 import org.broad.igv.util.ResourceLocator;
+import org.broad.tribble.readers.AsciiLineReader;
 
 import java.io.*;
 import java.util.*;
@@ -65,20 +67,20 @@ public class EmblFeatureTableParser implements FeatureParser {
      * @param locator
      * @return
      */
-    public List<FeatureTrack> loadTracks(ResourceLocator locator) {
+    public List<FeatureTrack> loadTracks(ResourceLocator locator, Genome genome) {
 
         AsciiLineReader reader = null;
         try {
             reader = ParsingUtils.openAsciiReader(locator);
 
-            List<Feature> features = loadFeatures(reader);
+            List<org.broad.tribble.Feature> features = loadFeatures(reader);
 
             if (features.isEmpty()) {
                 return null;
             } else {
-                FeatureTrack track = new FeatureTrack(locator, new FeatureCollectionSource(features));
+                FeatureTrack track = new FeatureTrack(locator, new FeatureCollectionSource(features, genome));
                 track.setName(locator.getTrackName());
-                track.setRendererClass(BasicFeatureRenderer.class);
+                track.setRendererClass(IGVFeatureRenderer.class);
                 track.setMinimumHeight(35);
                 track.setHeight(45);
                 track.setRendererClass(GeneTrackRenderer.class);
@@ -106,7 +108,7 @@ public class EmblFeatureTableParser implements FeatureParser {
      * @param reader
      * @return
      */
-    public List<Feature> loadFeatures(AsciiLineReader reader) {
+    public List<org.broad.tribble.Feature> loadFeatures(AsciiLineReader reader) {
 
         List<BasicFeature> features = new ArrayList();
 
@@ -185,9 +187,9 @@ public class EmblFeatureTableParser implements FeatureParser {
 
     // Combine the constituitve gene parts into a single feature (3', 5', and cds).  This is
     // necccessary with the current IGV gene model, which is heavily influence by ucsc conventions
-    static List<Feature> combineGeneParts(List<BasicFeature> features) {
+    static List<org.broad.tribble.Feature> combineGeneParts(List<BasicFeature> features) {
 
-        List<Feature> newFeatureList = new ArrayList(features.size());
+        List<org.broad.tribble.Feature> newFeatureList = new ArrayList(features.size());
         Map<String, BasicFeature> genes = new HashMap();
 
         // Step 1 -- find the coding regions
@@ -231,7 +233,7 @@ public class EmblFeatureTableParser implements FeatureParser {
         // Compute the phases (reading shifts) for the genes
         // TODO -- should we be doing this?  It generally works but could there be reading shifts
         // between exons that would throw this off?
-        for (Feature gene : genes.values()) {
+        for (IGVFeature gene : genes.values()) {
             ParsingUtils.computeReadingShifts(gene);
         }
 
@@ -296,6 +298,8 @@ public class EmblFeatureTableParser implements FeatureParser {
 
     static class EmblRecord {
 
+        private static Logger log = Logger.getLogger(EmblRecord.class);
+
         boolean isNegative;
         private String type;
         private String chromosome;
@@ -306,6 +310,7 @@ public class EmblFeatureTableParser implements FeatureParser {
         private int end;
         List<Exon> exons;
 
+
         EmblRecord(String type, String chromosome, String lociString, boolean isNegative) {
             this.isNegative = isNegative;
             this.type = type;
@@ -482,8 +487,7 @@ public class EmblFeatureTableParser implements FeatureParser {
 
 
                 } catch (NumberFormatException e) {
-                    System.out.println("Error parsing exon number; " + joinString);
-                    e.printStackTrace();
+                    log.error("Error parsing exon number; " + joinString, e);
                 }
             }
         }
@@ -641,15 +645,15 @@ public class EmblFeatureTableParser implements FeatureParser {
         * String fnTemp =
         *   "/Users/jrobinso/IGVTestData/SPombe/chromosome$.contig.embl";
         *
-        * List<Feature> allFeatures = new ArrayList();
+        * List<IGVFeature> allFeatures = new ArrayList();
         *
         * for (int i = 1; i <= 3; i++)
         * {
         *   String chromosome = "chr" + i;
         *   String fn = fnTemp.replace("$", "" + i);
         *
-        *   List<Feature> features = (new EmblFeatureTableParser()).loadTracks(fn);
-        *   for (Feature f : features)
+        *   List<IGVFeature> features = (new EmblFeatureTableParser()).loadTracks(fn);
+        *   for (IGVFeature f : features)
         *   {
         *       ((BasicFeature) f).setChromosome(chromosome);
         *       ((BasicFeature) f).sortExons();
