@@ -1,19 +1,19 @@
 /*
- * Copyright (c) 2007-2010 by The Broad Institute, Inc. and the Massachusetts Institute of Technology.
- * All Rights Reserved.
+ * Copyright (c) 2007-2011 by The Broad Institute of MIT and Harvard.  All Rights Reserved.
  *
- * This software is licensed under the terms of the GNU Lesser General Public License (LGPL), Version 2.1 which
- * is available at http://www.opensource.org/licenses/lgpl-2.1.php.
+ * This software is licensed under the terms of the GNU Lesser General Public License (LGPL),
+ * Version 2.1 which is available at http://www.opensource.org/licenses/lgpl-2.1.php.
  *
- * THE SOFTWARE IS PROVIDED "AS IS." THE BROAD AND MIT MAKE NO REPRESENTATIONS OR WARRANTIES OF
- * ANY KIND CONCERNING THE SOFTWARE, EXPRESS OR IMPLIED, INCLUDING, WITHOUT LIMITATION, WARRANTIES
- * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NONINFRINGEMENT, OR THE ABSENCE OF LATENT
- * OR OTHER DEFECTS, WHETHER OR NOT DISCOVERABLE.  IN NO EVENT SHALL THE BROAD OR MIT, OR THEIR
- * RESPECTIVE TRUSTEES, DIRECTORS, OFFICERS, EMPLOYEES, AND AFFILIATES BE LIABLE FOR ANY DAMAGES OF
- * ANY KIND, INCLUDING, WITHOUT LIMITATION, INCIDENTAL OR CONSEQUENTIAL DAMAGES, ECONOMIC
- * DAMAGES OR INJURY TO PROPERTY AND LOST PROFITS, REGARDLESS OF WHETHER THE BROAD OR MIT SHALL
- * BE ADVISED, SHALL HAVE OTHER REASON TO KNOW, OR IN FACT SHALL KNOW OF THE POSSIBILITY OF THE
- * FOREGOING.
+ * THE SOFTWARE IS PROVIDED "AS IS." THE BROAD AND MIT MAKE NO REPRESENTATIONS OR
+ * WARRANTES OF ANY KIND CONCERNING THE SOFTWARE, EXPRESS OR IMPLIED, INCLUDING,
+ * WITHOUT LIMITATION, WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
+ * PURPOSE, NONINFRINGEMENT, OR THE ABSENCE OF LATENT OR OTHER DEFECTS, WHETHER
+ * OR NOT DISCOVERABLE.  IN NO EVENT SHALL THE BROAD OR MIT, OR THEIR RESPECTIVE
+ * TRUSTEES, DIRECTORS, OFFICERS, EMPLOYEES, AND AFFILIATES BE LIABLE FOR ANY DAMAGES
+ * OF ANY KIND, INCLUDING, WITHOUT LIMITATION, INCIDENTAL OR CONSEQUENTIAL DAMAGES,
+ * ECONOMIC DAMAGES OR INJURY TO PROPERTY AND LOST PROFITS, REGARDLESS OF WHETHER
+ * THE BROAD OR MIT SHALL BE ADVISED, SHALL HAVE OTHER REASON TO KNOW, OR IN FACT
+ * SHALL KNOW OF THE POSSIBILITY OF THE FOREGOING.
  */
 
 
@@ -21,8 +21,9 @@ package org.broad.igv.feature;
 
 //~--- non-JDK imports --------------------------------------------------------
 
-import org.broad.igv.session.ViewContext;
+import org.broad.igv.feature.genome.GenomeManager;
 import org.broad.igv.track.WindowFunction;
+import org.broad.igv.ui.IGV;
 
 //~--- JDK imports ------------------------------------------------------------
 
@@ -78,14 +79,7 @@ public class Exon extends AbstractFeature {
         }
     }
 
-    /**
-     * Constructs ...
-     *
-     * @param chr
-     * @param start
-     * @param end
-     * @param strand
-     */
+
     public Exon(String chr, int start, int end, Strand strand) {
         super(chr, start, end, strand);
 
@@ -108,38 +102,30 @@ public class Exon extends AbstractFeature {
         }
     }
 
-    /**
-     * Method description
-     *
-     * @param codingStart
-     */
+    public boolean isUTR() {
+        return utr;
+    }
+
+
+    public boolean isUTR(int position) {
+        return utr || (position < codingStart || position > codingEnd);
+    }
+
     public void setCodingStart(int codingStart) {
         this.codingStart = Math.max(getStart(), codingStart);
     }
 
-    /**
-     * Method description
-     *
-     * @param codingEnd
-     */
+
     public void setCodingEnd(int codingEnd) {
         this.codingEnd = Math.min(getEnd(), codingEnd);
     }
 
-    /**
-     * Method description
-     *
-     * @param offset
-     */
+
     public void setReadingFrame(int offset) {
         this.readingFrame = offset;
     }
 
-    /**
-     * Method description
-     *
-     * @param phase
-     */
+
     public void setPhase(int phase) {
         if (getStrand() == Strand.POSITIVE) {
             readingFrame = phase;
@@ -150,48 +136,25 @@ public class Exon extends AbstractFeature {
     }
 
 
-    /**
-     * Method description
-     *
-     * @return
-     */
     public int getCdStart() {
         return codingStart;
     }
 
-    /**
-     * Method description
-     *
-     * @return
-     */
+
     public int getCdEnd() {
         return this.codingEnd;
     }
 
-    /**
-     * Method description
-     *
-     * @return
-     */
+
     public int getCodingLength() {
         return utr ? 0 : Math.max(0, codingEnd - codingStart);
     }
 
-    /**
-     * This is exposed for unit tests.
-     *
-     * @return
-     */
+
     public int getReadingShift() {
         return readingFrame;
     }
 
-
-    /**
-     * Method description
-     *
-     * @return
-     */
     public AminoAcidSequence getAminoAcidSequence() {
         if (aminoAcidSequence == null) {
             computeAminoAcidSequence();
@@ -199,17 +162,7 @@ public class Exon extends AbstractFeature {
         return aminoAcidSequence;
     }
 
-    /**
-     * Method description
-     *
-     * @param aminoAcidSequence
-     */
-    public void setAminoAcidSequence(AminoAcidSequence aminoAcidSequence) {
-        this.aminoAcidSequence = aminoAcidSequence;
-    }
-
     private void computeAminoAcidSequence() {
-
         if (utr) {
             return;
         }
@@ -220,12 +173,64 @@ public class Exon extends AbstractFeature {
             int readStart = (codingStart > start) ? codingStart : start + readingFrame;
             int readEnd = Math.min(end, codingEnd);
             if (readEnd > readStart + 3) {
-                String genome = ViewContext.getInstance().getGenomeId();
-                aminoAcidSequence = AminoAcidManager.getAminoAcidSequence(genome, chr, readStart, readEnd, getStrand());
+                String genome = IGV.getInstance().getGenomeManager().getGenomeId();
+                byte[] seqBytes = SequenceManager.readSequence(genome, chr, readStart, readEnd);
+                aminoAcidSequence = AminoAcidManager.getAminoAcidSequence(seqBytes, readStart, getStrand());
             }
         }
     }
 
+    public byte[] getCodon(int position) {
+        int readStart = (codingStart > start) ? codingStart : start + readingFrame;
+        if (position >= readStart) {
+            int codonNumber = (position - readStart) / 3;
+            int codonStart = readStart + codonNumber * 3;
+            int codonEnd = Math.min(codonStart + 3, codingEnd);
+            if (codonEnd - codonStart == 3) {
+                String genome = IGV.getInstance().getGenomeManager().getGenomeId();
+                return SequenceManager.readSequence(genome, getChr(), codonStart, codonEnd);
+            }
+        }
+        return null;
+    }
+
+    public byte[] getCodon(int position, byte altBase) {
+        int readStart = (codingStart > start) ? codingStart : start + readingFrame;
+        if (position >= readStart) {
+            int codonNumber = (position - readStart) / 3;
+            int codonStart = readStart + codonNumber * 3;
+            int codonEnd = Math.min(codonStart + 3, codingEnd);
+            if (codonEnd - codonStart == 3) {
+                String genome = IGV.getInstance().getGenomeManager().getGenomeId();
+                byte [] refBytes = SequenceManager.readSequence(genome, getChr(), codonStart, codonEnd);
+                int delta = position - codonStart;
+                if(delta > 3) {
+                    System.out.println("Error getting alternate codon.  Position out of bounds: " + position);
+                    return null;
+                }
+                refBytes[delta] = altBase;
+            }
+        }
+        return null;
+    }
+
+    public AminoAcid getAminoAcid(int position) {
+        if (aminoAcidSequence == null) {
+            computeAminoAcidSequence();
+        }
+        if (aminoAcidSequence == null) {
+            return null;
+        }
+
+        int delta = position - aminoAcidSequence.getStartPosition();
+        if (delta < 0) {
+            System.out.println("Warning: negative amino acide offset");
+        }
+        int aaIndex = delta / 3;
+        return aaIndex >= aminoAcidSequence.getSequence().size() ? null : aminoAcidSequence.getSequence().get(aaIndex);
+
+    }
+
     public LocusScore copy() {
         Exon copy = new Exon(getChr(), getStart(), getEnd(), getStrand());
         copy.aminoAcidSequence = this.aminoAcidSequence;
@@ -240,13 +245,11 @@ public class Exon extends AbstractFeature {
         if (aaNumber > 0) {
             msg += "<br>Amino acid number: " + aaNumber;
         }
+        msg += "<br>" + getLocusString();
+        if (description != null) msg += description;
         return msg;
     }
 
-    public int getNumber() {
-        return number;
-    }
-
     public void setNumber(int number) {
         this.number = number;
     }
@@ -254,4 +257,5 @@ public class Exon extends AbstractFeature {
     public String getURL() {
         return null;  //To change body of implemented methods use File | Settings | File Templates.
     }
+
 }
