@@ -1,33 +1,34 @@
 /*
- * Copyright (c) 2007-2010 by The Broad Institute, Inc. and the Massachusetts Institute of Technology.
- * All Rights Reserved.
+ * Copyright (c) 2007-2011 by The Broad Institute of MIT and Harvard.  All Rights Reserved.
  *
- * This software is licensed under the terms of the GNU Lesser General Public License (LGPL), Version 2.1 which
- * is available at http://www.opensource.org/licenses/lgpl-2.1.php.
+ * This software is licensed under the terms of the GNU Lesser General Public License (LGPL),
+ * Version 2.1 which is available at http://www.opensource.org/licenses/lgpl-2.1.php.
  *
- * THE SOFTWARE IS PROVIDED "AS IS." THE BROAD AND MIT MAKE NO REPRESENTATIONS OR WARRANTIES OF
- * ANY KIND CONCERNING THE SOFTWARE, EXPRESS OR IMPLIED, INCLUDING, WITHOUT LIMITATION, WARRANTIES
- * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NONINFRINGEMENT, OR THE ABSENCE OF LATENT
- * OR OTHER DEFECTS, WHETHER OR NOT DISCOVERABLE.  IN NO EVENT SHALL THE BROAD OR MIT, OR THEIR
- * RESPECTIVE TRUSTEES, DIRECTORS, OFFICERS, EMPLOYEES, AND AFFILIATES BE LIABLE FOR ANY DAMAGES OF
- * ANY KIND, INCLUDING, WITHOUT LIMITATION, INCIDENTAL OR CONSEQUENTIAL DAMAGES, ECONOMIC
- * DAMAGES OR INJURY TO PROPERTY AND LOST PROFITS, REGARDLESS OF WHETHER THE BROAD OR MIT SHALL
- * BE ADVISED, SHALL HAVE OTHER REASON TO KNOW, OR IN FACT SHALL KNOW OF THE POSSIBILITY OF THE
- * FOREGOING.
+ * THE SOFTWARE IS PROVIDED "AS IS." THE BROAD AND MIT MAKE NO REPRESENTATIONS OR
+ * WARRANTES OF ANY KIND CONCERNING THE SOFTWARE, EXPRESS OR IMPLIED, INCLUDING,
+ * WITHOUT LIMITATION, WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
+ * PURPOSE, NONINFRINGEMENT, OR THE ABSENCE OF LATENT OR OTHER DEFECTS, WHETHER
+ * OR NOT DISCOVERABLE.  IN NO EVENT SHALL THE BROAD OR MIT, OR THEIR RESPECTIVE
+ * TRUSTEES, DIRECTORS, OFFICERS, EMPLOYEES, AND AFFILIATES BE LIABLE FOR ANY DAMAGES
+ * OF ANY KIND, INCLUDING, WITHOUT LIMITATION, INCIDENTAL OR CONSEQUENTIAL DAMAGES,
+ * ECONOMIC DAMAGES OR INJURY TO PROPERTY AND LOST PROFITS, REGARDLESS OF WHETHER
+ * THE BROAD OR MIT SHALL BE ADVISED, SHALL HAVE OTHER REASON TO KNOW, OR IN FACT
+ * SHALL KNOW OF THE POSSIBILITY OF THE FOREGOING.
  */
 
 package org.broad.igv.feature;
 
 import org.apache.log4j.Logger;
-import org.broad.igv.exceptions.DataLoadException;
 import org.broad.igv.exceptions.ParserException;
-import org.broad.igv.renderer.BasicFeatureRenderer;
+import org.broad.igv.feature.genome.Genome;
+import org.broad.igv.renderer.IGVFeatureRenderer;
 import org.broad.igv.renderer.GeneTrackRenderer;
-import org.broad.igv.session.ViewContext;
 import org.broad.igv.track.FeatureCollectionSource;
 import org.broad.igv.track.FeatureTrack;
 import org.broad.igv.track.TrackProperties;
+import org.broad.igv.ui.IGV;
 import org.broad.igv.util.*;
+import org.broad.tribble.readers.AsciiLineReader;
 
 import java.io.*;
 import java.net.URLDecoder;
@@ -47,24 +48,27 @@ public class GFFParser implements FeatureParser {
     static Logger log = Logger.getLogger(GFFParser.class);
 
     static HashSet exonTerms = new HashSet();
-    //static Set<String> ignoreAttributes = new HashSet();
+    static HashSet utrTerms = new HashSet();
     static HashSet<String> geneParts = new HashSet();
     static HashSet<String> ignoredTypes = new HashSet();
 
-    static String[] nameFields = {"gene", "Name", "name", "primary_name", "Locus", "locus", "alias", "systematic_id", "ID"};
+    //static String[] nameFields = {"gene", "Name", "name", "primary_name", "Locus", "locus","Alias", "alias", "systematic_id", "ID"};
 
 
-    static String[] tokens = new String[200];
 
     static {
-        exonTerms.add("five_prime_UTR");
-        exonTerms.add("three_prime_UTR");
-        exonTerms.add("5'-utr");
-        exonTerms.add("3'-utr");
-        exonTerms.add("3'-UTR");
-        exonTerms.add("5'-UTR");
-        exonTerms.add("5utr");
-        exonTerms.add("3utr");
+        utrTerms.add("five_prime_UTR");
+        utrTerms.add("three_prime_UTR");
+        utrTerms.add("5'-utr");
+        utrTerms.add("3'-utr");
+        utrTerms.add("3'-UTR");
+        utrTerms.add("5'-UTR");
+        utrTerms.add("5utr");
+        utrTerms.add("3utr");
+    }
+
+    static {
+        exonTerms.addAll(utrTerms);
         exonTerms.add("exon");
         exonTerms.add("coding_exon");
         exonTerms.add("CDS");
@@ -113,7 +117,7 @@ public class GFFParser implements FeatureParser {
     Map<String, BasicFeature> geneCache = new HashMap(50000);
     private TrackProperties trackProperties = null;
 
-    static StringBuffer buf = new StringBuffer(1000);
+    static StringBuffer buf = new StringBuffer();
 
 
     public GFFParser(String path) {
@@ -133,7 +137,7 @@ public class GFFParser implements FeatureParser {
     }
 
 
-    public List<FeatureTrack> loadTracks(ResourceLocator locator) {
+    public List<FeatureTrack> loadTracks(ResourceLocator locator, Genome genome) {
 
         if (locator.getPath().toLowerCase().endsWith("gff3") || locator.getPath().toLowerCase().endsWith("gff3.gz")) {
             helper = new GFF3Helper();
@@ -143,21 +147,17 @@ public class GFFParser implements FeatureParser {
         try {
             reader = ParsingUtils.openAsciiReader(locator);
 
-            List<Feature> features = loadFeatures(reader);
-            if (features.size() == 0) {
-                throw new DataLoadException("No features were found in this file with chromosomes mapped to the current genome", locator.getPath());
-            }
+            List<org.broad.tribble.Feature> features = loadFeatures(reader);
 
-
-            FeatureTrack track = new FeatureTrack(locator, new FeatureCollectionSource(features));
+            FeatureTrack track = new FeatureTrack(locator, new FeatureCollectionSource(features, genome));
             track.setName(locator.getTrackName());
-            track.setRendererClass(BasicFeatureRenderer.class);
+            track.setRendererClass(IGVFeatureRenderer.class);
             track.setMinimumHeight(35);
             track.setHeight(45);
             track.setRendererClass(GeneTrackRenderer.class);
 
             if (trackProperties != null) {
-                track.setTrackProperties(trackProperties);
+                track.setProperties(trackProperties);
             }
 
             List<FeatureTrack> tracks = new ArrayList();
@@ -184,14 +184,15 @@ public class GFFParser implements FeatureParser {
      */
 
 
-    public List<Feature> loadFeatures(AsciiLineReader reader) {
-        List<Feature> features = new ArrayList();
-        URLDecoder d = new URLDecoder();
+    public List<org.broad.tribble.Feature> loadFeatures(AsciiLineReader reader) {
+        List<org.broad.tribble.Feature> features = new ArrayList();
         String line = null;
         try {
 
-            Genome genome = ViewContext.getInstance().getGenome();
+            Genome genome = IGV.getInstance().getGenomeManager().getCurrentGenome();
             Set<String> featuresToHide = new HashSet();
+            String[] tokens = new String[200];
+
             while ((line = reader.readLine()) != null) {
 
                 if (line.startsWith("##gff-version") && line.endsWith("3")) {
@@ -200,7 +201,7 @@ public class GFFParser implements FeatureParser {
 
 
                 if (line.startsWith("#")) {
-                    if (line.startsWith("#track")) {
+                    if (line.startsWith("#track") || line.startsWith("##track")) {
                         trackProperties = new TrackProperties();
                         ParsingUtils.parseTrackLine(line, trackProperties);
                     } else if (line.startsWith("#nodecode")) {
@@ -210,6 +211,18 @@ public class GFFParser implements FeatureParser {
                         if (kv.length > 1) {
                             featuresToHide.addAll(Arrays.asList(kv[1].split(",")));
                         }
+                    } else if (line.startsWith("#displayName")) {
+                        String [] nameTokens = line.split("=");
+                        if(nameTokens.length < 2) {
+                            helper.setNameFields(null);
+                        }
+                        else {
+                            String [] fields = nameTokens[1].split(",");
+                            helper.setNameFields(fields);
+                        }
+
+                    } else if (line.startsWith("#colorBy")) {
+
                     }
                     continue;
                 }
@@ -219,6 +232,11 @@ public class GFFParser implements FeatureParser {
 
                 // GFF files have 9 tokens
                 if (nTokens < 9) {
+                    // Maybe its a track line?
+                    if (line.startsWith("track")) {
+                        trackProperties = new TrackProperties();
+                        ParsingUtils.parseTrackLine(line, trackProperties);
+                    }
                     continue;
                 }
 
@@ -230,7 +248,7 @@ public class GFFParser implements FeatureParser {
                 }
 
 
-                String chromosome = genome.getChromosomeAlias(tokens[0]);
+                String chromosome = genome == null ? tokens[0] : genome.getChromosomeAlias(tokens[0]);
 
                 // GFF coordinates are 1-based inclusive (length = end - start + 1)
                 // IGV (UCSC) coordinates are 0-based exclusive.  Adjust start and end accordingly
@@ -274,14 +292,14 @@ public class GFFParser implements FeatureParser {
                     for (String pid : parentIds) {
                         getGFF3Transcript(pid).addCDSParts(chromosome, start, end, description);
                     }
-                } else if (exonTerms.contains(featureType) && parentIds != null && parentIds.length > 0) {
-
-                    Exon exon = new Exon(chromosome, start, end, strand);
-                    exon.setDescription(description);
+                } else if (exonTerms.contains(featureType) && parentIds != null && parentIds.length > 0 &&
+                        parentIds[0] != null && parentIds[0].length() > 0 && !parentIds[0].equals(".")) {
+                    String name = getName(attributes);
+                    int phase = -1;
                     String phaseString = tokens[7].trim();
                     if (!phaseString.equals(".")) {
                         try {
-                            exon.setPhase(Integer.parseInt(phaseString));
+                            phase = Integer.parseInt(phaseString);
 
                         } catch (NumberFormatException numberFormatException) {
 
@@ -290,10 +308,19 @@ public class GFFParser implements FeatureParser {
                         }
                     }
 
-                    exon.setName(getName(attributes));
-
-
+                    // Make a copy of the exon record for each parent
                     for (String pid : parentIds) {
+
+                        Exon exon = new Exon(chromosome, start, end, strand);
+                        exon.setDescription(description);
+                        exon.setUTR(utrTerms.contains(featureType));
+
+                        if (phase >= 0) {
+                            exon.setPhase(phase);
+
+                        }
+                        exon.setName(name);
+
                         if (featureType.equals("exon")) {
                             getGFF3Transcript(pid).addExon(exon);
                         } else if (featureType.equals("CDS")) {
@@ -305,16 +332,17 @@ public class GFFParser implements FeatureParser {
                         }
                     }
 
+
                 } else {
                     BasicFeature f = new BasicFeature(chromosome, start, end, strand);
                     f.setName(getName(attributes));
                     f.setDescription(description);
 
                     if (attributes.containsKey("color")) {
-                        f.setColor(ColorUtilities.getColorFromString(attributes.get("color")));
+                        f.setColor(ColorUtilities.stringToColor(attributes.get("color")));
                     }
                     if (attributes.containsKey("Color")) {
-                        f.setColor(ColorUtilities.getColorFromString(attributes.get("Color")));
+                        f.setColor(ColorUtilities.stringToColor(attributes.get("Color")));
                     }
 
 
@@ -347,17 +375,16 @@ public class GFFParser implements FeatureParser {
 
             // Create and add IGV genes
             for (GFF3Transcript transcript : transcriptCache.values()) {
-                BasicFeature igvGene = transcript.createIGVGene();
-                if (igvGene != null) {
-                    features.add(igvGene);
+                BasicFeature igvTranscript = transcript.createTranscript();
+                if (igvTranscript != null) {
+                    features.add(igvTranscript);
                 }
             }
 
             FeatureDB.addFeatures(features);
 
         }
-        catch (ParserException
-                e) {
+        catch (ParserException e) {
             throw e;
         }
         catch (Exception
@@ -374,11 +401,22 @@ public class GFFParser implements FeatureParser {
         return features;
     }
 
+    static String getDescription(Map<String, String> attributes) {
+        return getDescription(attributes, null);
+    }
+
 
     static String getDescription(Map<String, String> attributes, String type) {
+        // 30 attributes is the maximum visible on a typical screen
+        int max = 30;
+        int count = 0;
+
+        // Reset buffer.
         buf.setLength(0);
-        buf.append(type);
-        buf.append("<br>");
+        if (type != null) {
+            buf.append(type);
+            buf.append("<br>");
+        }
         for (Map.Entry<String, String> att : attributes.entrySet()) {
 
             //if (!ignoreAttributes.contains(att.getKey())) {
@@ -387,6 +425,11 @@ public class GFFParser implements FeatureParser {
             buf.append(" = ");
             buf.append(attValue);
             buf.append("<br>");
+            if (++count > max) {
+                buf.append("...");
+                break;
+            }
+
             //}
         }
         String description = buf.toString();
@@ -421,13 +464,7 @@ public class GFFParser implements FeatureParser {
             return null;
         }
 
-        for (String nf : nameFields) {
-            if (attributes.containsKey(nf)) {
-                return attributes.get(nf);
-            }
-        }
-
-        return attributes.values().iterator().next();
+        return helper.getName(attributes);
 
     }
 
@@ -436,8 +473,11 @@ public class GFFParser implements FeatureParser {
 
         BufferedReader br = new BufferedReader(new FileReader(gffFile));
         String nextLine;
+        String ext = gffFile.substring(gffFile.length() - 4);
 
         Map<String, PrintWriter> writers = new HashMap();
+        String [] tokens = new String[20];
+        
         while ((nextLine = br.readLine()) != null) {
             nextLine = nextLine.trim();
             if (!nextLine.startsWith("#")) {
@@ -451,7 +491,7 @@ public class GFFParser implements FeatureParser {
                 if (!writers.containsKey(type)) {
                     writers.put(type,
                             new PrintWriter(new FileWriter(new File(outputDirectory,
-                                    type + ".gff"))));
+                                    type + ext))));
                 }
             }
         }
@@ -500,7 +540,6 @@ public class GFFParser implements FeatureParser {
         private String id;
         private Set<Exon> exons = new HashSet();
         private List<Exon> cdss = new ArrayList();
-        private List<Exon> cdsParts = null;
         private Exon fivePrimeUTR;
         private Exon threePrimeUTR;
         private BasicFeature transcript;
@@ -547,6 +586,8 @@ public class GFFParser implements FeatureParser {
 
         void addCDS(Exon cds) {
             cdss.add(cds);
+            this.start = Math.min(cds.getStart(), start);
+            this.end = Math.max(cds.getEnd(), end);
         }
 
         void addCDSParts(String chr, int start, int end, String desc) {
@@ -566,41 +607,38 @@ public class GFFParser implements FeatureParser {
         }
 
         /**
-         * Create an IGV "gene (transcript)" from a GFF mRNA (transcript). "
+         * Create a transcript from its constituitive parts. "
          *
          * @return
          */
-        BasicFeature createIGVGene() {
+        BasicFeature createTranscript() {
 
             Strand strand = Strand.NONE;
             String name = null;
 
-            // Combine exon & cds  if feature contains both CDS and CDS_parts use CDS_parts
-            List<Exon> cdsList = cdsParts;
-            if (cdsList == null) {
-                cdsList = cdss;
-            }
-            if (exons.size() > 0) {
-                while (!cdsList.isEmpty()) {
-                    Exon cds = cdsList.get(0);
-                    Exon exon = findMatchingExon(cds);
-                    if (exon == null) {
-                        exons.add(cds);
-                    } else {
-                        exon.setCodingStart(cds.getStart());
-                        exon.setCodingEnd(cds.getEnd());
-                        exon.setReadingFrame(cds.getReadingShift());
-                    }
-                    cdsList.remove(0);
-                }
-                for (Exon exon : exons) {
-                    chr = exon.getChr();
-                    strand = exon.getStrand();
-                    start = Math.min(exon.getStart(), start);
-                    end = Math.max(exon.getEnd(), end);
-                    name = exon.getName();
+            // Combine CDS and exons
+            while (!cdss.isEmpty()) {
+                Exon cds = cdss.get(0);
+                Exon exon = findMatchingExon(cds);
+                if (exon == null) {
+                    cds.setCodingStart(cds.getStart());
+                    cds.setCodingEnd(cds.getEnd());
+                    exons.add(cds);
+                } else {
+                    exon.setCodingStart(cds.getStart());
+                    exon.setCodingEnd(cds.getEnd());
+                    exon.setReadingFrame(cds.getReadingShift());
                 }
+                cdss.remove(0);
             }
+            for (Exon exon : exons) {
+                chr = exon.getChr();
+                strand = exon.getStrand();
+                start = Math.min(exon.getStart(), start);
+                end = Math.max(exon.getEnd(), end);
+                name = exon.getName();
+            }
+
 
             if (transcript == null) {
                 // transcript is implied
@@ -612,6 +650,7 @@ public class GFFParser implements FeatureParser {
 
             if ((parentId != null) && geneCache.containsKey(parentId)) {
                 BasicFeature gene = geneCache.get(parentId);
+                geneCache.remove(parentId);
                 if (transcript.getName() == null && gene.getName() != null) {
                     transcript.setName(gene.getName());
                 }
@@ -659,7 +698,7 @@ public class GFFParser implements FeatureParser {
             return transcript;
         }
 
-        Exon findMatchingExon(Feature cds) {
+        Exon findMatchingExon(IGVFeature cds) {
             for (Exon exon : exons) {
                 if (exon.contains(cds)) {
                     return exon;
@@ -679,12 +718,28 @@ public class GFFParser implements FeatureParser {
 
         void setUrlDecoding(boolean b);
 
+        String getName(Map<String, String> attributes);
+
+        void setNameFields(String[] fields);
     }
 
-    class GFF2Helper implements Helper {
+    public static class GFF2Helper implements Helper {
 
-        String[] idFields = {"systematic_id", "ID", "Name", "name", "primary_name", "gene", "Locus", "locus", "alias"};
+        static String[] idFields = {"systematic_id", "ID", "transcript_id", "Name", "name", "primary_name", "gene", "Locus", "locus", "alias"};
+        static String[] DEFAULT_NAME_FIELDS = {"gene", "Name", "name", "primary_name", "Locus", "locus", "Alias", "alias", "systematic_id", "ID"};
 
+        private String[] nameFields;
+
+        GFF2Helper() {
+            this(DEFAULT_NAME_FIELDS);
+        }
+
+        GFF2Helper(String[] nameFields) {
+            if (nameFields != null) {
+                this.nameFields = nameFields;
+            }
+
+        }
 
         public void setUrlDecoding(boolean b) {
             // Ignored,  GFF files are never url DECODED
@@ -745,13 +800,43 @@ public class GFFParser implements FeatureParser {
             }
             return getName(attributes);
         }
+
+        public String getName(Map<String, String> attributes) {
+
+            if (attributes.size() > 0 && nameFields != null) {
+                for (String nf : nameFields) {
+                    if (attributes.containsKey(nf)) {
+                        return attributes.get(nf);
+                    }
+                }
+            }
+
+            return null;
+        }
+
+        public void setNameFields(String[] nameFields) {
+            this.nameFields = nameFields;
+        }
     }
 
-    class GFF3Helper implements Helper {
-
+    public static class GFF3Helper implements Helper {
 
+        static String[] DEFAULT_NAME_FIELDS = {"Name", "Alias", "ID", "Gene", "gene", "Locus", "locus"};
         private boolean useUrlDecoding = true;
 
+        private String[] nameFields;
+
+        GFF3Helper() {
+            this(DEFAULT_NAME_FIELDS);
+        }
+
+        GFF3Helper(String[] nameFields) {
+            if (nameFields != null) {
+                this.nameFields = nameFields;
+            }
+
+        }
+
 
         public String[] getParentIds(Map<String, String> attributes, String ignored) {
             String parentIdString = attributes.get("Parent");
@@ -777,6 +862,10 @@ public class GFFParser implements FeatureParser {
                     if (useUrlDecoding) {
                         key = URLDecoder.decode(key);
                         value = URLDecoder.decode(value);
+                        // Limit values to 50 characters
+                        if(value.length() > 50) {
+                            value = value.substring(0, 50) + " ...";
+                        }
                     }
                     kvalues.put(key, value);
                 } else {
@@ -789,11 +878,31 @@ public class GFFParser implements FeatureParser {
             this.useUrlDecoding = useUrlDecoding;
         }
 
+        public String getName(Map<String, String> attributes) {
+
+            if (attributes.size() > 0 && nameFields != null) {
+                for (String nf : nameFields) {
+                    if (attributes.containsKey(nf)) {
+                        return attributes.get(nf);
+                    }
+                }
+            }
+
+            return null;
+        }
 
         public String getID(Map<String, String> attributes) {
             String id = attributes.get("ID");
             return id;
         }
+
+        public String[] getNameFields() {
+            return nameFields;
+        }
+
+        public void setNameFields(String[] nameFields) {
+            this.nameFields = nameFields;
+        }
     }
 
 
