@@ -1,27 +1,27 @@
 /*
- * Copyright (c) 2007-2010 by The Broad Institute, Inc. and the Massachusetts Institute of Technology.
- * All Rights Reserved.
+ * Copyright (c) 2007-2011 by The Broad Institute of MIT and Harvard.  All Rights Reserved.
  *
- * This software is licensed under the terms of the GNU Lesser General Public License (LGPL), Version 2.1 which
- * is available at http://www.opensource.org/licenses/lgpl-2.1.php.
+ * This software is licensed under the terms of the GNU Lesser General Public License (LGPL),
+ * Version 2.1 which is available at http://www.opensource.org/licenses/lgpl-2.1.php.
  *
- * THE SOFTWARE IS PROVIDED "AS IS." THE BROAD AND MIT MAKE NO REPRESENTATIONS OR WARRANTIES OF
- * ANY KIND CONCERNING THE SOFTWARE, EXPRESS OR IMPLIED, INCLUDING, WITHOUT LIMITATION, WARRANTIES
- * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NONINFRINGEMENT, OR THE ABSENCE OF LATENT
- * OR OTHER DEFECTS, WHETHER OR NOT DISCOVERABLE.  IN NO EVENT SHALL THE BROAD OR MIT, OR THEIR
- * RESPECTIVE TRUSTEES, DIRECTORS, OFFICERS, EMPLOYEES, AND AFFILIATES BE LIABLE FOR ANY DAMAGES OF
- * ANY KIND, INCLUDING, WITHOUT LIMITATION, INCIDENTAL OR CONSEQUENTIAL DAMAGES, ECONOMIC
- * DAMAGES OR INJURY TO PROPERTY AND LOST PROFITS, REGARDLESS OF WHETHER THE BROAD OR MIT SHALL
- * BE ADVISED, SHALL HAVE OTHER REASON TO KNOW, OR IN FACT SHALL KNOW OF THE POSSIBILITY OF THE
- * FOREGOING.
+ * THE SOFTWARE IS PROVIDED "AS IS." THE BROAD AND MIT MAKE NO REPRESENTATIONS OR
+ * WARRANTES OF ANY KIND CONCERNING THE SOFTWARE, EXPRESS OR IMPLIED, INCLUDING,
+ * WITHOUT LIMITATION, WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
+ * PURPOSE, NONINFRINGEMENT, OR THE ABSENCE OF LATENT OR OTHER DEFECTS, WHETHER
+ * OR NOT DISCOVERABLE.  IN NO EVENT SHALL THE BROAD OR MIT, OR THEIR RESPECTIVE
+ * TRUSTEES, DIRECTORS, OFFICERS, EMPLOYEES, AND AFFILIATES BE LIABLE FOR ANY DAMAGES
+ * OF ANY KIND, INCLUDING, WITHOUT LIMITATION, INCIDENTAL OR CONSEQUENTIAL DAMAGES,
+ * ECONOMIC DAMAGES OR INJURY TO PROPERTY AND LOST PROFITS, REGARDLESS OF WHETHER
+ * THE BROAD OR MIT SHALL BE ADVISED, SHALL HAVE OTHER REASON TO KNOW, OR IN FACT
+ * SHALL KNOW OF THE POSSIBILITY OF THE FOREGOING.
  */
 package org.broad.igv.sam;
 
 import org.broad.igv.PreferenceManager;
 import org.broad.igv.feature.SequenceManager;
 import org.broad.igv.feature.Strand;
+import org.broad.igv.renderer.ContinuousColorScale;
 import org.broad.igv.renderer.GraphicUtils;
-import org.broad.igv.session.ViewContext;
 import org.broad.igv.track.RenderContext;
 import org.broad.igv.ui.FontManager;
 import org.broad.igv.util.ChromosomeColors;
@@ -37,10 +37,12 @@ import java.util.Map;
  */
 public class AlignmentRenderer implements FeatureRenderer {
 
-    public static Map<Character, Color> nucleotideColors;
+    private static Map<Character, Color> nucleotideColors;
     // Static because all alignment tracks are in color space, or none are
     public static boolean colorSpace;
 
+    private static Color smallISizeColor = new Color(0, 0, 150);
+    private static Color largeISizeColor = new Color(150, 0, 0);
     private static Color purple = new Color(118, 24, 220);
     private static Color deletionColor = Color.black;
     private static Color skippedColor = new Color(150, 184, 200);
@@ -51,19 +53,24 @@ public class AlignmentRenderer implements FeatureRenderer {
     private static Color grey2 = new Color(165, 165, 165);
     public static Color grey1 = new Color(200, 200, 200);
 
-
-    static Font font = FontManager.getScalableFont(10);
-    private ViewContext viewContext;
     private static Stroke thickStroke = new BasicStroke(2.0f);
-    private static float dash[] = {4.0f, 1.0f};
-    private static Stroke dashedStroke = new BasicStroke(2.0f, BasicStroke.CAP_BUTT,
-            BasicStroke.JOIN_MITER, 1.0f, dash, 0.0f);
 
     private static final Color negStrandColor = new Color(110, 145, 225);
     private static final Color posStrandColor = new Color(165, 35, 39);
-    PreferenceManager.SAMPreferences prefs;
 
-    synchronized static void initColorMap() {
+    private static HashMap<String, Color> readGroupColors = new HashMap();
+
+    private static final Color LR_COLOR = grey1; // "Normal" alignment color
+    private static final Color RL_COLOR = new Color(0, 150, 0);
+    private static final Color RR_COLOR = new Color(0, 0, 150);
+    private static final Color LL_COLOR = new Color(0, 150, 150);
+    static Map<String, Color> frOrientationColors;
+    static Map<String, Color> ffOrientationColors;
+    static Map<String, Color> rfOrientationColors;
+
+    PreferenceManager prefs;
+
+    static {
         nucleotideColors = new HashMap();
         nucleotideColors.put('A', Color.GREEN);
         nucleotideColors.put('a', Color.GREEN);
@@ -76,6 +83,62 @@ public class AlignmentRenderer implements FeatureRenderer {
         nucleotideColors.put('N', Color.gray.brighter());
         nucleotideColors.put('n', Color.gray.brighter());
 
+
+        // fr Orienations (e.g. Illumina paired-end libraries)
+        frOrientationColors = new HashMap();
+        //LR
+        frOrientationColors.put("F1R2", LR_COLOR);
+        frOrientationColors.put("F2R1", LR_COLOR);
+        frOrientationColors.put("F R ", LR_COLOR);
+        //LL
+        frOrientationColors.put("F1F2", LL_COLOR);
+        frOrientationColors.put("F2F1", LL_COLOR);
+        frOrientationColors.put("F F ", LL_COLOR);
+        frOrientationColors.put("FF", LL_COLOR);
+        //RR
+        frOrientationColors.put("R1R2", RR_COLOR);
+        frOrientationColors.put("R2R1", RR_COLOR);
+        frOrientationColors.put("R R ", RR_COLOR);
+        frOrientationColors.put("RR", RR_COLOR);
+        //RL
+        frOrientationColors.put("R1F2", RL_COLOR);
+        frOrientationColors.put("R2F1", RL_COLOR);
+        frOrientationColors.put("R F ", RL_COLOR);
+
+        // rf orienation  (e.g. Illumina mate-pair libraries)
+        rfOrientationColors = new HashMap();
+        //LR
+        rfOrientationColors.put("R1F2", LR_COLOR);
+        rfOrientationColors.put("R2F1", LR_COLOR);
+        rfOrientationColors.put("R F ", LR_COLOR);
+        //LL
+        rfOrientationColors.put("R1R2", LL_COLOR);
+        rfOrientationColors.put("R2R1", LL_COLOR);
+        rfOrientationColors.put("R r ", LL_COLOR);
+        //RR
+        rfOrientationColors.put("F1F2", RR_COLOR);
+        rfOrientationColors.put("F2F1", RR_COLOR);
+        rfOrientationColors.put("F F ", RR_COLOR);
+        //RL
+        rfOrientationColors.put("F1R2", RL_COLOR);
+        rfOrientationColors.put("F2R1", RL_COLOR);
+        rfOrientationColors.put("F R ", RL_COLOR);
+
+
+        // ff orienation  (e.g. SOLID libraries)
+        ffOrientationColors = new HashMap();
+        //LR
+        ffOrientationColors.put("F1F2", LR_COLOR);
+        ffOrientationColors.put("R2R1", LR_COLOR);
+        //LL -- switched with RR color per Bob's instructions
+        ffOrientationColors.put("F1R2", RR_COLOR);
+        ffOrientationColors.put("R2F1", RR_COLOR);
+        //RR
+        ffOrientationColors.put("R1F2", LL_COLOR);
+        ffOrientationColors.put("F2R1", LL_COLOR);
+        //RL
+        ffOrientationColors.put("R1R2", RL_COLOR);
+        ffOrientationColors.put("F2F1", RL_COLOR);
     }
 
 
@@ -83,11 +146,7 @@ public class AlignmentRenderer implements FeatureRenderer {
      * Constructs ...
      */
     public AlignmentRenderer() {
-        this.viewContext = ViewContext.getInstance();
-        this.prefs = PreferenceManager.getInstance().getSAMPreferences();
-        if (nucleotideColors == null) {
-            initColorMap();
-        }
+        this.prefs = PreferenceManager.getInstance();
 
     }
 
@@ -104,6 +163,7 @@ public class AlignmentRenderer implements FeatureRenderer {
         double origin = context.getOrigin();
         double locScale = context.getScale();
         Rectangle screenRect = context.getVisibleRect();
+        Font font = FontManager.getFont(10);
 
         if ((alignments != null) && (alignments.size() > 0)) {
 
@@ -117,40 +177,48 @@ public class AlignmentRenderer implements FeatureRenderer {
                 double pixelEnd = ((alignment.getEnd() - origin) / locScale);
 
                 // If the any part of the feature fits in the track rectangle draw  it
+                if (pixelEnd < rect.x) {
+                    continue;
+                } else if (pixelStart > rect.getMaxX()) {
+                    break;
+                }
 
 
-                if ((pixelEnd >= rect.getX()) && (pixelStart <= rect.getMaxX())) {
-                    Color alignmentColor = getAlignmentColor(alignment, locScale, viewContext.getCenter(), renderOptions);
-
+                // If the alignment is 3 pixels or less,  draw alignment as posA single block,
+                // further detail would not be seen and just add to drawing overhead
+                if (pixelEnd - pixelStart < 4) {
+                    Color alignmentColor = getAlignmentColor(alignment, locScale, context.getReferenceFrame().getCenter(), renderOptions);
                     Graphics2D g = context.getGraphic2DForColor(alignmentColor);
                     g.setFont(font);
 
-                    // If the alignment is 3 pixels or less,  draw alignment as posA single block,
-                    // further detail would not be seen and just add to drawing overhead
-                    if (pixelEnd - pixelStart < 4) {
-                        int w = Math.max(1, (int) (pixelEnd - pixelStart));
-                        int h = (int) Math.max(1, rect.getHeight() - 2);
-                        int y = (int) (rect.getY() + (rect.getHeight() - h) / 2);
-                        g.fillRect((int) pixelStart, y, w, h);
+                    int w = Math.max(1, (int) (pixelEnd - pixelStart));
+                    int h = (int) Math.max(1, rect.getHeight() - 2);
+                    int y = (int) (rect.getY() + (rect.getHeight() - h) / 2);
+                    g.fillRect((int) pixelStart, y, w, h);
+                } else {
+                    if (alignment instanceof PairedAlignment) {
+                        drawPairedAlignment((PairedAlignment) alignment, rect, context, renderOptions, leaveMargin, selectedReadNames, font);
                     } else {
+                        Color alignmentColor = getAlignmentColor(alignment, locScale, context.getReferenceFrame().getCenter(), renderOptions);
+                        Graphics2D g = context.getGraphic2DForColor(alignmentColor);
+                        g.setFont(font);
                         drawAlignment(alignment, rect, g, context, alignmentColor, renderOptions, leaveMargin, selectedReadNames);
                     }
-
                 }
             }
 
             // Draw posA border around the center base
             if (locScale < 5 && renderOptions.shadeCenters) {
                 // Calculate center lines
-                double center = (int) (viewContext.getCenter() - origin);
+                double center = (int) (context.getReferenceFrame().getCenter() - origin);
                 int centerLeftP = (int) (center / locScale);
                 int centerRightP = (int) ((center + 1) / locScale);
-                float transparency = Math.max(0.5f, (float) Math.round(10 * (1 - .75 * locScale)) / 10);
-                Graphics2D gBlack = context.getGraphic2DForColor(new Color(0, 0, 0, transparency));
-                GraphicUtils.drawDashedLine(gBlack, centerLeftP, rect.y, centerLeftP,
+                //float transparency = Math.max(0.5f, (float) Math.round(10 * (1 - .75 * locScale)) / 10);
+                Graphics2D gBlack = context.getGraphic2DForColor(Color.black); //new Color(0, 0, 0, transparency));
+                GraphicUtils.drawDottedDashLine(gBlack, centerLeftP, rect.y, centerLeftP,
                         rect.y + rect.height);
                 if ((centerRightP - centerLeftP > 2)) {
-                    GraphicUtils.drawDashedLine(gBlack, centerRightP, rect.y, centerRightP,
+                    GraphicUtils.drawDottedDashLine(gBlack, centerRightP, rect.y, centerRightP,
                             rect.y + rect.height);
                 }
             }
@@ -215,6 +283,44 @@ public class AlignmentRenderer implements FeatureRenderer {
         }
     }
 
+    private void drawPairedAlignment(
+            PairedAlignment pair,
+            Rectangle rect,
+            RenderContext context,
+            AlignmentTrack.RenderOptions renderOptions,
+            boolean leaveMargin,
+            Map<String, Color> selectedReadNames,
+            Font font) {
+
+        double locScale = context.getScale();
+        Color alignmentColor = getAlignmentColor(pair.firstAlignment, locScale, context.getReferenceFrame().getCenter(), renderOptions);
+        Graphics2D g = context.getGraphic2DForColor(alignmentColor);
+        g.setFont(font);
+        drawAlignment(pair.firstAlignment, rect, g, context, alignmentColor, renderOptions, leaveMargin, selectedReadNames);
+
+        if (pair.secondAlignment != null) {
+
+            alignmentColor = getAlignmentColor(pair.secondAlignment, locScale, context.getReferenceFrame().getCenter(), renderOptions);
+            g = context.getGraphic2DForColor(alignmentColor);
+
+            drawAlignment(pair.secondAlignment, rect, g, context, alignmentColor, renderOptions, leaveMargin, selectedReadNames);
+
+            Graphics2D gLine = context.getGraphic2DForColor(grey1);
+            double origin = context.getOrigin();
+            int startX = (int) ((pair.firstAlignment.getEnd() - origin) / locScale);
+            startX = Math.max(rect.x, startX);
+
+            int endX = (int) ((pair.secondAlignment.getStart() - origin) / locScale);
+            endX = Math.min(rect.x + rect.width, endX);
+
+            int h = (int) Math.max(1, rect.getHeight() - (leaveMargin ? 2 : 0));
+            int y = (int) (rect.getY()); // + (rect.getHeight() - h) / 2);
+            gLine.drawLine(startX, y + h / 2, endX, y + h / 2);
+
+        }
+
+    }
+
     /**
      * @param alignment
      * @param rect
@@ -239,7 +345,7 @@ public class AlignmentRenderer implements FeatureRenderer {
         double locScale = context.getScale();
         AlignmentBlock[] blocks = alignment.getAlignmentBlocks();
 
-        if (blocks == null) {
+        if (blocks == null || blocks.length == 0) {
             drawSimpleAlignment(alignment, rect, g, context, renderOptions.flagUnmappedPairs);
             return;
         }
@@ -261,6 +367,7 @@ public class AlignmentRenderer implements FeatureRenderer {
             int h = (int) Math.max(1, rect.getHeight() - (leaveMargin ? 2 : 0));
             int y = (int) (rect.getY()); // + (rect.getHeight() - h) / 2);
 
+
             // Create polygon to represent the alignment.
             boolean isZeroQuality = alignment.getMappingQuality() == 0;
 
@@ -269,86 +376,94 @@ public class AlignmentRenderer implements FeatureRenderer {
                 w -= 2;
             }
 
-
-            if (w <= 10 || h <= 10 || aBlock != terminalBlock) {
-                g.fillRect(x, y, w, h);
-                if (isZeroQuality) {
-                    greyGraphics.drawRect(x, y, w - 1, h);
-                }
-
-                if (renderOptions.flagUnmappedPairs && alignment.isPaired() && !alignment.getMate().isMapped()) {
-                    Graphics2D cRed = context.getGraphic2DForColor(Color.red);
-                    cRed.drawRect(x, y, w, h);
-                }
-
-                if (selectedReadNames.containsKey(alignment.getReadName())) {
-                    Color c = selectedReadNames.get(alignment.getReadName());
-                    if (c == null) {
-                        c = Color.blue;
+            // If block is out of view skip
+            if (x + w >= rect.x && x <= rect.getMaxX()) {
+                if (w <= 10 || h <= 10 || aBlock != terminalBlock) {
+                    g.fillRect(x, y, w, h);
+                    if (isZeroQuality) {
+                        greyGraphics.drawRect(x, y, w - 1, h);
                     }
-                    Graphics2D cBlue = context.getGraphic2DForColor(c);
-                    Stroke s = cBlue.getStroke();
-                    cBlue.setStroke(thickStroke);
-                    cBlue.drawRect(x, y, w, h);
-                    cBlue.setStroke(s);
-                }
 
-            } else {
-                int arrowLength = Math.min(5, w / 6);
-
-                // Don't draw off edge of clipping rect
-                if (x < rect.x && (x + w) > (rect.x + rect.width)) {
-                    x = rect.x;
-                    w = rect.width;
-                    arrowLength = 0;
-                } else if (x < rect.x) {
-                    int delta = rect.x - x;
-                    x = rect.x;
-                    w -= delta;
-                    if (alignment.isNegativeStrand()) {
-                        arrowLength = 0;
+                    if (renderOptions.flagUnmappedPairs && alignment.isPaired() && !alignment.getMate().isMapped()) {
+                        Graphics2D cRed = context.getGraphic2DForColor(Color.red);
+                        cRed.drawRect(x, y, w, h);
                     }
-                } else if ((x + w) > (rect.x + rect.width)) {
-                    w -= ((x + w) - (rect.x + rect.width));
-                    if (!alignment.isNegativeStrand()) {
-                        arrowLength = 0;
+
+                    if (selectedReadNames.containsKey(alignment.getReadName())) {
+                        Color c = selectedReadNames.get(alignment.getReadName());
+                        if (c == null) {
+                            c = Color.blue;
+                        }
+                        Graphics2D cBlue = context.getGraphic2DForColor(c);
+                        Stroke s = cBlue.getStroke();
+                        cBlue.setStroke(thickStroke);
+                        cBlue.drawRect(x, y, w, h);
+                        cBlue.setStroke(s);
                     }
-                }
 
-                int[] xPoly = null;
-                int[] yPoly = {y, y, y + h / 2, y + h, y + h};
-
-                if (alignment.isNegativeStrand()) {
-                    xPoly = new int[]{x + w, x, x - arrowLength, x, x + w};
                 } else {
-                    xPoly = new int[]{x, x + w, x + w + arrowLength, x + w, x};
-                }
-                g.fillPolygon(xPoly, yPoly, xPoly.length);
+                    int arrowLength = Math.min(5, w / 6);
 
+                    // Don't draw off edge of clipping rect
+                    if (x < rect.x && (x + w) > (rect.x + rect.width)) {
+                        x = rect.x;
+                        w = rect.width;
+                        arrowLength = 0;
+                    } else if (x < rect.x) {
+                        int delta = rect.x - x;
+                        x = rect.x;
+                        w -= delta;
+                        if (alignment.isNegativeStrand()) {
+                            arrowLength = 0;
+                        }
+                    } else if ((x + w) > (rect.x + rect.width)) {
+                        w -= ((x + w) - (rect.x + rect.width));
+                        if (!alignment.isNegativeStrand()) {
+                            arrowLength = 0;
+                        }
+                    }
 
-                if (isZeroQuality) {
-                    greyGraphics.drawPolygon(xPoly, yPoly, xPoly.length);
-                }
+                    int[] xPoly;
+                    int[] yPoly = {y, y, y + h / 2, y + h, y + h};
 
-                if (renderOptions.flagUnmappedPairs && alignment.isPaired() && !alignment.getMate().isMapped()) {
-                    Graphics2D cRed = context.getGraphic2DForColor(Color.red);
-                    cRed.drawPolygon(xPoly, yPoly, xPoly.length);
-                }
+                    if (alignment.isNegativeStrand()) {
+                        xPoly = new int[]{x + w, x, x - arrowLength, x, x + w};
+                    } else {
+                        xPoly = new int[]{x, x + w, x + w + arrowLength, x + w, x};
+                    }
+                    g.fillPolygon(xPoly, yPoly, xPoly.length);
 
-                if (selectedReadNames.containsKey(alignment.getReadName())) {
-                    Color c = selectedReadNames.get(alignment.getReadName());
-                    if (c == null) {
-                        c = Color.blue;
+
+                    if (isZeroQuality) {
+                        greyGraphics.drawPolygon(xPoly, yPoly, xPoly.length);
+                    }
+
+                    if (renderOptions.flagUnmappedPairs && alignment.isPaired() && !alignment.getMate().isMapped()) {
+                        Graphics2D cRed = context.getGraphic2DForColor(Color.red);
+                        cRed.drawPolygon(xPoly, yPoly, xPoly.length);
+                    }
+
+                    if (selectedReadNames.containsKey(alignment.getReadName())) {
+                        Color c = selectedReadNames.get(alignment.getReadName());
+                        if (c == null) {
+                            c = Color.blue;
+                        }
+                        Graphics2D cBlue = context.getGraphic2DForColor(c);
+                        Stroke s = cBlue.getStroke();
+                        cBlue.setStroke(thickStroke);
+                        cBlue.drawPolygon(xPoly, yPoly, xPoly.length);
+                        cBlue.setStroke(s);
                     }
-                    Graphics2D cBlue = context.getGraphic2DForColor(c);
-                    Stroke s = cBlue.getStroke();
-                    cBlue.setStroke(thickStroke);
-                    cBlue.drawPolygon(xPoly, yPoly, xPoly.length);
-                    cBlue.setStroke(s);
                 }
             }
 
-            if (lastBlockEnd > Integer.MIN_VALUE) {
+
+            if (locScale < 5) {
+                drawBases(context, rect, aBlock, alignmentColor, renderOptions.shadeBases, renderOptions.showAllBases);
+            }
+
+            // Draw connecting lines between blocks, if in view
+            if (lastBlockEnd > Integer.MIN_VALUE && x > rect.x) {
                 Graphics2D gLine;
                 Stroke stroke;
                 int gapIdx = blockNumber - 1;
@@ -359,7 +474,8 @@ public class AlignmentRenderer implements FeatureRenderer {
                 } else {
                     gLine = context.getGraphic2DForColor(gapLineColor);
                     stroke = gLine.getStroke();
-                    gLine.setStroke(dashedStroke);
+                    //gLine.setStroke(dashedStroke);
+                    gLine.setStroke(thickStroke);
                 }
 
                 int startX = Math.max(rect.x, lastBlockEnd);
@@ -370,9 +486,11 @@ public class AlignmentRenderer implements FeatureRenderer {
             }
             lastBlockEnd = x + w;
 
-            if (locScale < 5) {
-                drawBases(context, rect, aBlock, alignmentColor, renderOptions.shadeBases, renderOptions.showAllBases);
+            // Next block cannot start before lastBlockEnd.  If its out of view we are done.
+            if (lastBlockEnd > rect.getMaxX()) {
+                break;
             }
+
         }
 
         // Render insertions if locScale ~ 0.25 (base level)
@@ -412,11 +530,12 @@ public class AlignmentRenderer implements FeatureRenderer {
             int dX = (int) Math.max(1, (1.0 / locScale));
             Graphics2D g = (Graphics2D) context.getGraphics().create();
             if (dX >= 8) {
-                Font f = FontManager.getScalableFont(Font.BOLD, Math.min(dX, 12));
+                Font f = FontManager.getFont(Font.BOLD, Math.min(dX, 12));
                 g.setFont(f);
             }
 
             // Get the base qualities, start/end,  and reference sequence
+
             int start = block.getStart();
             int end = start + read.length;
             byte[] reference = isSoftClipped ? null : SequenceManager.readSequence(genome, chr, start, end);
@@ -434,11 +553,11 @@ public class AlignmentRenderer implements FeatureRenderer {
                 // bases are considered mismatched by definition
                 boolean misMatch =
                         isSoftClipped ||
-                        (read[idx] != '=' &&
-                                reference != null &&
-                                idx < reference.length &&
-                                reference[idx] != 0 &&
-                                reference[idx] != read[idx]);
+                                (read[idx] != '=' &&
+                                        reference != null &&
+                                        idx < reference.length &&
+                                        reference[idx] != 0 &&
+                                        reference[idx] != read[idx]);
 
                 if (misMatch || showAllBases) {
                     char c = (char) read[loc - start];
@@ -450,7 +569,7 @@ public class AlignmentRenderer implements FeatureRenderer {
                     }
 
                     if (shadeBases) {
-                        byte qual = block.getQuality(loc - start);
+                        byte qual = block.qualities[loc - start];
                         color = getShadedColor(qual, color, prefs);
                     }
 
@@ -488,14 +607,14 @@ public class AlignmentRenderer implements FeatureRenderer {
         }
     }
 
-    private Color getShadedColor(byte qual, Color color, PreferenceManager.SAMPreferences prefs) {
+    private Color getShadedColor(byte qual, Color color, PreferenceManager prefs) {
         float alpha = 0;
-        int minQ = prefs.getBaseQualityMin();
+        int minQ = prefs.getAsInt(PreferenceManager.SAM_BASE_QUALITY_MIN);
         color.getRGBColorComponents(colorComps);
         if (qual < minQ) {
             alpha = 0.1f;
         } else {
-            int maxQ = prefs.getBaseQualityMax();
+            int maxQ = prefs.getAsInt(PreferenceManager.SAM_BASE_QUALITY_MAX);
             alpha = Math.max(0.1f, Math.min(1.0f, 0.1f + 0.9f * (qual - minQ) / (maxQ - minQ)));
         }
 
@@ -555,33 +674,58 @@ public class AlignmentRenderer implements FeatureRenderer {
         }
     }
 
+    ContinuousColorScale cs = null;
+
+
     private Color getAlignmentColor(Alignment alignment, double locScale,
                                     double center, AlignmentTrack.RenderOptions renderOptions) {
 
         // Set color used to draw the feature.  Highlight features that intersect the
         // center line.  Also restorePersistentState row "score" if alignment intersects center line
 
+        String lb = alignment.getLibrary();
+        if (lb == null) lb = "null";
+        PEStats peStats = renderOptions.peStats.get(lb);
+
         Color c = grey1;
         switch (renderOptions.colorOption) {
             case INSERT_SIZE:
-                if (alignment.isPaired() && alignment.getMate().isMapped()) {
-                    boolean sameChr = alignment.getMate().mateChr.equals(alignment.getChr());
+                boolean isPairedAlignment = alignment instanceof PairedAlignment;
+                if (alignment.isPaired() && alignment.getMate().isMapped() || isPairedAlignment) {
+                    boolean sameChr = isPairedAlignment ||
+                            alignment.getMate().getChr().equals(alignment.getChr());
                     if (sameChr) {
                         int readDistance = Math.abs(alignment.getInferredInsertSize());
-                        if (readDistance > renderOptions.insertSizeThreshold) {
-                            c = ChromosomeColors.getColor(alignment.getMate().mateChr);
-                            //c = getDistanceColor(readDistance);
+                        if (readDistance > 0) {
+                            int minThreshold = renderOptions.getMinInsertSize();
+                            int maxThreshold = renderOptions.getMaxInsertSize();
+                            if (renderOptions.isComputeIsizes() && renderOptions.peStats != null) {
+                                if (peStats != null) {
+                                    minThreshold = peStats.getMinThreshold();
+                                    maxThreshold = peStats.getMaxThreshold();
+                                }
+
+                            }
+
+                            if (readDistance < minThreshold) {
+                                c = smallISizeColor;
+                            } else if (readDistance > maxThreshold) {
+                                c = largeISizeColor;
+                            }
+                            //return renderOptions.insertSizeColorScale.getColor(readDistance);
                         }
                     } else {
-                        c = ChromosomeColors.getColor(alignment.getMate().mateChr);
+                        c = ChromosomeColors.getColor(alignment.getMate().getChr());
                         if (c == null) {
                             c = Color.black;
                         }
                     }
                 }
+
+
                 break;
             case PAIR_ORIENTATION:
-                c = getOrientationColor(alignment);
+                c = getOrientationColor(alignment, peStats);
                 break;
             case READ_STRAND:
                 if (alignment.isNegativeStrand()) {
@@ -597,6 +741,27 @@ public class AlignmentRenderer implements FeatureRenderer {
                     c = posStrandColor;
                 }
                 break;
+            case READ_GROUP:
+                String rg = alignment.getReadGroup();
+                if (rg != null) {
+                    c = readGroupColors.get(rg);
+                    if (c == null) {
+                        c = ColorUtilities.randomColor(readGroupColors.size() + 1);
+                        readGroupColors.put(rg, c);
+                    }
+                }
+                break;
+            case SAMPLE:
+                String sample = alignment.getSample();
+                if (sample != null) {
+                    c = readGroupColors.get(sample);
+                    if (c == null) {
+                        c = ColorUtilities.randomColor(readGroupColors.size() + 1);
+                        readGroupColors.put(sample, c);
+                    }
+                }
+                break;
+
             default:
                 if (renderOptions.shadeCenters && center >= alignment.getStart() && center <= alignment.getEnd()) {
                     if (locScale < 1) {
@@ -623,14 +788,35 @@ public class AlignmentRenderer implements FeatureRenderer {
      * @return
      */
 
-    private Color getOrientationColor(Alignment alignment) {
+    private Color getOrientationColor(Alignment alignment, PEStats peStats) {
 
         Color c = null;
-        if (!alignment.isProperPair() && !alignment.isSmallInsert()) {
-            if (alignment.getAttribute("CS") != null) {
-                c = getSolidScheme().get(alignment.getPairOrientation());
+        if (!alignment.isProperPair()) {
+
+            final String pairOrientation = alignment.getPairOrientation();
+            if (peStats != null) {
+                PEStats.Orientation libraryOrientation = peStats.getOrientation();
+                switch (libraryOrientation) {
+                    case FR:
+                        if (!alignment.isSmallInsert()) {
+                            // if the isize < read length the reads might overlap, invalidating this test
+                            c = frOrientationColors.get(pairOrientation);
+                        }
+                        break;
+                    case RF:
+                        c = rfOrientationColors.get(pairOrientation);
+                        break;
+                    case FF:
+                        c = ffOrientationColors.get(pairOrientation);
+                        break;
+                }
+
             } else {
-                c = getIlluminaScheme().get(alignment.getPairOrientation());
+                if (alignment.getAttribute("CS") != null) {
+                    c = ffOrientationColors.get(pairOrientation);
+                } else {
+                    c = frOrientationColors.get(pairOrientation);
+                }
             }
         }
 
@@ -638,52 +824,12 @@ public class AlignmentRenderer implements FeatureRenderer {
 
     }
 
-    private static final Color LR_COLOR = grey1; // "Normal" alignment color
-    private static final Color RL_COLOR = new Color(0, 150, 0);
-    private static final Color RR_COLOR = new Color(0, 0, 150);
-    private static final Color LL_COLOR = new Color(0, 150, 150);
+    /**
+     * @return
+     */
 
-    Map<String, Color> illuminaScheme;
-    Map<String, Color> solidScheme;
 
-    private Map<String, Color> getIlluminaScheme() {
-        if (illuminaScheme == null) {
-            illuminaScheme = new HashMap();
-            //LR
-            illuminaScheme.put("F1R2", LR_COLOR);
-            illuminaScheme.put("F2R1", LR_COLOR);
-            //LL
-            illuminaScheme.put("F1F2", LL_COLOR);
-            illuminaScheme.put("F2F1", LL_COLOR);
-            //RR
-            illuminaScheme.put("R1R2", RR_COLOR);
-            illuminaScheme.put("R2R1", RR_COLOR);
-            //RL
-            illuminaScheme.put("R1F2", RL_COLOR);
-            illuminaScheme.put("R2F1", RL_COLOR);
-        }
-        return illuminaScheme;
+    public static Map<Character, Color> getNucleotideColors() {
+        return nucleotideColors;
     }
-
-
-    private Map<String, Color> getSolidScheme() {
-        if (solidScheme == null) {
-            solidScheme = new HashMap();
-            //LR
-            solidScheme.put("F1F2", LR_COLOR);
-            solidScheme.put("R2R1", LR_COLOR);
-            //LL -- switched with RR color per Bob's instructions
-            solidScheme.put("F1R2", RR_COLOR);
-            solidScheme.put("R2F1", RR_COLOR);
-            //RR
-            solidScheme.put("R1F2", LL_COLOR);
-            solidScheme.put("F2R1", LL_COLOR);
-            //RL
-            solidScheme.put("R1R2", RL_COLOR);
-            solidScheme.put("F2F1", RL_COLOR);
-        }
-        return solidScheme;
-    }
-
-
 }
