@@ -1,19 +1,19 @@
 /*
- * Copyright (c) 2007-2010 by The Broad Institute, Inc. and the Massachusetts Institute of Technology.
- * All Rights Reserved.
+ * Copyright (c) 2007-2011 by The Broad Institute of MIT and Harvard.  All Rights Reserved.
  *
- * This software is licensed under the terms of the GNU Lesser General Public License (LGPL), Version 2.1 which
- * is available at http://www.opensource.org/licenses/lgpl-2.1.php.
+ * This software is licensed under the terms of the GNU Lesser General Public License (LGPL),
+ * Version 2.1 which is available at http://www.opensource.org/licenses/lgpl-2.1.php.
  *
- * THE SOFTWARE IS PROVIDED "AS IS." THE BROAD AND MIT MAKE NO REPRESENTATIONS OR WARRANTIES OF
- * ANY KIND CONCERNING THE SOFTWARE, EXPRESS OR IMPLIED, INCLUDING, WITHOUT LIMITATION, WARRANTIES
- * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NONINFRINGEMENT, OR THE ABSENCE OF LATENT
- * OR OTHER DEFECTS, WHETHER OR NOT DISCOVERABLE.  IN NO EVENT SHALL THE BROAD OR MIT, OR THEIR
- * RESPECTIVE TRUSTEES, DIRECTORS, OFFICERS, EMPLOYEES, AND AFFILIATES BE LIABLE FOR ANY DAMAGES OF
- * ANY KIND, INCLUDING, WITHOUT LIMITATION, INCIDENTAL OR CONSEQUENTIAL DAMAGES, ECONOMIC
- * DAMAGES OR INJURY TO PROPERTY AND LOST PROFITS, REGARDLESS OF WHETHER THE BROAD OR MIT SHALL
- * BE ADVISED, SHALL HAVE OTHER REASON TO KNOW, OR IN FACT SHALL KNOW OF THE POSSIBILITY OF THE
- * FOREGOING.
+ * THE SOFTWARE IS PROVIDED "AS IS." THE BROAD AND MIT MAKE NO REPRESENTATIONS OR
+ * WARRANTES OF ANY KIND CONCERNING THE SOFTWARE, EXPRESS OR IMPLIED, INCLUDING,
+ * WITHOUT LIMITATION, WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
+ * PURPOSE, NONINFRINGEMENT, OR THE ABSENCE OF LATENT OR OTHER DEFECTS, WHETHER
+ * OR NOT DISCOVERABLE.  IN NO EVENT SHALL THE BROAD OR MIT, OR THEIR RESPECTIVE
+ * TRUSTEES, DIRECTORS, OFFICERS, EMPLOYEES, AND AFFILIATES BE LIABLE FOR ANY DAMAGES
+ * OF ANY KIND, INCLUDING, WITHOUT LIMITATION, INCIDENTAL OR CONSEQUENTIAL DAMAGES,
+ * ECONOMIC DAMAGES OR INJURY TO PROPERTY AND LOST PROFITS, REGARDLESS OF WHETHER
+ * THE BROAD OR MIT SHALL BE ADVISED, SHALL HAVE OTHER REASON TO KNOW, OR IN FACT
+ * SHALL KNOW OF THE POSSIBILITY OF THE FOREGOING.
  */
 
 /*
@@ -25,8 +25,13 @@ package org.broad.igv.sam;
 import com.jidesoft.swing.JidePopupMenu;
 import org.apache.log4j.Logger;
 import org.broad.igv.PreferenceManager;
+import org.broad.igv.data.CoverageDataSource;
+import org.broad.igv.data.DataSource;
+import org.broad.igv.feature.genome.Genome;
+import org.broad.igv.ui.panel.IGVPopupMenu;
+import org.broad.igv.ui.panel.ReferenceFrame;
 import org.broad.igv.util.ColorUtilities;
-import org.broad.igv.util.AsciiLineReader;
+import org.broad.tribble.readers.AsciiLineReader;
 import org.broad.igv.util.ParsingUtils;
 import org.broad.igv.util.ResourceLocator;
 import org.broad.igv.feature.FeatureUtils;
@@ -35,22 +40,19 @@ import org.broad.igv.renderer.BarChartRenderer;
 import org.broad.igv.renderer.DataRange;
 import org.broad.igv.renderer.DataRenderer;
 import org.broad.igv.renderer.Renderer;
-import org.broad.igv.session.ViewContext;
 import org.broad.igv.tdf.TDFDataSource;
 import org.broad.igv.tdf.TDFReader;
 import org.broad.igv.track.*;
 import org.broad.igv.ui.DataRangeDialog;
 import org.broad.igv.ui.FontManager;
-import org.broad.igv.ui.IGVMainFrame;
+import org.broad.igv.ui.IGV;
 import org.broad.igv.ui.util.FileChooserDialog;
 import org.broad.igv.ui.util.MessageUtils;
-import org.broad.igv.ui.util.UIUtilities;
 
 import javax.swing.*;
 import java.awt.*;
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
-import java.awt.event.MouseEvent;
 import java.io.File;
 import java.util.*;
 import java.util.List;
@@ -76,27 +78,29 @@ public class CoverageTrack extends AbstractTrack {
     private boolean showAllSnps;
 
     AlignmentDataManager dataManager;
-    TDFDataSource dataSource;
+    CoverageDataSource dataSource;
     DataRenderer dataSourceRenderer; // = new BarChartRenderer();
     IntervalRenderer intervalRenderer;
     PreferenceManager prefs;
     JMenuItem dataRangeItem;
     JMenuItem autoscaleItem;
+    Genome genome;
 
 
-    public CoverageTrack(String id, String name) {
-        super(id, name);
+    public CoverageTrack(ResourceLocator locator, String name, Genome genome) {
+        super(locator, locator.getPath() + "_coverage", name);
         super.setDataRange(new DataRange(0, 0, 60));
+        this.genome = genome;
         intervalRenderer = new IntervalRenderer();
 
         setColor(AlignmentRenderer.grey1);
         AlignmentRenderer.grey1.getColorComponents(bgColorComps);
 
         prefs = PreferenceManager.getInstance();
-        snpThreshold = prefs.getSAMPreferences().getAlleleFreqThreshold();
+        snpThreshold = prefs.getAsFloat(PreferenceManager.SAM_ALLELE_THRESHOLD);
         autoScale = DEFAULT_AUTOSCALE;
         showReference = DEFAULT_SHOW_REFERENCE;
-        showAllSnps = prefs.getBooleanPreference("COVERAGE.SHOW_ALL_MISMATCHES", DEFAULT_SHOW_REFERENCE);
+        showAllSnps = prefs.getAsBoolean(PreferenceManager.COVERAGE_SHOW_ALL_MISMATCHES);
         //TODO  logScale = prefs.
 
         String snpsFile = prefs.get(PreferenceManager.KNOWN_SNPS, null);
@@ -109,8 +113,8 @@ public class CoverageTrack extends AbstractTrack {
         this.dataManager = dataManager;
     }
 
-    public void setDataSource(TDFDataSource dataSource) {
-        this.dataSource = dataSource;
+    public void setDataSource(DataSource dataSource) {
+        this.dataSource = (CoverageDataSource) dataSource;
         dataSourceRenderer = new BarChartRenderer();
         setDataRange(new DataRange(0, 0, 1.5f * (float) dataSource.getDataMax()));
 
@@ -121,35 +125,50 @@ public class CoverageTrack extends AbstractTrack {
     public void setDataRange(DataRange axisDefinition) {
         // Explicitly setting a data range turns off auto-scale
         autoScale = false;
-
         super.setDataRange(axisDefinition);
     }
 
     public void rescale() {
         if (autoScale & dataManager != null) {
-            AlignmentInterval interval = dataManager.getLoadedInterval();
-            if (interval != null) {
-                int max = Math.max(10, interval.getMaxCount());
-                DataRange.Type type = getDataRange().getType();
-                super.setDataRange(new DataRange(0, 0, max));
-                getDataRange().setType(type);
+            for (AlignmentInterval interval : dataManager.getLoadedIntervals()) {
+                rescaleInterval(interval);
             }
         }
     }
 
+    public void rescale(ReferenceFrame frame) {
+        rescaleInterval(dataManager.getLoadedInterval(frame));
+    }
+
+    private void rescaleInterval(AlignmentInterval interval) {
+        if (interval != null) {
+            int max = Math.max(10, interval.getMaxCount());
+            DataRange.Type type = getDataRange().getType();
+            super.setDataRange(new DataRange(0, 0, max));
+            getDataRange().setType(type);
+        }
+    }
+
 
     public void render(RenderContext context, Rectangle rect) {
 
-        // TODO -- merge the two renderers,  this is very clumsy
-        AlignmentInterval interval = null;
-        if (dataManager != null) {
-            interval = dataManager.getLoadedInterval();
-        }
+        float maxRange = PreferenceManager.getInstance().getAsFloat(PreferenceManager.SAM_MAX_VISIBLE_RANGE);
+        float minVisibleScale = (maxRange * 1000) / 700;
+
+        if (context.getScale() < minVisibleScale) {
+
+            AlignmentInterval interval = null;
+            if (dataManager != null) {
+                interval = dataManager.getLoadedInterval(context);
+
+            }
 
 
-        if (interval != null && interval.overlaps(context.getGenomeId(), context.getChr(), (int) context.getOrigin(),
-                (int) context.getEndLocation())) {
-            intervalRenderer.paint(context, rect, interval);
+            if (interval != null && interval.contains(context.getGenomeId(), context.getChr(), (int) context.getOrigin(),
+                    (int) context.getEndLocation())) {
+                List<AlignmentCounts> counts = interval.getCounts();
+                intervalRenderer.paint(context, rect, counts);
+            }
         }
         // Use precomputed data source, if any
         else if (dataSource != null) {
@@ -159,14 +178,16 @@ public class CoverageTrack extends AbstractTrack {
             int zoom = context.getZoom();
             List<LocusScore> scores = dataSource.getSummaryScoresForRange(chr, start, end, zoom);
             if (scores != null) {
-                dataSourceRenderer.render(this, scores, context, rect);
+                dataSourceRenderer.render(scores, context, rect, this);
             }
 
         }
+        drawBorder(context, rect);
 
 
+    }
 
-
+    private void drawBorder(RenderContext context, Rectangle rect) {
         // Draw border
         context.getGraphic2DForColor(Color.gray).drawLine(
                 rect.x, rect.y + rect.height,
@@ -177,7 +198,7 @@ public class CoverageTrack extends AbstractTrack {
         if (range != null) {
             Graphics2D g = context.getGraphic2DForColor(Color.black);
             Font font = g.getFont();
-            Font smallFont = FontManager.getScalableFont(8);
+            Font smallFont = FontManager.getFont(8);
             try {
                 g.setFont(smallFont);
                 String scale = "[" + (int) range.getMinimum() + " - " +
@@ -188,10 +209,9 @@ public class CoverageTrack extends AbstractTrack {
                 g.setFont(font);
             }
         }
-
     }
 
-    public void setStatType(WindowFunction type) {
+    public void setWindowFunction(WindowFunction type) {
     }
 
     public WindowFunction getWindowFunction() {
@@ -209,54 +229,61 @@ public class CoverageTrack extends AbstractTrack {
         return false;
     }
 
-    public String getValueStringAt(String chr, double position, int y) {
-        AlignmentInterval interval = dataManager.getLoadedInterval();
-        if (interval == null || !interval.contains(chr, (int) position, (int) position)) {
-            return getPrecomputedValueString(chr, position);
-        } else {
-            StringBuffer buf = new StringBuffer();
-            final int pos = (int) position - 1;
-            int totalCount = interval.getTotalCount(pos);
-            buf.append("Total count: " + totalCount + "<br>");
-            for (char c : nucleotides) {
-                int negCount = interval.getNegCount(pos, (byte) c);
-                int posCount = interval.getPosCount(pos, (byte) c);
-                int count = negCount + posCount;
-                int percent = (int) Math.round(((float) count) * 100 / totalCount);
-                char cU = Character.toUpperCase(c);
-                buf.append(cU + "      : " + count);
-                if (count == 0) {
-                    buf.append("<br>");
-                } else {
-                    buf.append("  (" + percent + "%,     " + posCount + "+,   " + negCount + "- )<br>");
+    public String getValueStringAt(String chr, double position, int y, ReferenceFrame frame) {
+
+        float maxRange = PreferenceManager.getInstance().getAsFloat(PreferenceManager.SAM_MAX_VISIBLE_RANGE);
+        float minVisibleScale = (maxRange * 1000) / 700;
+        if (frame.getScale() < minVisibleScale) {
+            AlignmentInterval interval = dataManager.getLoadedInterval(frame);
+            if (interval != null && interval.contains(chr, (int) position, (int) position)) {
+                StringBuffer buf = new StringBuffer();
+                final int pos = (int) position - 1;
+                int totalCount = interval.getTotalCount(pos);
+                buf.append("Total count: " + totalCount + "<br>");
+                for (char c : nucleotides) {
+                    int negCount = interval.getNegCount(pos, (byte) c);
+                    int posCount = interval.getPosCount(pos, (byte) c);
+                    int count = negCount + posCount;
+                    int percent = (int) Math.round(((float) count) * 100 / totalCount);
+                    char cU = Character.toUpperCase(c);
+                    buf.append(cU + "      : " + count);
+                    if (count == 0) {
+                        buf.append("<br>");
+                    } else {
+                        buf.append("  (" + percent + "%,     " + posCount + "+,   " + negCount + "- )<br>");
+                    }
                 }
+                return buf.toString();
             }
-            return buf.toString();
+        } else {
+            return getPrecomputedValueString(chr, position, frame);
+
         }
+        return null;
 
     }
 
-    private String getPrecomputedValueString(String chr, double position) {
+    private String getPrecomputedValueString(String chr, double position, ReferenceFrame frame) {
 
         if (dataSource == null) {
             return "";
         }
-        int zoom = Math.max(0, ViewContext.getInstance().getZoom());
+        int zoom = Math.max(0, frame.getZoom());
         List<LocusScore> scores = dataSource.getSummaryScoresForRange(chr, (int) position - 10, (int) position + 10, zoom);
 
         // give a 2 pixel window, otherwise very narrow features will be missed.
-        double bpPerPixel = ViewContext.getInstance().getScale();
+        double bpPerPixel = frame.getScale();
         double minWidth = 2 * bpPerPixel;    /* * */
 
         if (scores == null) {
             return "";
         } else {
-            LocusScore score = FeatureUtils.getFeatureAt(position, minWidth, scores);
+            LocusScore score = (LocusScore) FeatureUtils.getFeatureAt(position, 0, scores);
             return score == null ? "" : "Mean count: " + score.getScore();
         }
     }
 
-    public float getRegionScore(String chr, int start, int end, int zoom, RegionScoreType type) {
+    public float getRegionScore(String chr, int start, int end, int zoom, RegionScoreType type, ReferenceFrame frame) {
         return 0;
     }
 
@@ -378,7 +405,7 @@ public class CoverageTrack extends AbstractTrack {
     class IntervalRenderer {
 
 
-        private void paint(RenderContext context, Rectangle rect, AlignmentInterval interval) {
+        private void paint(RenderContext context, Rectangle rect, List<AlignmentCounts> countList) {
 
             Graphics2D graphics = context.getGraphic2DForColor(AlignmentRenderer.grey1);
 
@@ -387,119 +414,131 @@ public class CoverageTrack extends AbstractTrack {
 
             // Temporary until proper windowing is implemented
             int lastpX = -1;
+            final double rectX = rect.getX();
+            final double rectMaxX = rect.getMaxX();
+            final double rectY = rect.getY();            
+            final double rectMaxY = rect.getMaxY();
+            final double rectHeight = rect.getHeight();
+            final double origin = context.getOrigin();
+            final double colorScaleMax = getColorScale().getMaximum();
+            final double scale = context.getScale();
 
-            for (int pos = interval.getStart(); pos < interval.getEnd(); pos++) {
+            for (AlignmentCounts alignmentCounts : countList) {
 
-                int pX = (int) (rect.getX() + (pos - context.getOrigin()) / context.getScale());
-                int dX = Math.max(1,
-                        (int) (rect.getX() + (pos + 1 - context.getOrigin()) / context.getScale()) - pX);
-                if (dX > 3) {
-                    dX--;
-                }
+                final int intervalEnd = alignmentCounts.getEnd();
+                final int intervalStart = alignmentCounts.getStart();
+                for (int pos = intervalStart; pos < intervalEnd; pos++) {
 
-                if (pX > rect.getMaxX()) {
-                    break;
-                }
-
-                if (pX + dX >= 0) {
-
-                    // Test to see if any single nucleotide mismatch  (nucleotide other than the reference)
-                    // has posA quality weight > 20% of the total
-                    // Skip this test if the position is in the list of known snps
-                    char ref = Character.toLowerCase((char) interval.getReference(pos));
-                    boolean mismatch = false;
-
-                    Set<Integer> snps = knownSnps == null ? null : knownSnps.get(context.getChr());
-                    if (snps == null || !snps.contains(pos + 1)) {
-                        float threshold = snpThreshold * interval.getTotalQuality(pos);
-                        if (ref > 0) {
-                            for (char c : nucleotides) {
-                                if (c != ref && c != 'n' && interval.getQuality(pos, (byte) c) > threshold) {
-                                    mismatch = true;
-                                    break;
-                                }
-                            }
-                        }
+                    int pX = (int) (rectX + (pos - origin) / scale);
+                    int dX = Math.max(1,
+                            (int) (rectX + (pos + 1 - origin) / scale) - pX);
+                    if (dX > 3) {
+                        dX--;
                     }
 
-                    if (pX > lastpX || mismatch) {
+                    if (pX > rectMaxX) {
+                        break;
+                    }
 
-                        boolean strandOption = false;
+                    if (pX + dX >= 0) {
 
-                        if (strandOption) {
+                        // Test to see if any single nucleotide mismatch  (nucleotide other than the reference)
+                        // has posA quality weight > 20% of the total
+                        // Skip this test if the position is in the list of known snps
+                        char ref = Character.toLowerCase((char) alignmentCounts.getReference(pos));
+                        boolean mismatch = false;
 
-                            int pY = (int) (rect.getY() + rect.getMaxY()) / 2;
-
-                            int totalNegCount = interval.getNegTotal(pos);
-                            int height = (int) (totalNegCount * rect.getHeight() / getColorScale().getMaximum());
-                            height = Math.min(height, rect.height / 2 - 1);
-                            if (height > 0) {
-
-                                graphics.fillRect(pX, pY, dX, height);
-
-                                if (mismatch || showAllSnps) {
-                                    for (char c : nucleotides) {
-                                        if (c != ref) {
-                                            pY = drawStrandBar(context, pos, rect, getColorScale().getMaximum(), pY, pX, dX, c,
-                                                    false, interval);
-                                        }
-                                    }
-                                }
-
-                            }
-
-                            pY = (int) (rect.getY() + rect.getMaxY()) / 2;
-                            int totalPosCount = interval.getPosTotal(pos);
-
-                            height = (int) (totalPosCount * rect.getHeight() / getColorScale().getMaximum());
-                            height = Math.min(height, rect.height / 2 - 1);
-                            int topY = (pY - height);
-
-                            if (height > 0) {
-
-                                graphics.fillRect(pX, topY, dX, height);
-
-                                if (mismatch || showAllSnps) {
-                                    for (char c : nucleotides) {
-                                        if (c != ref) {
-                                            pY = drawStrandBar(context, pos, rect, getColorScale().getMaximum(), pY, pX, dX, c,
-                                                    true, interval);
-                                        }
-                                    }
-                                }
-                            }
-
-                            pY = (int) (rect.getY() + rect.getMaxY()) / 2;
-                            Graphics2D blackGraphics = context.getGraphic2DForColor(lightBlue);
-                            blackGraphics.drawLine(0, pY, rect.width, pY);
-                        } else {
-
-                            int pY = (int) rect.getMaxY() - 1;
-
-                            int totalCount = interval.getTotalCount(pos);
-
-                            double tmp = range.isLog() ? Math.log10(totalCount) / max : totalCount / max;
-                            int height = (int) (tmp * rect.getHeight());
-
-                            height = Math.min(height, rect.height - 1);
-                            int topY = (pY - height);
-
-                            if (height > 0) {
-                                graphics.fillRect(pX, topY, dX, height);
-
-                                if (mismatch || showAllSnps) {
-                                    for (char c : nucleotides) {
-                                        if (!(c == ref && showAllSnps)) {
-                                            pY = drawBar(context, pos, rect, totalCount, max,
-                                                    pY, pX, dX, c, interval, range.isLog());
-                                        }
+                        Set<Integer> snps = knownSnps == null ? null : knownSnps.get(context.getChr());
+                        if (snps == null || !snps.contains(pos + 1)) {
+                            float threshold = snpThreshold * alignmentCounts.getTotalQuality(pos);
+                            if (ref > 0) {
+                                for (char c : nucleotides) {
+                                    if (c != ref && c != 'n' && alignmentCounts.getQuality(pos, (byte) c) > threshold) {
+                                        mismatch = true;
+                                        break;
                                     }
                                 }
                             }
                         }
-                        lastpX = pX;
-                    }
 
+                        if (pX > lastpX || mismatch) {
+
+                            boolean strandOption = false;
+
+                            if (strandOption) {
+
+                                int pY = (int) (rectY + rectMaxY) / 2;
+
+                                int totalNegCount = alignmentCounts.getNegTotal(pos);
+                                int height = (int) (totalNegCount * rectHeight / colorScaleMax);
+                                height = Math.min(height, rect.height / 2 - 1);
+                                if (height > 0) {
+
+                                    graphics.fillRect(pX, pY, dX, height);
+
+                                    if (mismatch || showAllSnps) {
+                                        for (char c : nucleotides) {
+                                            if (c != ref) {
+                                                pY = drawStrandBar(context, pos, rect, colorScaleMax, pY, pX, dX, c,
+                                                        false, alignmentCounts);
+                                            }
+                                        }
+                                    }
+
+                                }
+
+                                pY = (int) (rectY + rectMaxY) / 2;
+                                int totalPosCount = alignmentCounts.getPosTotal(pos);
+
+                                height = (int) (totalPosCount * rectHeight / colorScaleMax);
+                                height = Math.min(height, rect.height / 2 - 1);
+                                int topY = (pY - height);
+
+                                if (height > 0) {
+
+                                    graphics.fillRect(pX, topY, dX, height);
+
+                                    if (mismatch || showAllSnps) {
+                                        for (char c : nucleotides) {
+                                            if (c != ref) {
+                                                pY = drawStrandBar(context, pos, rect, colorScaleMax, pY, pX, dX, c,
+                                                        true, alignmentCounts);
+                                            }
+                                        }
+                                    }
+                                }
+
+                                pY = (int) (rectY + rectMaxY) / 2;
+                                Graphics2D blackGraphics = context.getGraphic2DForColor(lightBlue);
+                                blackGraphics.drawLine(0, pY, rect.width, pY);
+                            } else {
+
+                                int pY = (int) rectMaxY - 1;
+
+                                int totalCount = alignmentCounts.getTotalCount(pos);
+
+                                double tmp = range.isLog() ? Math.log10(totalCount) / max : totalCount / max;
+                                int height = (int) (tmp * rectHeight);
+
+                                height = Math.min(height, rect.height - 1);
+                                int topY = (pY - height);
+
+                                if (height > 0) {
+                                    graphics.fillRect(pX, topY, dX, height);
+
+                                    if (mismatch || showAllSnps) {
+                                        for (char c : nucleotides) {
+                                            if (!(c == ref && showAllSnps)) {
+                                                pY = drawBar(context, pos, rect, totalCount, max,
+                                                        pY, pX, dX, c, alignmentCounts, range.isLog());
+                                            }
+                                        }
+                                    }
+                                }
+                            }
+                            lastpX = pX;
+                        }
+                    }
                 }
             }
         }
@@ -529,12 +568,12 @@ public class CoverageTrack extends AbstractTrack {
                     int pX,
                     int dX,
                     char nucleotide,
-                    AlignmentInterval interval,
+                    AlignmentCounts interval,
                     boolean isLog) {
 
             int count = interval.getCount(pos, (byte) nucleotide);
 
-            Color c = AlignmentRenderer.nucleotideColors.get(nucleotide);
+            Color c = AlignmentRenderer.getNucleotideColors().get(nucleotide);
 
             if (showAllSnps) {
                 int q = interval.getAvgQuality(pos, (byte) nucleotide);
@@ -582,10 +621,10 @@ public class CoverageTrack extends AbstractTrack {
                           int dX,
                           char nucleotide,
                           boolean isPositive,
-                          AlignmentInterval interval) {
+                          AlignmentCounts interval) {
 
 
-            Color c = AlignmentRenderer.nucleotideColors.get(nucleotide);
+            Color c = AlignmentRenderer.getNucleotideColors().get(nucleotide);
             Graphics2D tGraphics = context.getGraphic2DForColor(c);
 
 
@@ -610,12 +649,12 @@ public class CoverageTrack extends AbstractTrack {
 
     private Color getShadedColor(int qual, Color color) {
         float alpha = 0;
-        int minQ = prefs.getSAMPreferences().getBaseQualityMin();
+        int minQ = prefs.getAsInt(PreferenceManager.SAM_BASE_QUALITY_MIN);
         color.getRGBColorComponents(colorComps);
         if (qual < minQ) {
             alpha = 0.1f;
         } else {
-            int maxQ = prefs.getSAMPreferences().getBaseQualityMax();
+            int maxQ = prefs.getAsInt(PreferenceManager.SAM_BASE_QUALITY_MAX);
             alpha = Math.max(0.1f, Math.min(1.0f, 0.1f + 0.9f * (qual - minQ) / (maxQ - minQ)));
         }
 
@@ -639,7 +678,7 @@ public class CoverageTrack extends AbstractTrack {
             attributes.put("path", dataSource.getPath());
         }
         prefs = PreferenceManager.getInstance();
-        if (snpThreshold != prefs.getSAMPreferences().getAlleleFreqThreshold()) {
+        if (snpThreshold != prefs.getAsFloat(PreferenceManager.SAM_ALLELE_THRESHOLD)) {
             attributes.put("snpThreshold", String.valueOf(snpThreshold));
         }
         if (autoScale != DEFAULT_AUTOSCALE) {
@@ -648,7 +687,7 @@ public class CoverageTrack extends AbstractTrack {
         if (showReference != DEFAULT_SHOW_REFERENCE) {
             attributes.put("showReference", String.valueOf(showReference));
         }
-        if (showAllSnps != prefs.getBooleanPreference("COVERAGE.SHOW_ALL_MISMATCHES", false)) {
+        if (showAllSnps != prefs.getAsBoolean(PreferenceManager.COVERAGE_SHOW_ALL_MISMATCHES)) {
             attributes.put("showAllSnps", String.valueOf(showAllSnps));
         }
 
@@ -668,7 +707,7 @@ public class CoverageTrack extends AbstractTrack {
         value = attributes.get("path");
         if (value != null) {
             TDFReader reader = TDFReader.getReader(value);
-            TDFDataSource ds = new TDFDataSource(reader, 0, "");
+            TDFDataSource ds = new TDFDataSource(reader, 0, "", genome);
             setDataSource(ds);
         }
         value = attributes.get("snpThreshold");
@@ -690,24 +729,15 @@ public class CoverageTrack extends AbstractTrack {
 
     }
 
-
+    /**
+     * Override to return a specialized popup menu
+     *
+     * @return
+     */
     @Override
-    public boolean handleClick(MouseEvent e) {
-        if (e.isPopupTrigger()) {
-            getPopupMenu(e).show(e.getComponent(), e.getX(), e.getY());
-            //sortRows();
-            //IGVMainFrame.getInstance().repaintDataPanels();
-            return true;
-        } else {
-            return super.handleClick(e);
-        }
+    public IGVPopupMenu getPopupMenu(TrackClickEvent te) {
 
-    }
-
-    public JPopupMenu getPopupMenu(
-            final MouseEvent evt) {
-
-        JPopupMenu popupMenu = new JidePopupMenu();
+        IGVPopupMenu popupMenu = new IGVPopupMenu();
 
         JLabel popupTitle = new JLabel("  " + getName(), JLabel.CENTER);
 
@@ -764,9 +794,7 @@ public class CoverageTrack extends AbstractTrack {
                 if (selectedTracks.size() > 0) {
 
                     DataRange prevAxisDefinition = selectedTracks.iterator().next().getDataRange();
-                    DataRangeDialog dlg = new DataRangeDialog(
-                            IGVMainFrame.getInstance(),
-                            prevAxisDefinition);
+                    DataRangeDialog dlg = new DataRangeDialog(IGV.getMainFrame(), prevAxisDefinition);
                     dlg.setHideMid(true);
                     dlg.setVisible(true);
                     if (!dlg.isCanceled()) {
@@ -782,7 +810,7 @@ public class CoverageTrack extends AbstractTrack {
                         for (Track track : selectedTracks) {
                             track.setDataRange(dataRange);
                         }
-                        IGVMainFrame.getInstance().repaint();
+                        IGV.getMainFrame().repaint();
                     }
                 }
 
@@ -807,7 +835,7 @@ public class CoverageTrack extends AbstractTrack {
                 try {
                     float tmp = Float.parseFloat(value);
                     snpThreshold = tmp;
-                    IGVMainFrame.getInstance().repaintDataPanels();
+                    IGV.getInstance().repaintDataPanels();
                 }
                 catch (Exception exc) {
                     //log
@@ -828,7 +856,7 @@ public class CoverageTrack extends AbstractTrack {
             public void actionPerformed(ActionEvent e) {
 
 
-                FileChooserDialog trackFileDialog = IGVMainFrame.getInstance().getTrackFileChooser();
+                FileChooserDialog trackFileDialog = IGV.getInstance().getTrackFileChooser();
                 trackFileDialog.setMultiSelectionEnabled(false);
                 trackFileDialog.setVisible(true);
                 if (!trackFileDialog.isCanceled()) {
@@ -837,9 +865,9 @@ public class CoverageTrack extends AbstractTrack {
                     if (path.endsWith(".tdf") || path.endsWith(".tdf")) {
 
                         TDFReader reader = TDFReader.getReader(file.getAbsolutePath());
-                        TDFDataSource ds = new TDFDataSource(reader, 0, getName() + " coverage");
+                        TDFDataSource ds = new TDFDataSource(reader, 0, getName() + " coverage", genome);
                         setDataSource(ds);
-                        IGVMainFrame.getInstance().repaintDataPanels();
+                        IGV.getInstance().repaintDataPanels();
                     } else {
                         MessageUtils.showMessage("Coverage data must be in .tdf format");
                     }
@@ -863,7 +891,7 @@ public class CoverageTrack extends AbstractTrack {
                 if (autoScale) {
                     rescale();
                 }
-                IGVMainFrame.getInstance().repaintDataPanels();
+                IGV.getInstance().repaintDataPanels();
 
             }
         });
@@ -885,7 +913,7 @@ public class CoverageTrack extends AbstractTrack {
                         DataRange.Type.LOG :
                         DataRange.Type.LINEAR;
                 dataRange.setType(scaleType);
-                IGVMainFrame.getInstance().repaintDataPanels();
+                IGV.getInstance().repaintDataPanels();
             }
         });
 
