@@ -1,45 +1,44 @@
 /*
- * Copyright (c) 2007-2010 by The Broad Institute, Inc. and the Massachusetts Institute of Technology.
- * All Rights Reserved.
+ * Copyright (c) 2007-2011 by The Broad Institute of MIT and Harvard.  All Rights Reserved.
  *
- * This software is licensed under the terms of the GNU Lesser General Public License (LGPL), Version 2.1 which
- * is available at http://www.opensource.org/licenses/lgpl-2.1.php.
+ * This software is licensed under the terms of the GNU Lesser General Public License (LGPL),
+ * Version 2.1 which is available at http://www.opensource.org/licenses/lgpl-2.1.php.
  *
- * THE SOFTWARE IS PROVIDED "AS IS." THE BROAD AND MIT MAKE NO REPRESENTATIONS OR WARRANTIES OF
- * ANY KIND CONCERNING THE SOFTWARE, EXPRESS OR IMPLIED, INCLUDING, WITHOUT LIMITATION, WARRANTIES
- * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NONINFRINGEMENT, OR THE ABSENCE OF LATENT
- * OR OTHER DEFECTS, WHETHER OR NOT DISCOVERABLE.  IN NO EVENT SHALL THE BROAD OR MIT, OR THEIR
- * RESPECTIVE TRUSTEES, DIRECTORS, OFFICERS, EMPLOYEES, AND AFFILIATES BE LIABLE FOR ANY DAMAGES OF
- * ANY KIND, INCLUDING, WITHOUT LIMITATION, INCIDENTAL OR CONSEQUENTIAL DAMAGES, ECONOMIC
- * DAMAGES OR INJURY TO PROPERTY AND LOST PROFITS, REGARDLESS OF WHETHER THE BROAD OR MIT SHALL
- * BE ADVISED, SHALL HAVE OTHER REASON TO KNOW, OR IN FACT SHALL KNOW OF THE POSSIBILITY OF THE
- * FOREGOING.
+ * THE SOFTWARE IS PROVIDED "AS IS." THE BROAD AND MIT MAKE NO REPRESENTATIONS OR
+ * WARRANTES OF ANY KIND CONCERNING THE SOFTWARE, EXPRESS OR IMPLIED, INCLUDING,
+ * WITHOUT LIMITATION, WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
+ * PURPOSE, NONINFRINGEMENT, OR THE ABSENCE OF LATENT OR OTHER DEFECTS, WHETHER
+ * OR NOT DISCOVERABLE.  IN NO EVENT SHALL THE BROAD OR MIT, OR THEIR RESPECTIVE
+ * TRUSTEES, DIRECTORS, OFFICERS, EMPLOYEES, AND AFFILIATES BE LIABLE FOR ANY DAMAGES
+ * OF ANY KIND, INCLUDING, WITHOUT LIMITATION, INCIDENTAL OR CONSEQUENTIAL DAMAGES,
+ * ECONOMIC DAMAGES OR INJURY TO PROPERTY AND LOST PROFITS, REGARDLESS OF WHETHER
+ * THE BROAD OR MIT SHALL BE ADVISED, SHALL HAVE OTHER REASON TO KNOW, OR IN FACT
+ * SHALL KNOW OF THE POSSIBILITY OF THE FOREGOING.
  */
 package org.broad.igv.session;
 
 import org.apache.log4j.Logger;
+import org.broad.igv.feature.RegionOfInterest;
+import org.broad.igv.lists.GeneList;
+import org.broad.igv.lists.GeneListManager;
 import org.broad.igv.renderer.ColorScale;
 import org.broad.igv.renderer.ColorScaleFactory;
 import org.broad.igv.renderer.ContinuousColorScale;
 import org.broad.igv.renderer.DataRange;
-import org.broad.igv.track.AttributeManager;
-import org.broad.igv.track.Track;
-import org.broad.igv.track.TrackLoader;
-import org.broad.igv.track.TrackType;
-import org.broad.igv.ui.IGVMainFrame;
-import org.broad.igv.ui.RegionOfInterest;
+import org.broad.igv.track.*;
+import org.broad.igv.ui.IGV;
 import org.broad.igv.ui.TrackFilter;
 import org.broad.igv.ui.TrackFilterElement;
+import org.broad.igv.ui.panel.FrameManager;
+import org.broad.igv.ui.panel.ReferenceFrame;
 import org.broad.igv.ui.panel.TrackPanel;
 import org.broad.igv.ui.util.MessageUtils;
-import org.broad.igv.util.ColorUtilities;
+import org.broad.igv.util.*;
 import org.broad.igv.util.FilterElement.BooleanOperator;
 import org.broad.igv.util.FilterElement.Operator;
-import org.broad.igv.util.ResourceLocator;
 import org.w3c.dom.*;
 import org.xml.sax.SAXException;
 
-import javax.swing.*;
 import javax.xml.parsers.DocumentBuilder;
 import javax.xml.parsers.DocumentBuilderFactory;
 import javax.xml.parsers.ParserConfigurationException;
@@ -65,12 +64,16 @@ public class SessionReader {
     private boolean panelElementPresent = false;
     private int version;
 
+    private IGV igv;
+
 
     /**
      * Map of track id -> track.  It is important to maintin the order in which tracks are added, thus
      * the use of LinkedHashMap.
      */
     Map<String, List<Track>> trackDictionary = new LinkedHashMap();
+    private Track geneTrack = null;
+    private Track seqTrack = null;
 
 
     static {
@@ -83,7 +86,9 @@ public class SessionReader {
      */
     public static enum SessionElement {
 
+
         PANEL("Panel"),
+        PANEL_LAYOUT("PanelLayout"),
         TRACK("Track"),
         COLOR_SCALE("ColorScale"),
         COLOR_SCALES("ColorScales"),
@@ -102,7 +107,11 @@ public class SessionReader {
         REGIONS("Regions"),
         DATA_RANGE("DataRange"),
         PREFERENCES("Preferences"),
-        PROPERTY("Property");
+        PROPERTY("Property"),
+        GENE_LIST("GeneList"),
+        HIDDEN_ATTRIBUTES("HiddenAttributes"),
+        ATTRIBUTE("Attribute"),
+        FRAME("Frame");
 
         private String name;
 
@@ -110,31 +119,15 @@ public class SessionReader {
             this.name = name;
         }
 
-        /**
-         * Method description
-         *
-         * @return
-         */
         public String getText() {
             return name;
         }
 
-        /**
-         * Method description
-         *
-         * @return
-         */
         @Override
         public String toString() {
             return getText();
         }
 
-        /**
-         * Method description
-         *
-         * @param value
-         * @return
-         */
         static public SessionElement findEnum(String value) {
 
             if (value == null) {
@@ -152,9 +145,13 @@ public class SessionReader {
 
         BOOLEAN_OPERATOR("booleanOperator"),
         COLOR("color"),
+        ALT_COLOR("altColor"),
+        COLOR_MODE("colorMode"),
         CHROMOSOME("chromosome"),
         END_INDEX("end"),
         EXPAND("expand"),
+        SQUISH("squish"),
+        DISPLAY_MODE("displayMode"),
         FILTER_MATCH("match"),
         FILTER_SHOW_ALL_TRACKS("showTracks"),
         GENOME("genome"),
@@ -175,6 +172,11 @@ public class SessionReader {
         VERSION("version"),
         VISIBLE("visible"),
         WINDOW_FUNCTION("windowFunction"),
+        RENDER_NAME("renderName"),
+        GENOTYPE_HEIGHT("genotypeHeight"),
+        VARIANT_HEIGHT("variantHeight"),
+        PREVIOUS_HEIGHT("previousHeight"),
+        FEATURE_WINDOW("featureVisibilityWindow"),
         DISPLAY_NAME("displayName"),
         COLOR_SCALE("colorScale"),
 
@@ -189,7 +191,11 @@ public class SessionReader {
         DESCRIPTION("description"),
         TYPE("type"),
         COVERAGE("coverage"),
-        TRACK_LINE("trackLine");
+        TRACK_LINE("trackLine"),
+
+        CHR("chr"),
+        START("start"),
+        END("end");
 
         //TODO Add the following into the Attributes
         /*
@@ -229,16 +235,22 @@ public class SessionReader {
     }
 
 
+    public SessionReader(IGV igv) {
+        this.igv = igv;
+    }
+
+
     /**
      * @param inputStream
-     * @param sessionName
-     * @return
+     * @param session
+     * @param sessionName @return
      * @throws RuntimeException
      */
 
-    public Session loadSession(InputStream inputStream, Session session, String sessionName)
+    public void loadSession(InputStream inputStream, Session session, String sessionName)
             throws RuntimeException {
 
+
         log.debug("Load session");
 
 
@@ -265,28 +277,98 @@ public class SessionReader {
         addLeftoverTracks(trackDictionary.values());
 
         if (session.getGroupTracksBy() != null && session.getGroupTracksBy().length() > 0) {
-            IGVMainFrame.getInstance().getTrackManager().setGroupByAttribute(session.getGroupTracksBy());
+            IGV.getInstance().getTrackManager().setGroupByAttribute(session.getGroupTracksBy());
         }
 
-        IGVMainFrame.getInstance().getTrackManager().resetOverlayTracks();
+        if (session.isRemoveEmptyPanels()) {
+            IGV.getInstance().getMainPanel().removeEmptyDataPanels();
+        }
 
-        return session;
+        IGV.getInstance().getTrackManager().resetOverlayTracks();
+
+    }
+
+
+    private void processRootNode(Session session, Node node, HashMap additionalInformation) {
+
+        if ((node == null) || (session == null)) {
+            MessageUtils.showMessage("Invalid session file: root node not found");
+            return;
+        }
+
+        String nodeName = node.getNodeName();
+        if (!(nodeName.equalsIgnoreCase(SessionElement.GLOBAL.getText()) || nodeName.equalsIgnoreCase(SessionElement.SESSION.getText()))) {
+            MessageUtils.showMessage("Session files must begin with a \"Global\" or \"Session\" element.  Found: " + nodeName);
+        }
+        process(session, node, additionalInformation);
+
+        Element element = (Element) node;
+
+        IGV.getInstance().selectGenomeFromList(getAttribute(element, SessionAttribute.GENOME.getText()));
+
+        session.setLocus(getAttribute(element, SessionAttribute.LOCUS.getText()));
+        session.setGroupTracksBy(getAttribute(element, SessionAttribute.GROUP_TRACKS_BY.getText()));
+
+        String removeEmptyTracks = getAttribute(element, "removeEmptyTracks");
+        if (removeEmptyTracks != null) {
+            try {
+                Boolean b = Boolean.parseBoolean(removeEmptyTracks);
+                session.setRemoveEmptyPanels(b);
+            }
+            catch (Exception e) {
+                log.error("Error parsing removeEmptyTracks string: " + removeEmptyTracks, e);
+            }
+        }
+
+        String versionString = getAttribute(element, SessionAttribute.VERSION.getText());
+        try {
+            version = Integer.parseInt(versionString);
+        }
+        catch (NumberFormatException e) {
+            log.error("Non integer version number in session file: " + versionString);
+        }
+        session.setVersion(version);
+
+        geneTrack = IGV.getInstance().getTrackManager().getGeneTrack();
+        if (geneTrack != null) {
+            trackDictionary.put(geneTrack.getId(), Arrays.asList(geneTrack));
+        }
+        seqTrack = IGV.getInstance().getTrackManager().getSequenceTrack();
+        if (seqTrack != null) {
+            trackDictionary.put(seqTrack.getId(), Arrays.asList(seqTrack));
+        }
+
+
+        NodeList elements = element.getChildNodes();
+        process(session, elements, additionalInformation);
+
+        // ReferenceFrame.getInstance().invalidateLocationScale();
     }
 
     //TODO Check to make sure tracks are not being created twice
     //TODO -- DONT DO THIS FOR NEW SESSIONS
 
     private void addLeftoverTracks(Collection<List<Track>> tmp) {
+        Map<String, TrackPanel> trackPanelCache = new HashMap();
         if (version < 3 || !panelElementPresent) {
             for (List<Track> tracks : tmp) {
                 for (Track track : tracks) {
-                    TrackPanel group = IGVMainFrame.getInstance().getTrackManager().getPanelFor(new ResourceLocator(track.getSourceFile()));
-                    group.addTrack(track);
+                    if (track != geneTrack && track != seqTrack && track.getResourceLocator() != null) {
+
+                        TrackPanel panel = trackPanelCache.get(track.getResourceLocator().getPath());
+                        if (panel == null) {
+                            panel = IGV.getInstance().getTrackManager().getPanelFor(track.getResourceLocator());
+                            trackPanelCache.put(track.getResourceLocator().getPath(), panel);
+                        }
+                        panel.addTrack(track);
+                    }
                 }
             }
         }
+
     }
 
+
     /**
      * Process a single session element node.
      *
@@ -300,73 +382,43 @@ public class SessionReader {
         }
 
         String nodeName = element.getNodeName();
-        if (true) {
 
-            if (nodeName.equalsIgnoreCase(SessionElement.GLOBAL.getText()) ||
-                    nodeName.equalsIgnoreCase(SessionElement.SESSION.getText())) {
-                processGlobal(session, (Element) element, additionalInformation);
-            } else if (nodeName.equalsIgnoreCase(SessionElement.FILES.getText())) {
-                processFiles(session, (Element) element, additionalInformation);
-            } else if (nodeName.equalsIgnoreCase(SessionElement.DATA_FILE.getText())) {
-                processDataFile(session, (Element) element, additionalInformation);
-            } else if (nodeName.equalsIgnoreCase(SessionElement.RESOURCES.getText())) {
-                processResources(session, (Element) element, additionalInformation);
-            } else if (nodeName.equalsIgnoreCase(SessionElement.RESOURCE.getText())) {
-                processResource(session, (Element) element, additionalInformation);
-            } else if (nodeName.equalsIgnoreCase(SessionElement.REGIONS.getText())) {
-                processRegions(session, (Element) element, additionalInformation);
-            } else if (nodeName.equalsIgnoreCase(SessionElement.REGION.getText())) {
-                processRegion(session, (Element) element, additionalInformation);
-            } else if (nodeName.equalsIgnoreCase(SessionElement.FILTER.getText())) {
-                processFilter(session, (Element) element, additionalInformation);
-            } else if (nodeName.equalsIgnoreCase(SessionElement.FILTER_ELEMENT.getText())) {
-                processFilterElement(session, (Element) element, additionalInformation);
-            } else if (nodeName.equalsIgnoreCase(SessionElement.COLOR_SCALES.getText())) {
-                processColorScales(session, (Element) element, additionalInformation);
-            } else if (nodeName.equalsIgnoreCase(SessionElement.COLOR_SCALE.getText())) {
-                processColorScale(session, (Element) element, additionalInformation);
-            } else if (nodeName.equalsIgnoreCase(SessionElement.PREFERENCES.getText())) {
-                processPreferences(session, (Element) element, additionalInformation);
-            } else if (nodeName.equalsIgnoreCase(SessionElement.PROPERTY.getText())) {
-                processPreference(session, (Element) element, additionalInformation);
-            } else if (nodeName.equalsIgnoreCase(SessionElement.DATA_TRACKS.getText()) ||
-                    nodeName.equalsIgnoreCase(SessionElement.FEATURE_TRACKS.getText()) ||
-                    nodeName.equalsIgnoreCase(SessionElement.PANEL.getText())) {
-                processPanel(session, (Element) element, additionalInformation);
-            }
+        if (nodeName.equalsIgnoreCase(SessionElement.FILES.getText())) {
+            processFiles(session, (Element) element, additionalInformation);
+        } else if (nodeName.equalsIgnoreCase(SessionElement.DATA_FILE.getText())) {
+            processDataFile(session, (Element) element, additionalInformation);
+        } else if (nodeName.equalsIgnoreCase(SessionElement.RESOURCES.getText())) {
+            processResources(session, (Element) element, additionalInformation);
+        } else if (nodeName.equalsIgnoreCase(SessionElement.RESOURCE.getText())) {
+            processResource(session, (Element) element, additionalInformation);
+        } else if (nodeName.equalsIgnoreCase(SessionElement.REGIONS.getText())) {
+            processRegions(session, (Element) element, additionalInformation);
+        } else if (nodeName.equalsIgnoreCase(SessionElement.REGION.getText())) {
+            processRegion(session, (Element) element, additionalInformation);
+        } else if (nodeName.equalsIgnoreCase(SessionElement.GENE_LIST.getText())) {
+            processGeneList(session, (Element) element, additionalInformation);
+        } else if (nodeName.equalsIgnoreCase(SessionElement.FILTER.getText())) {
+            processFilter(session, (Element) element, additionalInformation);
+        } else if (nodeName.equalsIgnoreCase(SessionElement.FILTER_ELEMENT.getText())) {
+            processFilterElement(session, (Element) element, additionalInformation);
+        } else if (nodeName.equalsIgnoreCase(SessionElement.COLOR_SCALES.getText())) {
+            processColorScales(session, (Element) element, additionalInformation);
+        } else if (nodeName.equalsIgnoreCase(SessionElement.COLOR_SCALE.getText())) {
+            processColorScale(session, (Element) element, additionalInformation);
+        } else if (nodeName.equalsIgnoreCase(SessionElement.PREFERENCES.getText())) {
+            processPreferences(session, (Element) element, additionalInformation);
+        } else if (nodeName.equalsIgnoreCase(SessionElement.DATA_TRACKS.getText()) ||
+                nodeName.equalsIgnoreCase(SessionElement.FEATURE_TRACKS.getText()) ||
+                nodeName.equalsIgnoreCase(SessionElement.PANEL.getText())) {
+            processPanel(session, (Element) element, additionalInformation);
+        } else if (nodeName.equalsIgnoreCase(SessionElement.PANEL_LAYOUT.getText())) {
+            processPanelLayout(session, (Element) element, additionalInformation);
+        } else if (nodeName.equalsIgnoreCase(SessionElement.HIDDEN_ATTRIBUTES.getText())) {
+            processHiddenAttributes(session, (Element) element, additionalInformation);
         }
+
     }
 
-    private void processRootNode(Session session, Node element, HashMap additionalInformation) {
-
-        if ((element == null) || (session == null)) {
-            return;
-        }
-
-        String nodeName = element.getNodeName();
-        if (!(nodeName.equalsIgnoreCase(SessionElement.GLOBAL.getText()) || nodeName.equalsIgnoreCase(SessionElement.SESSION.getText()))) {
-            MessageUtils.showMessage("Session files must begin with a \"Global\" or \"Session\" element.  Found: " + nodeName);
-        }
-        process(session, element, additionalInformation);
-    }
-
-    private void processGlobal(Session session, Element element, HashMap additionalInformation) {
-
-        IGVMainFrame.getInstance().selectGenomeFromList(getAttribute(element, SessionAttribute.GENOME.getText()));
-        session.setLocus(getAttribute(element, SessionAttribute.LOCUS.getText()));
-        session.setGroupTracksBy(getAttribute(element, SessionAttribute.GROUP_TRACKS_BY.getText()));
-        String versionString = getAttribute(element, SessionAttribute.VERSION.getText());
-        try {
-            version = Integer.parseInt(versionString);
-        }
-        catch (NumberFormatException e) {
-            log.error("Non integer version number in session file: " + versionString);
-        }
-        NodeList elements = element.getChildNodes();
-        process(session, elements, additionalInformation);
-
-        // ViewContext.getInstance().invalidateLocationScale();
-    }
 
     /**
      * Process the Files element.
@@ -410,50 +462,38 @@ public class SessionReader {
         }
         if (dataFiles.size() > 0) {
 
-            TrackLoader loader = new TrackLoader();
             for (ResourceLocator locator : dataFiles) {
-                //TODO Go through the tracks and reassign them to the resourceLocator
-                try {
-                    List<Track> tracks = loader.load(locator);
-                    if (tracks.size() > 0) {
-                        for (Track track : tracks) {
 
-                            String id = version < 3 ? track.getId_142() : track.getId();
+                List<Track> tracks = igv.getTrackManager().load(locator);
 
-                            List<Track> trackList = trackDictionary.get(id);
-                            if (trackList == null) {
-                                trackList = new ArrayList();
-                                trackDictionary.put(id, trackList);
+                for (Track track : tracks) {
 
-                            }
-                            trackList.add(track);
+                    String id = track.getId();
+                    List<Track> trackList = trackDictionary.get(id);
+                    if (trackList == null) {
+                        trackList = new ArrayList();
+                        trackDictionary.put(id, trackList);
 
-                            track.setAttributeValue("DATA FILE", locator.getPath());
-                            track.setAttributeValue("DATA TYPE", track.getTrackType().toString());
-
-                        }
-                    } else {
-                        AttributeManager.getInstance().loadSampleInfo(locator);
                     }
-                } catch (Exception e) {
-                    log.error("Error loading " + locator.getPath(), e);
-                    MessageUtils.showMessage("<html>Error loading: " + locator.getPath() + "<br>" + e.toString());
+                    trackList.add(track);
                 }
+
             }
-
-            // TODO -- sample info file
-
         }
         dataFiles = null;
     }
 
     /**
-     * Process the data file element.  If relativePaths == true treat the
-     * file path as relative to the session file path.  If false
+     * //TODO -- I think this method can be removed
      *
      * @param session
      * @param element
+     * @Deprecated
+     * @Deprectated -- user processResource
+     * Process the data file element.  If relativePaths == true treat the
+     * file path as relative to the session file path.  If false
      */
+
     private void processDataFile(Session session, Element element, HashMap additionalInformation) {
 
         ResourceLocator resourceLocator = null;
@@ -465,8 +505,7 @@ public class SessionReader {
 
         // If file is local
         if ((serverURL == null || serverURL.trim().equals("")) &&
-                !(filePath.startsWith("http:") || filePath.startsWith("https:") ||
-                        filePath.startsWith("file:"))) {
+                !(IGVHttpClientUtils.isURL(filePath.toLowerCase()))) {
             String relPathValue = getAttribute(element, SessionAttribute.RELATIVE_PATH.getText());
             boolean relativePaths = ((relPathValue != null) && relPathValue.equalsIgnoreCase("true"));
             File parent = (relativePaths ? new File(session.getPath()).getParentFile() : null);
@@ -496,10 +535,6 @@ public class SessionReader {
 
     private void processResource(Session session, Element element, HashMap additionalInformation) {
 
-        String serverURL = getAttribute(element, SessionAttribute.SERVER_URL.getText());
-        String path = getAttribute(element, SessionAttribute.PATH.getText());
-        ResourceLocator resourceLocator = new ResourceLocator(serverURL, path);
-
         String label = getAttribute(element, SessionAttribute.LABEL.getText());
         String name = getAttribute(element, SessionAttribute.NAME.getText());
         String sampleId = getAttribute(element, SessionAttribute.SAMPLE_ID.getText());
@@ -509,6 +544,26 @@ public class SessionReader {
         String trackLine = getAttribute(element, SessionAttribute.TRACK_LINE.getText());
         String colorString = getAttribute(element, SessionAttribute.COLOR.getText());
 
+        String relPathValue = getAttribute(element, SessionAttribute.RELATIVE_PATH.getText());
+        boolean relativePaths = ((relPathValue != null) && relPathValue.equalsIgnoreCase("true"));
+        String serverURL = getAttribute(element, SessionAttribute.SERVER_URL.getText());
+        String path = getAttribute(element, SessionAttribute.PATH.getText());
+
+        ResourceLocator resourceLocator = new ResourceLocator(serverURL, path);
+        if (relativePaths) {
+            if (FileUtils.isRemote(session.getPath())) {
+                int idx = session.getPath().lastIndexOf("/");
+                String basePath = session.getPath().substring(0, idx);
+                String resPath = basePath + "/" + path;
+                resourceLocator = new ResourceLocator(serverURL, resPath);
+            } else {
+                File parent = (relativePaths ? new File(session.getPath()).getParentFile() : null);
+                File file = new File(parent, path);
+                resourceLocator = new ResourceLocator(serverURL, file.getAbsolutePath());
+            }
+        }
+
+
         String url = getAttribute(element, SessionAttribute.URL.getText());
         if (url == null) {
             url = getAttribute(element, SessionAttribute.FEATURE_URL.getText());
@@ -542,7 +597,7 @@ public class SessionReader {
 
         if (colorString != null) {
             try {
-                Color c = ColorUtilities.getColorFromString(colorString);
+                Color c = ColorUtilities.stringToColor(colorString);
                 resourceLocator.setColor(c);
             } catch (Exception e) {
                 log.error("Error setting color: ", e);
@@ -575,12 +630,79 @@ public class SessionReader {
         String description = getAttribute(element, SessionAttribute.DESCRIPTION.getText());
 
         RegionOfInterest region = new RegionOfInterest(chromosome, new Integer(start), new Integer(end), description);
-        IGVMainFrame.getInstance().addRegionOfInterest(region);
+        IGV.getInstance().addRegionOfInterest(region);
 
         NodeList elements = element.getChildNodes();
         process(session, elements, additionalInformation);
     }
 
+
+    private void processHiddenAttributes(Session session, Element element, HashMap additionalInformation) {
+
+//        session.clearRegionsOfInterest();
+        NodeList elements = element.getChildNodes();
+        if (elements.getLength() > 0) {
+            Set<String> attributes = new HashSet();
+            for (int i = 0; i < elements.getLength(); i++) {
+                Node childNode = elements.item(i);
+                if (childNode.getNodeName().equals(SessionReader.SessionElement.ATTRIBUTE.getText())) {
+                    attributes.add(((Element) childNode).getAttribute(SessionReader.SessionAttribute.NAME.getText()));
+                }
+            }
+            session.setHiddenAttributes(attributes);
+        }
+    }
+
+    private void processGeneList(Session session, Element element, HashMap additionalInformation) {
+
+        String name = getAttribute(element, SessionAttribute.NAME.getText());
+
+        String txt = element.getTextContent();
+        String[] genes = txt.trim().split("\\s+");
+        GeneList gl = new GeneList(name, Arrays.asList(genes));
+        GeneListManager.getInstance().addGeneList(gl);
+        session.setCurrentGeneList(gl);
+
+        // Adjust frames
+        processFrames(element);
+    }
+
+    private void processFrames(Element element) {
+        NodeList elements = element.getChildNodes();
+        if (elements.getLength() > 0) {
+            Map<String, ReferenceFrame> frames = new HashMap();
+            for (ReferenceFrame f : FrameManager.getFrames()) {
+                frames.put(f.getName(), f);
+            }
+            List<ReferenceFrame> reorderedFrames = new ArrayList();
+
+            for (int i = 0; i < elements.getLength(); i++) {
+                Node childNode = elements.item(i);
+                if (childNode.getNodeName().equalsIgnoreCase(SessionElement.FRAME.getText())) {
+                    String frameName = getAttribute((Element) childNode, SessionAttribute.NAME.getText());
+
+                    ReferenceFrame f = frames.get(frameName);
+                    if (f != null) {
+                        reorderedFrames.add(f);
+                        try {
+                            String chr = getAttribute((Element) childNode, SessionAttribute.CHR.getText());
+                            int start = (int) Double.parseDouble(getAttribute((Element) childNode, SessionAttribute.START.getText()));
+                            int end = (int) Double.parseDouble(getAttribute((Element) childNode, SessionAttribute.END.getText()));
+                            f.setInterval(chr, start, end);
+                        } catch (NumberFormatException e) {
+                            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
+                        }
+                    }
+
+                }
+            }
+            if (reorderedFrames.size() > 0) {
+                FrameManager.setFrames(reorderedFrames);
+            }
+        }
+        IGV.getInstance().resetFrames();
+    }
+
     private void processFilter(Session session, Element element, HashMap additionalInformation) {
 
         String match = getAttribute(element, SessionAttribute.FILTER_MATCH.getText());
@@ -598,15 +720,15 @@ public class SessionReader {
 
         // Set filter properties
         if ("all".equalsIgnoreCase(match)) {
-            IGVMainFrame.getInstance().setFilterMatchAll(true);
+            IGV.getInstance().setFilterMatchAll(true);
         } else if ("any".equalsIgnoreCase(match)) {
-            IGVMainFrame.getInstance().setFilterMatchAll(false);
+            IGV.getInstance().setFilterMatchAll(false);
         }
 
         if ("true".equalsIgnoreCase(showAllTracks)) {
-            IGVMainFrame.getInstance().setFilterShowAllTracks(true);
+            IGV.getInstance().setFilterShowAllTracks(true);
         } else {
-            IGVMainFrame.getInstance().setFilterShowAllTracks(false);
+            IGV.getInstance().setFilterShowAllTracks(false);
         }
     }
 
@@ -628,26 +750,23 @@ public class SessionReader {
         process(session, elements, additionalInformation);
     }
 
+    /**
+     * A counter to generate unique panel names.  Needed for backward-compatibility of old session files.
+     */
+    private int panelCounter = 1;
+
     private void processPanel(Session session, Element element, HashMap additionalInformation) {
         panelElementPresent = true;
-        String nodeName = element.getNodeName();
-        String panelName = nodeName;
-        int height = 0;
-        int width = 0;
-        if (nodeName.equalsIgnoreCase(SessionElement.PANEL.getText())) {
-            panelName = element.getAttribute("name");
-            try {
-                height = Integer.valueOf(element.getAttribute("height"));
-                width = Integer.valueOf(element.getAttribute("width"));
-            } catch (NumberFormatException nfe) {
-            }
+        String panelName = element.getAttribute("name");
+        if (panelName == null) {
+            panelName = "Panel" + panelCounter++;
         }
 
         List<Track> panelTracks = new ArrayList();
         NodeList elements = element.getChildNodes();
         for (int i = 0; i < elements.getLength(); i++) {
             Node childNode = elements.item(i);
-            if (childNode.getNodeName().equalsIgnoreCase(SessionElement.DATA_TRACK.getText()) ||
+            if (childNode.getNodeName().equalsIgnoreCase(SessionElement.DATA_TRACK.getText()) ||  // Is this a track?
                     childNode.getNodeName().equalsIgnoreCase(SessionElement.TRACK.getText())) {
 
                 List<Track> tracks = processTrack(session, (Element) childNode, additionalInformation);
@@ -659,17 +778,36 @@ public class SessionReader {
             }
         }
 
-        TrackPanel panel = IGVMainFrame.getInstance().getDataPanel(panelName);
-        if (height == 0) {
-            height = panel.getPreferredPanelHeight();
-        }
-        if (width == 0) {
-            width = panel.getViewportWidth();
-        }
-        panel.setHeight(height);
+        TrackPanel panel = IGV.getInstance().getDataPanel(panelName);
         panel.addTracks(panelTracks);
     }
 
+    private void processPanelLayout(Session session, Element element, HashMap additionalInformation) {
+
+        String nodeName = element.getNodeName();
+        String panelName = nodeName;
+
+        NamedNodeMap tNodeMap = element.getAttributes();
+        for (int i = 0; i < tNodeMap.getLength(); i++) {
+            Node node = tNodeMap.item(i);
+            String name = node.getNodeName();
+            if (name.equals("dividerFractions")) {
+                String value = node.getNodeValue();
+                String[] tokens = value.split(",");
+                double[] divs = new double[tokens.length];
+                try {
+                    for (int j = 0; j < tokens.length; j++) {
+                        divs[j] = Double.parseDouble(tokens[j]);
+                    }
+                    session.setDividerFractions(divs);
+                }
+                catch (NumberFormatException e) {
+                    log.error("Error parsing divider locations", e);
+                }
+            }
+        }
+    }
+
 
     /**
      * Process a track element.  This should return a single track, but could return multiple tracks since the
@@ -687,7 +825,7 @@ public class SessionReader {
 
         // TODo -- put in utility method, extacts attributes from element **Definitely need to do this
         HashMap<String, String> tAttributes = new HashMap();
-        HashMap<String, String> drAttributes = new HashMap();
+        HashMap<String, String> drAttributes = null;
 
         NamedNodeMap tNodeMap = element.getAttributes();
         for (int i = 0; i < tNodeMap.getLength(); i++) {
@@ -700,15 +838,18 @@ public class SessionReader {
 
 
         if (element.hasChildNodes()) {
+            drAttributes = new HashMap();
             Node childNode = element.getFirstChild();
             Node sibNode = childNode.getNextSibling();
-            NamedNodeMap drNodeMap = sibNode.getAttributes();
-
-            for (int i = 0; i < drNodeMap.getLength(); i++) {
-                Node node = drNodeMap.item(i);
-                String value = node.getNodeValue();
-                if (value != null && value.length() > 0) {
-                    drAttributes.put(node.getNodeName(), value);
+            String sibName = sibNode.getNodeName();
+            if (sibName.equals(SessionElement.DATA_RANGE.getText())) {
+                NamedNodeMap drNodeMap = sibNode.getAttributes();
+                for (int i = 0; i < drNodeMap.getLength(); i++) {
+                    Node node = drNodeMap.item(i);
+                    String value = node.getNodeValue();
+                    if (value != null && value.length() > 0) {
+                        drAttributes.put(node.getNodeName(), value);
+                    }
                 }
             }
         }
@@ -720,12 +861,22 @@ public class SessionReader {
             log.info("Warning.  No tracks were found with id: " + id + " in session file");
         } else {
             for (final Track track : matchedTracks) {
+
+                // Special case for sequence & gene tracks,  they need to be removed before being placed.
+                if (version >= 4 && track == geneTrack || track == seqTrack) {
+                    IGV.getInstance().getTrackManager().removeTracks(Arrays.asList(track));
+                }
+
                 track.restorePersistentState(tAttributes);
-                DataRange dr = track.getDataRange();
-                dr.restorePersistentState(drAttributes);
-                track.setDataRange(dr);
+                if (drAttributes != null) {
+                    DataRange dr = track.getDataRange();
+                    dr.restorePersistentState(drAttributes);
+                    track.setDataRange(dr);
+                }
             }
             trackDictionary.remove(id);
+
+
         }
 
         NodeList elements = element.getChildNodes();
@@ -754,19 +905,17 @@ public class SessionReader {
     private void processPreferences(Session session, Element element, HashMap additionalInformation) {
 
         NodeList elements = element.getChildNodes();
-        process(session, elements, additionalInformation);
+        for (int i = 0; i < elements.getLength(); i++) {
+            Node child = elements.item(i);
+            if (child.getNodeName().equalsIgnoreCase(SessionElement.PROPERTY.getText())) {
+                Element childNode = (Element) child;
+                String name = getAttribute(childNode, SessionAttribute.NAME.getText());
+                String value = getAttribute(childNode, SessionAttribute.VALUE.getText());
+                session.setPreference(name, value);
+            }
+        }
     }
 
-    private void processPreference(Session session, Element element, HashMap additionalInformation) {
-
-        String name = getAttribute(element, SessionAttribute.NAME.getText());
-        String value = getAttribute(element, SessionAttribute.VALUE.getText());
-
-        session.setPreference(name, value);
-
-        NodeList elements = element.getChildNodes();
-        process(session, elements, additionalInformation);
-    }
 
     /**
      * Process a list of session element nodes.
