@@ -1,28 +1,29 @@
 /*
- * Copyright (c) 2007-2010 by The Broad Institute, Inc. and the Massachusetts Institute of Technology.
- * All Rights Reserved.
+ * Copyright (c) 2007-2011 by The Broad Institute of MIT and Harvard.  All Rights Reserved.
  *
- * This software is licensed under the terms of the GNU Lesser General Public License (LGPL), Version 2.1 which
- * is available at http://www.opensource.org/licenses/lgpl-2.1.php.
+ * This software is licensed under the terms of the GNU Lesser General Public License (LGPL),
+ * Version 2.1 which is available at http://www.opensource.org/licenses/lgpl-2.1.php.
  *
- * THE SOFTWARE IS PROVIDED "AS IS." THE BROAD AND MIT MAKE NO REPRESENTATIONS OR WARRANTIES OF
- * ANY KIND CONCERNING THE SOFTWARE, EXPRESS OR IMPLIED, INCLUDING, WITHOUT LIMITATION, WARRANTIES
- * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NONINFRINGEMENT, OR THE ABSENCE OF LATENT
- * OR OTHER DEFECTS, WHETHER OR NOT DISCOVERABLE.  IN NO EVENT SHALL THE BROAD OR MIT, OR THEIR
- * RESPECTIVE TRUSTEES, DIRECTORS, OFFICERS, EMPLOYEES, AND AFFILIATES BE LIABLE FOR ANY DAMAGES OF
- * ANY KIND, INCLUDING, WITHOUT LIMITATION, INCIDENTAL OR CONSEQUENTIAL DAMAGES, ECONOMIC
- * DAMAGES OR INJURY TO PROPERTY AND LOST PROFITS, REGARDLESS OF WHETHER THE BROAD OR MIT SHALL
- * BE ADVISED, SHALL HAVE OTHER REASON TO KNOW, OR IN FACT SHALL KNOW OF THE POSSIBILITY OF THE
- * FOREGOING.
+ * THE SOFTWARE IS PROVIDED "AS IS." THE BROAD AND MIT MAKE NO REPRESENTATIONS OR
+ * WARRANTES OF ANY KIND CONCERNING THE SOFTWARE, EXPRESS OR IMPLIED, INCLUDING,
+ * WITHOUT LIMITATION, WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
+ * PURPOSE, NONINFRINGEMENT, OR THE ABSENCE OF LATENT OR OTHER DEFECTS, WHETHER
+ * OR NOT DISCOVERABLE.  IN NO EVENT SHALL THE BROAD OR MIT, OR THEIR RESPECTIVE
+ * TRUSTEES, DIRECTORS, OFFICERS, EMPLOYEES, AND AFFILIATES BE LIABLE FOR ANY DAMAGES
+ * OF ANY KIND, INCLUDING, WITHOUT LIMITATION, INCIDENTAL OR CONSEQUENTIAL DAMAGES,
+ * ECONOMIC DAMAGES OR INJURY TO PROPERTY AND LOST PROFITS, REGARDLESS OF WHETHER
+ * THE BROAD OR MIT SHALL BE ADVISED, SHALL HAVE OTHER REASON TO KNOW, OR IN FACT
+ * SHALL KNOW OF THE POSSIBILITY OF THE FOREGOING.
  */
 package org.broad.igv.tools.parsers;
 
 //~--- non-JDK imports --------------------------------------------------------
 
-import org.broad.igv.feature.Genome;
+import org.broad.igv.exceptions.ParserException;
+import org.broad.igv.feature.genome.Genome;
 import org.broad.igv.track.TrackProperties;
 import org.broad.igv.track.TrackType;
-import org.broad.igv.util.AsciiLineReader;
+import org.broad.tribble.readers.AsciiLineReader;
 import org.broad.igv.util.ParsingUtils;
 import org.broad.igv.util.ResourceLocator;
 
@@ -40,7 +41,7 @@ public class WiggleParser {
 
     private enum Type {
 
-        fixedStep, variableStep, bed
+        fixedStep, variableStep, bed, cpg
     }
 
     private DataConsumer dataConsumer;
@@ -108,6 +109,10 @@ public class WiggleParser {
         }
 
 
+        if (locator.getPath().endsWith("CpG.txt")) {
+            type = Type.cpg;
+        }
+
     }
 
     /**
@@ -165,7 +170,7 @@ public class WiggleParser {
 
                 // Skip comment lines
                 if (nextLine.startsWith("#") || nextLine.startsWith("data") || nextLine.startsWith(
-                        "browser")) {
+                        "browser")  || nextLine.trim().length() == 0) {
                     continue;
                 }
 
@@ -188,7 +193,7 @@ public class WiggleParser {
     /**
      * @return
      */
-    public void parse() {
+    public void parse() throws IOException {
 
 
         String[] tokens = new String[10];
@@ -208,11 +213,10 @@ public class WiggleParser {
             reader = ParsingUtils.openAsciiReader(resourceLocator);
 
             int position = -1;
-            while ((nextLine = reader.readLine()) != null && (nextLine.trim().length() > 0)) {
+            while ((nextLine = reader.readLine()) != null) {
 
                 // Skip comment lines
-                if (nextLine.startsWith("#") || nextLine.startsWith("data") || nextLine.startsWith(
-                        "browser")) {
+                if (nextLine.startsWith("#") || nextLine.startsWith("data") || nextLine.startsWith("browser") || nextLine.trim().length() == 0) {
                     continue;
                 }
 
@@ -252,7 +256,44 @@ public class WiggleParser {
                         continue;
                     }
                     try {
-                        if (type.equals(Type.bed)) {
+                        if (type.equals(Type.cpg)) {
+                            if (nTokens > 3) {
+
+                                // TODO -- validation on the data array length (# of data columns).
+                                if (dataArray == null) {
+                                    dataArray = new float[nTokens - 3];
+                                }
+
+                                chr = tokens[1].trim();
+                                if (!chr.equals(lastChr)) {
+                                    newChromosome();
+                                }
+                                lastChr = chr;
+
+                                int endPosition = -1;
+                                try {
+                                    endPosition = Integer.parseInt(tokens[2].trim());
+                                } catch (NumberFormatException numberFormatException) {
+                                    System.err.println("Column 2  is not a number");
+
+                                    throw new RuntimeException("Column 2 must be numeric." + " Found: " + tokens[1]);
+                                }
+                                int startPosition = endPosition - 1;
+
+                                if (startPosition < lastPosition) {
+                                    unsortedChromosomes.add(chr);
+                                }
+                                lastPosition = startPosition;
+
+
+                                float value = Float.parseFloat(tokens[4].trim());
+                                if (tokens[3].trim().equals("R")) {
+                                    value = -value;
+                                }
+                                dataArray[0] = value;
+                                getDataConsumer().addData(chr, startPosition, endPosition, dataArray, null);
+                            }
+                        } else if (type.equals(Type.bed)) {
                             if (nTokens > 3) {
 
                                 // TODO -- validation on the data array length (# of data columns).
@@ -336,8 +377,7 @@ public class WiggleParser {
 
             parsingComplete();
 
-        } catch (IOException e) {
-            e.printStackTrace();
+
         } finally {
             if (reader != null) {
                 reader.close();
@@ -346,6 +386,7 @@ public class WiggleParser {
     }
 
     // fixedStep chrom=chrM strt=1 step=1
+
     private void parseStepLine(String header) {
         String[] tokens = header.split("\\s+");
         for (String token : tokens) {
