@@ -1,54 +1,51 @@
 /*
- * Copyright (c) 2007-2010 by The Broad Institute, Inc. and the Massachusetts Institute of Technology.
- * All Rights Reserved.
+ * Copyright (c) 2007-2011 by The Broad Institute of MIT and Harvard.  All Rights Reserved.
  *
- * This software is licensed under the terms of the GNU Lesser General Public License (LGPL), Version 2.1 which
- * is available at http://www.opensource.org/licenses/lgpl-2.1.php.
+ * This software is licensed under the terms of the GNU Lesser General Public License (LGPL),
+ * Version 2.1 which is available at http://www.opensource.org/licenses/lgpl-2.1.php.
  *
- * THE SOFTWARE IS PROVIDED "AS IS." THE BROAD AND MIT MAKE NO REPRESENTATIONS OR WARRANTIES OF
- * ANY KIND CONCERNING THE SOFTWARE, EXPRESS OR IMPLIED, INCLUDING, WITHOUT LIMITATION, WARRANTIES
- * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NONINFRINGEMENT, OR THE ABSENCE OF LATENT
- * OR OTHER DEFECTS, WHETHER OR NOT DISCOVERABLE.  IN NO EVENT SHALL THE BROAD OR MIT, OR THEIR
- * RESPECTIVE TRUSTEES, DIRECTORS, OFFICERS, EMPLOYEES, AND AFFILIATES BE LIABLE FOR ANY DAMAGES OF
- * ANY KIND, INCLUDING, WITHOUT LIMITATION, INCIDENTAL OR CONSEQUENTIAL DAMAGES, ECONOMIC
- * DAMAGES OR INJURY TO PROPERTY AND LOST PROFITS, REGARDLESS OF WHETHER THE BROAD OR MIT SHALL
- * BE ADVISED, SHALL HAVE OTHER REASON TO KNOW, OR IN FACT SHALL KNOW OF THE POSSIBILITY OF THE
- * FOREGOING.
+ * THE SOFTWARE IS PROVIDED "AS IS." THE BROAD AND MIT MAKE NO REPRESENTATIONS OR
+ * WARRANTES OF ANY KIND CONCERNING THE SOFTWARE, EXPRESS OR IMPLIED, INCLUDING,
+ * WITHOUT LIMITATION, WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
+ * PURPOSE, NONINFRINGEMENT, OR THE ABSENCE OF LATENT OR OTHER DEFECTS, WHETHER
+ * OR NOT DISCOVERABLE.  IN NO EVENT SHALL THE BROAD OR MIT, OR THEIR RESPECTIVE
+ * TRUSTEES, DIRECTORS, OFFICERS, EMPLOYEES, AND AFFILIATES BE LIABLE FOR ANY DAMAGES
+ * OF ANY KIND, INCLUDING, WITHOUT LIMITATION, INCIDENTAL OR CONSEQUENTIAL DAMAGES,
+ * ECONOMIC DAMAGES OR INJURY TO PROPERTY AND LOST PROFITS, REGARDLESS OF WHETHER
+ * THE BROAD OR MIT SHALL BE ADVISED, SHALL HAVE OTHER REASON TO KNOW, OR IN FACT
+ * SHALL KNOW OF THE POSSIBILITY OF THE FOREGOING.
  */
 package org.broad.igv.track;
 
 //~--- non-JDK imports --------------------------------------------------------
 
-import net.sf.samtools.util.BlockCompressedInputStream;
 import org.apache.log4j.Logger;
 import org.broad.igv.PreferenceManager;
 import org.broad.igv.exceptions.DataLoadException;
-import org.broad.igv.feature.Feature;
-import org.broad.igv.feature.GeneManager;
-import org.broad.igv.feature.Genome;
-import org.broad.igv.feature.GenomeManager;
-import org.broad.igv.renderer.BasicFeatureRenderer;
+import org.broad.igv.feature.*;
+import org.broad.igv.feature.genome.Genome;
+import org.broad.igv.feature.genome.GenomeDescriptor;
+import org.broad.igv.ui.IGV;
+import org.broad.igv.ui.panel.ReferenceFrame;
+import org.broad.igv.renderer.IGVFeatureRenderer;
 import org.broad.igv.sam.AlignmentTrack;
-import org.broad.igv.sam.CoverageTrack;
-import org.broad.igv.session.ViewContext;
-import org.broad.igv.ui.IGVMainFrame;
 import org.broad.igv.ui.MessageCollection;
-import org.broad.igv.ui.RegionOfInterest;
-import org.broad.igv.ui.UIConstants;
-import org.broad.igv.ui.panel.DragEventManager;
-import org.broad.igv.ui.panel.DragListener;
-import org.broad.igv.ui.panel.TrackPanel;
-import org.broad.igv.ui.panel.TrackPanelScrollPane;
+import org.broad.igv.ui.panel.*;
 import org.broad.igv.ui.util.MessageUtils;
+import org.broad.igv.util.LongRunningTask;
+import org.broad.igv.util.NamedRunnable;
 import org.broad.igv.util.ResourceLocator;
+import org.broad.igv.vcf.VCFTrack;
+import org.broad.tribble.readers.AsciiLineReader;
 
 import java.awt.*;
+import java.io.BufferedInputStream;
 import java.io.File;
 import java.io.IOException;
 import java.io.InputStream;
-import java.net.URL;
 import java.util.*;
 import java.util.List;
+import java.util.zip.GZIPInputStream;
 
 
 /**
@@ -62,7 +59,7 @@ public class TrackManager {
     /**
      * Owning object
      */
-    IGVMainFrame mainFrame;
+    IGV igv;
 
     private TrackLoader loader;
 
@@ -78,27 +75,45 @@ public class TrackManager {
      */
     private Track geneTrack;
 
+    /**
+     * The sequence track for the current genome
+     */
+    private SequenceTrack sequenceTrack;
+
     private Map<String, List<Track>> overlayTracksMap = new HashMap();
+    private Set<Track> overlaidTracks = new HashSet();
 
-    private Set<TrackType> loadedTypes = new HashSet();
     public static final String DATA_PANEL_NAME = "DataPanel";
     public static final String FEATURE_PANEL_NAME = "FeaturePanel";
 
+
+    public TrackManager(IGV igv) {
+
+        this.igv = igv;
+        loader = new TrackLoader();
+    }
+
     public Track getGeneTrack() {
         return geneTrack;
     }
 
+
+    public SequenceTrack getSequenceTrack() {
+        return sequenceTrack;
+    }
+
     public Set<TrackType> getLoadedTypes() {
-        return loadedTypes;
+        Set<TrackType> types = new HashSet();
+        for (Track t : this.getAllTracks(false)) {
+            TrackType type = t.getTrackType();
+            if (t != null) {
+                types.add(type);
+            }
+        }
+        return types;
     }
 
 
-    public TrackManager(IGVMainFrame mainFrame) {
-
-        this.mainFrame = mainFrame;
-        loader = new TrackLoader();
-    }
-
     public void putScrollPane(String name, TrackPanelScrollPane sp) {
         trackPanelScrollPanes.put(name, sp);
     }
@@ -120,14 +135,12 @@ public class TrackManager {
     }
 
 
-    /**
-     * Method description
-     *
-     * @param attributeName
-     */
     public void setGroupByAttribute(String attributeName) {
         groupByAttribute = attributeName;
         groupTracksByAttribute();
+        // propagate change so VCFTracks and others can reoder their samples internally according to
+        // groupByAttribute as well:
+        refreshData();
     }
 
     public void reset() {
@@ -140,17 +153,17 @@ public class TrackManager {
 
 
     public void groupTracksByAttribute() {
-        //for (TrackPanelScrollPane tsp : getTrackPanelScrollPanes()) {
-        //    TrackPanel trackPanel = tsp.getTrackPanel();
-        //    trackPanel.groupTracksByAttribute(groupByAttribute);
-        //}
-        TrackPanelScrollPane tsp = trackPanelScrollPanes.get(DATA_PANEL_NAME);
-        if (tsp != null) {
-            tsp.getTrackPanel().groupTracksByAttribute(groupByAttribute);
+        for (TrackPanelScrollPane tsp : getTrackPanelScrollPanes()) {
+            TrackPanel trackPanel = tsp.getTrackPanel();
+            trackPanel.groupTracksByAttribute(groupByAttribute);
         }
+        //TODO We should go through all current panels not just the data panel
+        //TrackPanelScrollPane tsp = trackPanelScrollPanes.get(DATA_PANEL_NAME);
+        //if (tsp != null) {
+        //    tsp.getTrackPanel().groupTracksByAttribute(groupByAttribute);
+        //}
     }
 
-
     /**
      * A (hopefully) temporary solution to force SAM track reloads,  until we have a better
      * dependency scheme
@@ -158,15 +171,29 @@ public class TrackManager {
     public void reloadSAMTracks() {
         for (Track t : getAllTracks(false)) {
             if (t instanceof AlignmentTrack) {
-                ((AlignmentTrack) t).reloadData();
+                ((AlignmentTrack) t).clearCaches();
             }
         }
+        IGV.getInstance().repaintDataPanels();
     }
 
+
     public void sortAlignmentTracks(AlignmentTrack.SortOption option) {
         for (Track t : getAllTracks(false)) {
             if (t instanceof AlignmentTrack) {
-                ((AlignmentTrack) t).sortRows(option);
+                for (ReferenceFrame frame : FrameManager.getFrames()) {
+                    ((AlignmentTrack) t).sortRows(option, frame);
+                }
+            }
+        }
+    }
+
+    public void sortAlignmentTracks(AlignmentTrack.SortOption option, double location) {
+        for (Track t : getAllTracks(false)) {
+            if (t instanceof AlignmentTrack) {
+                for (ReferenceFrame frame : FrameManager.getFrames()) {
+                    ((AlignmentTrack) t).sortRows(option, frame, location);
+                }
             }
         }
     }
@@ -174,28 +201,30 @@ public class TrackManager {
     public void packAlignmentTracks() {
         for (Track t : getAllTracks(false)) {
             if (t instanceof AlignmentTrack) {
-                ((AlignmentTrack) t).packAlignments();
+                for (ReferenceFrame frame : FrameManager.getFrames()) {
+                    ((AlignmentTrack) t).packAlignments(frame);
+                }
             }
         }
     }
 
     public void collapseTracks() {
         for (Track t : getAllTracks(true)) {
-            t.setExpanded(false);
+            t.setDisplayMode(Track.DisplayMode.COLLAPSED);
         }
     }
 
 
     public void expandTracks() {
         for (Track t : getAllTracks(true)) {
-            t.setExpanded(true);
+            t.setDisplayMode(Track.DisplayMode.EXPANDED);
         }
     }
 
     public void collapseTrack(String trackName) {
         for (Track t : getAllTracks(true)) {
             if (t.getName().equals(trackName)) {
-                t.setExpanded(false);
+                t.setDisplayMode(Track.DisplayMode.COLLAPSED);
             }
         }
     }
@@ -204,15 +233,15 @@ public class TrackManager {
     public void expandTrack(String trackName) {
         for (Track t : getAllTracks(true)) {
             if (t.getName().equals(trackName)) {
-                t.setExpanded(true);
+                t.setDisplayMode(Track.DisplayMode.EXPANDED);
             }
         }
     }
 
 
-    public boolean loadResources(Collection<ResourceLocator> locators) {
+    public void loadResources(Collection<ResourceLocator> locators) {
 
-        boolean tracksWereLoaded = false;
+        //Set<TrackPanel> changedPanels = new HashSet();
 
         log.info("Loading" + locators.size() + " resources.");
         MessageCollection messages = new MessageCollection();
@@ -220,20 +249,34 @@ public class TrackManager {
 
         for (ResourceLocator locator : locators) {
 
+            // If its a local file, check explicitly for existence (rather than rely on exception)
             if (locator.isLocal()) {
                 File trackSetFile = new File(locator.getPath());
-
                 if (!trackSetFile.exists()) {
                     messages.append("File not found: " + locator.getPath() + "\n");
                     continue;
                 }
             }
-            TrackPanel group = getPanelFor(locator);
+
             try {
                 List<Track> tracks = load(locator);
-                group.addTracks(tracks);
-                tracksWereLoaded = tracks.size() > 0;
-            } catch (Exception e) {
+                if (tracks.size() > 0) {
+                    String path = locator.getPath();
+
+                    // Get an appropriate panel.  If its a VCF file create a new panel if the number of genotypes
+                    // is greater than 10
+                    TrackPanel panel = getPanelFor(locator);
+                    if (path.endsWith(".vcf") || path.endsWith(".vcf.gz") ||
+                            path.endsWith(".vcf4") || path.endsWith(".vcf4.gz")) {
+                        Track t = tracks.get(0);
+                        if (t instanceof VCFTrack && ((VCFTrack) t).getAllSamples().size() > 10) {
+                            String newPanelName = "Panel" + System.currentTimeMillis();
+                            panel = igv.addDataPanel(newPanelName).getTrackPanel();
+                        }
+                    }
+                    panel.addTracks(tracks);
+                }
+            } catch (Throwable e) {
                 log.error("Error loading tracks", e);
                 messages.append(e.getMessage());
             }
@@ -250,7 +293,6 @@ public class TrackManager {
                 MessageUtils.showMessage(message);
             }
         }
-        return tracksWereLoaded;
     }
 
     /**
@@ -260,20 +302,24 @@ public class TrackManager {
     public List<Track> load(ResourceLocator locator) {
 
         try {
-            List<Track> newTracks = loader.load(locator);
+            List<Track> newTracks = loader.load(locator, igv);
             if (newTracks.size() > 0) {
                 for (Track track : newTracks) {
-                    track.setSourceFile(locator.getPath());
-                    if (locator.getUrl() != null) {
-                        track.setUrl(locator.getUrl());
+                    String fn = locator.getPath();
+                    int lastSlashIdx = fn.lastIndexOf("/");
+                    if (lastSlashIdx < 0) {
+                        lastSlashIdx = fn.lastIndexOf("\\");
+                    }
+                    if (lastSlashIdx > 0) {
+                        fn = fn.substring(lastSlashIdx);
                     }
                     track.setAttributeValue("NAME", track.getName());
-                    track.setAttributeValue("DATA FILE", locator.getPath());
+                    track.setAttributeValue("DATA FILE", fn);
                     track.setAttributeValue("DATA TYPE", track.getTrackType().toString());
 
 
                 }
-            } 
+            }
             return newTracks;
 
         } catch (DataLoadException dle) {
@@ -285,12 +331,6 @@ public class TrackManager {
 
     }
 
-    /**
-     * @param type
-     */
-    public void addLoadedType(TrackType type) {
-        loadedTypes.add(type);
-    }
 
     /**
      * Load the data file into the specified panel.   Triggered via drag and drop.
@@ -303,6 +343,7 @@ public class TrackManager {
         ResourceLocator locator = new ResourceLocator(file.getAbsolutePath());
         List<Track> tracks = load(locator);
         panel.addTracks(tracks);
+        IGV.getInstance().doRefresh();
     }
 
     /**
@@ -310,19 +351,17 @@ public class TrackManager {
      * type is Mutation.  This method finds all mutation tracks and builds a map
      * of key -> mutatinon track,  where the key is the specified attribute value
      * for linking tracks for overlay.
-     * <p/>
-     * The method also resets all tracks overlay properties from the user
-     * user preference value.  That should probably be done in another method.
      */
     public void resetOverlayTracks() {
         overlayTracksMap.clear();
+        overlaidTracks.clear();
 
-        if (PreferenceManager.getInstance().getOverlayTracks()) {
-            String overlayAttribute = IGVMainFrame.getInstance().getSession().getOverlayAttribute();
+        if (PreferenceManager.getInstance().getAsBoolean(PreferenceManager.OVERLAY_MUTATION_TRACKS)) {
+            String overlayAttribute = IGV.getInstance().getSession().getOverlayAttribute();
             if (overlayAttribute != null) {
                 for (Track track : getAllTracks(false)) {
-                    if (track != null) {
-                        if (track.getTrackType() == UIConstants.overlayTrackType) {
+                    if (track != null) {  // <= this should not be neccessary
+                        if (track.getTrackType() == TrackType.MUTATION) {
                             String value = track.getAttributeValue(overlayAttribute);
 
                             if (value != null) {
@@ -339,20 +378,29 @@ public class TrackManager {
                     }
                 }
             }
-        }
-
-        boolean displayOverlays = IGVMainFrame.getInstance().getSession().getDisplayOverlayTracks();
-
-        for (Track track : getAllTracks(false)) {
-            if (track != null) {
-                if (track.getTrackType() == UIConstants.overlayTrackType) {
-                    track.setOverlayVisible(displayOverlays);
+            for (Track track : getAllTracks(false)) {
+                if (track != null) {  // <= this should not be neccessary
+                    if (track.getTrackType() != TrackType.MUTATION) {
+                        List<Track> trackList = getOverlayTracks(track);
+                        if (trackList != null) overlaidTracks.addAll(trackList);
+                    }
                 }
             }
 
+            boolean displayOverlays = IGV.getInstance().getSession().getOverlayMutationTracks();
+            for (Track track : getAllTracks(false)) {
+                if (track != null) {
+                    if (track.getTrackType() == TrackType.MUTATION) {
+                        track.setOverlayed(displayOverlays  && overlaidTracks.contains(track));
+                    }
+                }
+            }
         }
     }
 
+    public Set<Track> getOverlaidTracks() {
+        return overlaidTracks;
+    }
 
     /**
      * Method description
@@ -369,12 +417,6 @@ public class TrackManager {
         return count;
     }
 
-    /**
-     * Method description
-     *
-     * @param includeGeneTrack
-     * @return
-     */
     public List<Track> getAllTracks(boolean includeGeneTrack) {
         List<Track> allTracks = new ArrayList<Track>();
 
@@ -385,13 +427,17 @@ public class TrackManager {
         if ((geneTrack != null) && !includeGeneTrack) {
             allTracks.remove(geneTrack);
         }
+        if ((sequenceTrack != null) && !includeGeneTrack) {
+            allTracks.remove(sequenceTrack);
+        }
 
         return allTracks;
     }
 
     public void clearSelections() {
         for (Track t : getAllTracks(true)) {
-            t.setSelected(false);
+            if (t != null)
+                t.setSelected(false);
         }
 
     }
@@ -435,7 +481,7 @@ public class TrackManager {
     public Collection<Track> getSelectedTracks() {
         HashSet<Track> selectedTracks = new HashSet();
         for (Track t : getAllTracks(true)) {
-            if (t.isSelected()) {
+            if (t != null && t.isSelected()) {
                 selectedTracks.add(t);
             }
         }
@@ -475,19 +521,6 @@ public class TrackManager {
 
     }
 
-
-    public void unloadTracks(Collection<ResourceLocator> locators) {
-
-        Collection<Track> tracksToRemove = new ArrayList();
-        HashSet<ResourceLocator> locatorSet = new HashSet(locators);
-        for (Track t : getAllTracks(true)) {
-            if (locatorSet.contains(t.getResourceLocator())) {
-                tracksToRemove.add(t);
-            }
-        }
-        removeTracks(tracksToRemove);
-    }
-
     /**
      * Method description
      *
@@ -495,26 +528,18 @@ public class TrackManager {
      */
     public void removeTracks(Collection<Track> tracksToRemove) {
 
-        Set<Track> tmp = new HashSet<Track>(tracksToRemove);
-        // TODO -- replace with track.getDependentTracks(), or similar
+        // Make copy of list as we will be modifying the original in the loop
+        ArrayList<TrackPanelScrollPane> panes = new ArrayList(getTrackPanelScrollPanes());
+        for (TrackPanelScrollPane tsp : panes) {
+            TrackPanel trackPanel = tsp.getTrackPanel();
+            trackPanel.removeTracks(tracksToRemove);
+
+            if (!trackPanel.hasTracks()) {
+                igv.removeDataPanel(tsp.getTrackPanelName());
+            }
+        }
+
         for (Track t : tracksToRemove) {
-            if (t instanceof AlignmentTrack) {
-                CoverageTrack ct = ((AlignmentTrack) t).getCoverageTrack();
-                tmp.add(ct);
-            }
-        }
-
-        for (TrackPanelScrollPane tsp : getTrackPanelScrollPanes()) {
-            TrackPanel tsv = tsp.getTrackPanel();
-            tsv.removeTracks(tmp);
-
-
-            if (!tsv.hasTracks()) {
-                mainFrame.removeDataPanel(tsp.getName());
-            }
-        }
-
-        for (Track t : tmp) {
             if (t instanceof DragListener) {
                 DragEventManager.getInstance().removeDragListener((DragListener) t);
             }
@@ -522,6 +547,18 @@ public class TrackManager {
         }
     }
 
+    /**
+     * Refresh the data in all tracks that respond to this message (note: most do not,  it was added early
+     * on for copy number data and is kept for backward compatibility).
+     */
+    public void refreshData() {
+
+        long t0 = System.currentTimeMillis();
+        for (Track track : getAllTracks(false)) {
+            track.refreshData(t0);
+        }
+    }
+
     /**
      * Sort all groups (data and feature) by attribute value(s).  Tracks are
      * sorted within groups.
@@ -538,23 +575,26 @@ public class TrackManager {
         }
     }
 
-    public void sortAllTracksByPosition(List<String> trackIds) {
-        for (TrackPanelScrollPane tsp : getTrackPanelScrollPanes()) {
-            TrackPanel tsv = tsp.getTrackPanel();
-            tsv.sortTracksByPosition(trackIds);
-        }
-    }
-
+    /**
+     * Return a DataPanel appropriate for the resource type
+     *
+     * @param locator
+     * @return
+     */
     public TrackPanel getPanelFor(ResourceLocator locator) {
         String path = locator.getPath().toLowerCase();
-        if (PreferenceManager.getInstance().isShowSingleTrackPane()) {
-            return mainFrame.getDataPanel(DATA_PANEL_NAME);
+        if (PreferenceManager.getInstance().getAsBoolean(PreferenceManager.SHOW_SINGLE_TRACK_PANE_KEY)) {
+            return igv.getDataPanel(DATA_PANEL_NAME);
         } else if (path.endsWith(".sam") || path.endsWith(".bam") ||
                 path.endsWith(".sam.list") || path.endsWith(".bam.list") ||
                 path.endsWith(".aligned") || path.endsWith(".sorted.txt")) {
 
             String newPanelName = "Panel" + System.currentTimeMillis();
-            return mainFrame.addDataPanel(newPanelName).getTrackPanel();
+            return igv.addDataPanel(newPanelName).getTrackPanel();
+            //} else if (path.endsWith(".vcf") || path.endsWith(".vcf.gz") ||
+            //        path.endsWith(".vcf4") || path.endsWith(".vcf4.gz")) {
+            //    String newPanelName = "Panel" + System.currentTimeMillis();
+            //    return igv.addDataPanel(newPanelName).getTrackPanel();
         } else {
             return getDefaultPanel(locator);
         }
@@ -564,7 +604,7 @@ public class TrackManager {
     private TrackPanel getDefaultPanel(ResourceLocator locator) {
 
         if (locator.getType() != null && locator.getType().equalsIgnoreCase("das")) {
-            return mainFrame.getDataPanel(FEATURE_PANEL_NAME);
+            return igv.getDataPanel(FEATURE_PANEL_NAME);
         }
 
         String filename = locator.getPath().toLowerCase();
@@ -583,9 +623,9 @@ public class TrackManager {
                 filename.endsWith("bed") || filename.endsWith("gistic") ||
                 filename.endsWith("bedz") || filename.endsWith("repmask") ||
                 filename.contains("dranger")) {
-            return mainFrame.getDataPanel(FEATURE_PANEL_NAME);
+            return igv.getDataPanel(FEATURE_PANEL_NAME);
         } else {
-            return mainFrame.getDataPanel(DATA_PANEL_NAME);
+            return igv.getDataPanel(DATA_PANEL_NAME);
         }
     }
 
@@ -598,21 +638,27 @@ public class TrackManager {
      * @param type
      */
     public void sortByRegionScore(RegionOfInterest region,
-                                  RegionScoreType type) {
+                                  final RegionScoreType type,
+                                  final ReferenceFrame frame) {
 
-        if (region == null) {
-            ViewContext vc = ViewContext.getInstance();
-            String chr = vc.getChrName();
-            int start = (int) vc.getOrigin();
-            int end = (int) vc.getEnd() + 1;
-            region = new RegionOfInterest(chr, start, end, "");
+        final RegionOfInterest r = region == null ? new RegionOfInterest(frame.getChrName(), (int) frame.getOrigin(),
+                (int) frame.getEnd() + 1, frame.getName()) : region;
+        NamedRunnable runnable = new NamedRunnable() {
 
-        }
+            public String getName() {
+                return "Sort";
+            }
 
-        for (TrackPanelScrollPane tsp : getTrackPanelScrollPanes()) {
-            TrackPanel tsv = tsp.getTrackPanel();
-            tsv.sortByRegionsScore(region, type);
-        }
+            public void run() {
+                for (TrackPanelScrollPane tsp : getTrackPanelScrollPanes()) {
+                    TrackPanel tsv = tsp.getTrackPanel();
+                    tsv.sortByRegionsScore(r, type, frame);
+                }
+                IGV.getMainFrame().repaint();
+            }
+        };
+
+        LongRunningTask.submit(runnable);
 
     }
 
@@ -633,7 +679,7 @@ public class TrackManager {
      * @return
      */
     public List<Track> getOverlayTracks(Track track) {
-        String overlayAttribute = IGVMainFrame.getInstance().getSession().getOverlayAttribute();
+        String overlayAttribute = IGV.getInstance().getSession().getOverlayAttribute();
         String value = track.getAttributeValue(overlayAttribute);
         return overlayTracksMap.get(value);
 
@@ -642,47 +688,59 @@ public class TrackManager {
     /**
      * Method description
      *
-     * @param genomeId
+     * @param genome
      */
-    public void loadGeneTrack(String genomeId) {
+    public void createGeneTrack(Genome genome) {
 
-        // Load genes
-        GeneManager geneManager = GeneManager.getGeneManager(genomeId);
+        FeatureTrack geneFeatureTrack = null;
+        String genomeId = genome.getId();
+        GenomeDescriptor genomeDescriptor = IGV.getInstance().getGenomeManager().getGenomeDescriptor(genomeId);
+        AsciiLineReader reader = getGeneReader(genomeDescriptor);
+        if (reader != null) {
+            try {
 
+                String geneFilename = genomeDescriptor.getGeneFileName();
+                FeatureParser parser = AbstractFeatureParser.getInstanceFor(new ResourceLocator(geneFilename), genome);
+                if (parser == null) {
+                    MessageUtils.showMessage("ERROR: Unrecognized annotation file format: " + geneFilename +
+                            "<br>Annotations for genome: " + genomeId + " will not be loaded.");
+                } else {
+                    List<org.broad.tribble.Feature> genes = parser.loadFeatures(reader);
+                    String name = genomeDescriptor.getGeneTrackName();
+                    if (name == null) name = "Genes";
 
-        Map<String, List<Feature>> featureMap = null;
-        if (geneManager != null) {
-            featureMap = geneManager.getChromsomeGeneMap();
-        } else {
-            featureMap = new HashMap<String, List<Feature>>();
+                    String id = genome.getId() + "_genes";
+                    geneFeatureTrack = new FeatureTrack(id, name, new FeatureCollectionSource(genes, genome));
+                    geneFeatureTrack.setMinimumHeight(5);
+                    geneFeatureTrack.setHeight(35);
+                    geneFeatureTrack.setPreferredHeight(35);
+                    geneFeatureTrack.setRendererClass(IGVFeatureRenderer.class);
+                    geneFeatureTrack.setColor(Color.BLUE.darker());
+                    TrackProperties props = parser.getTrackProperties();
+                    if (props != null) {
+                        geneFeatureTrack.setProperties(parser.getTrackProperties());
+                    }
+                    geneFeatureTrack.setUrl(genome.getAnnotationURL());
+                }
+
+            } catch (Exception e) {
+                log.error("Error loading geneManager", e);
+            }
+            finally {
+
+                if (reader != null) {
+                    reader.close();
+                }
+
+            }
         }
 
-        FeatureTrack geneFeatureTrack = new FeatureTrack("Genes", new FeatureCollectionSource(featureMap));
-        geneFeatureTrack.setMinimumHeight(5);
-        geneFeatureTrack.setHeight(30);
-        geneFeatureTrack.setRendererClass(BasicFeatureRenderer.class);
-        geneFeatureTrack.setColor(Color.BLUE.darker());
 
-        SequenceTrack seqTrack = new SequenceTrack("Reference");
-        if (geneManager != null) {
-
-            if (geneManager.getTrackProperties() != null) {
-                geneFeatureTrack.setTrackProperties(geneManager.getTrackProperties());
-            }
-
-            String geneTrackName = geneManager.getGeneTrackName();
-
-            GeneTrack gt = new GeneTrack(geneFeatureTrack, seqTrack);
-            gt.setName(geneTrackName);
-
-            Genome genome = GenomeManager.getInstance().getGenome(genomeId);
-            if (genome != null && gt.getUrl() == null) {
-                gt.setUrl(genome.getAnnotationURL());
-            }
-
-            setGeneTrack(gt);
+        SequenceTrack seqTrack = new SequenceTrack("Reference sequence");
+        if (geneFeatureTrack != null) {
+            setGenomeTracks(geneFeatureTrack, seqTrack);
         } else {
-            setGeneTrack(seqTrack);
+            setGenomeTracks(null, seqTrack);
         }
 
     }
@@ -691,28 +749,69 @@ public class TrackManager {
      * Replace current gene track with new one.  This is called upon switching genomes
      *
      * @param newGeneTrack
+     * @param newSeqTrack
      */
-    private void setGeneTrack(Track newGeneTrack) {
+    private void setGenomeTracks(Track newGeneTrack, SequenceTrack newSeqTrack) {
+
+        boolean foundSeqTrack = false;
+        for (TrackPanelScrollPane tsp : getTrackPanelScrollPanes()) {
+            TrackPanel tsv = tsp.getTrackPanel();
+            foundSeqTrack = tsv.replaceTrack(sequenceTrack, newSeqTrack);
+            if (foundSeqTrack) {
+                break;
+            }
+        }
 
         boolean foundGeneTrack = false;
         for (TrackPanelScrollPane tsp : getTrackPanelScrollPanes()) {
             TrackPanel tsv = tsp.getTrackPanel();
-            foundGeneTrack = tsv.setGeneTrack(geneTrack, newGeneTrack);
+            foundGeneTrack = tsv.replaceTrack(geneTrack, newGeneTrack);
             if (foundGeneTrack) {
                 break;
             }
         }
 
-        if (!foundGeneTrack) {
-            if (PreferenceManager.getInstance().isShowSingleTrackPane()) {
-                mainFrame.getDataPanel(DATA_PANEL_NAME).addTrack(newGeneTrack);
-            } else {
-                mainFrame.getDataPanel(FEATURE_PANEL_NAME).addTrack(newGeneTrack);
-            }
+
+        if (!foundGeneTrack || !foundSeqTrack) {
+            TrackPanel panel = PreferenceManager.getInstance().getAsBoolean(PreferenceManager.SHOW_SINGLE_TRACK_PANE_KEY) ?
+                    igv.getDataPanel(DATA_PANEL_NAME) : igv.getDataPanel(FEATURE_PANEL_NAME);
+
+            if (!foundSeqTrack) panel.addTrack(newSeqTrack);
+            if (!foundGeneTrack && newGeneTrack != null) panel.addTrack(newGeneTrack);
+
         }
 
         // Keep a reference to this track so it can be removed
         geneTrack = newGeneTrack;
+        sequenceTrack = newSeqTrack;
+
+    }
+
+
+    public static AsciiLineReader getGeneReader(GenomeDescriptor genomeDescriptor) {
+
+        InputStream is = null;
+        try {
+
+            InputStream inputStream = genomeDescriptor.getGeneStream();
+            if (inputStream == null) {
+                return null;
+            }
+
+            AsciiLineReader reader = null;
+            if (genomeDescriptor.isGeneFileGZipFormat()) {
+                is = new GZIPInputStream(inputStream);
+                reader = new AsciiLineReader(is);
+            } else {
+                is = new BufferedInputStream(inputStream);
+                reader = new AsciiLineReader(is);
+            }
+
+            return reader;
+        } catch (IOException ex) {
+            log.warn("Error loading the genome!", ex);
+            return null;
+        }
 
     }
 
@@ -775,8 +874,8 @@ public class TrackManager {
     public void dumpData() {
     PrintWriter pw = null;
     try {
-    String chr = ViewContext.getInstance().getChromosome().getName();
-    double x = ViewContext.getInstance().getOrigin();
+    String chr = ReferenceFrame.getInstance().getChromosome().getName();
+    double x = ReferenceFrame.getInstance().getOrigin();
     pw = new PrintWriter(new FileWriter("DataDump.tab"));
     pw.println("Sample\tCopy Number\tExpression\tMethylation");
     for (String name : groupsMap.keySet()) {
@@ -833,5 +932,4 @@ public class TrackManager {
     }
     }     * */
 
-
 }
