@@ -1,19 +1,19 @@
 /*
- * Copyright (c) 2007-2010 by The Broad Institute, Inc. and the Massachusetts Institute of Technology.
- * All Rights Reserved.
+ * Copyright (c) 2007-2011 by The Broad Institute of MIT and Harvard.  All Rights Reserved.
  *
- * This software is licensed under the terms of the GNU Lesser General Public License (LGPL), Version 2.1 which
- * is available at http://www.opensource.org/licenses/lgpl-2.1.php.
+ * This software is licensed under the terms of the GNU Lesser General Public License (LGPL),
+ * Version 2.1 which is available at http://www.opensource.org/licenses/lgpl-2.1.php.
  *
- * THE SOFTWARE IS PROVIDED "AS IS." THE BROAD AND MIT MAKE NO REPRESENTATIONS OR WARRANTIES OF
- * ANY KIND CONCERNING THE SOFTWARE, EXPRESS OR IMPLIED, INCLUDING, WITHOUT LIMITATION, WARRANTIES
- * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NONINFRINGEMENT, OR THE ABSENCE OF LATENT
- * OR OTHER DEFECTS, WHETHER OR NOT DISCOVERABLE.  IN NO EVENT SHALL THE BROAD OR MIT, OR THEIR
- * RESPECTIVE TRUSTEES, DIRECTORS, OFFICERS, EMPLOYEES, AND AFFILIATES BE LIABLE FOR ANY DAMAGES OF
- * ANY KIND, INCLUDING, WITHOUT LIMITATION, INCIDENTAL OR CONSEQUENTIAL DAMAGES, ECONOMIC
- * DAMAGES OR INJURY TO PROPERTY AND LOST PROFITS, REGARDLESS OF WHETHER THE BROAD OR MIT SHALL
- * BE ADVISED, SHALL HAVE OTHER REASON TO KNOW, OR IN FACT SHALL KNOW OF THE POSSIBILITY OF THE
- * FOREGOING.
+ * THE SOFTWARE IS PROVIDED "AS IS." THE BROAD AND MIT MAKE NO REPRESENTATIONS OR
+ * WARRANTES OF ANY KIND CONCERNING THE SOFTWARE, EXPRESS OR IMPLIED, INCLUDING,
+ * WITHOUT LIMITATION, WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
+ * PURPOSE, NONINFRINGEMENT, OR THE ABSENCE OF LATENT OR OTHER DEFECTS, WHETHER
+ * OR NOT DISCOVERABLE.  IN NO EVENT SHALL THE BROAD OR MIT, OR THEIR RESPECTIVE
+ * TRUSTEES, DIRECTORS, OFFICERS, EMPLOYEES, AND AFFILIATES BE LIABLE FOR ANY DAMAGES
+ * OF ANY KIND, INCLUDING, WITHOUT LIMITATION, INCIDENTAL OR CONSEQUENTIAL DAMAGES,
+ * ECONOMIC DAMAGES OR INJURY TO PROPERTY AND LOST PROFITS, REGARDLESS OF WHETHER
+ * THE BROAD OR MIT SHALL BE ADVISED, SHALL HAVE OTHER REASON TO KNOW, OR IN FACT
+ * SHALL KNOW OF THE POSSIBILITY OF THE FOREGOING.
  */
 
 package org.broad.igv.ui;
@@ -56,7 +56,7 @@ public class ResourceTree {
     private HashMap<String, TreeNode> leafNodeMap = new HashMap();
     private JTree tree;
     private static ResourceTree theInstance;
-    private Set<String> selectedLeafNodePaths = new HashSet();
+    private Set<String> selectedLeafNodePaths = new LinkedHashSet();
 
     private static enum TreeExpansionFlag {
         EXPAND_ALL,
@@ -98,11 +98,11 @@ public class ResourceTree {
      * @param dialogTitle
      * @return the resources selected by user.
      */
-    public HashSet<ResourceLocator> showResourceTreeDialog(Component parent, Document document, String dialogTitle) {
+    public LinkedHashSet<ResourceLocator> showResourceTreeDialog(Component parent, Document document, String dialogTitle) {
 
         JDialog dialog = null;
 
-        final HashSet<ResourceLocator> locators = new HashSet();
+        final LinkedHashSet<ResourceLocator> locators = new LinkedHashSet();
         try {
 
             final ResourceTree resourceTree = getInstance();
@@ -131,11 +131,11 @@ public class ResourceTree {
 
                                     TreeModel model = dialogTree.getModel();
 
-                                    HashSet<DefaultMutableTreeNode> allCheckedLeafNodes =
+                                    LinkedHashSet<DefaultMutableTreeNode> allCheckedLeafNodes =
                                             resourceTree.findSelectedLeafNodes(model,
                                                     (TreeNode) model.getRoot(), null);
 
-                                    HashSet<ResourceLocator> selectedLocators =
+                                    LinkedHashSet<ResourceLocator> selectedLocators =
                                             resourceTree.getSelectedResourceLocators();
                                     for (ResourceLocator locator : selectedLocators) {
                                         locators.add(locator);
@@ -155,7 +155,7 @@ public class ResourceTree {
                                             buffer.append("\t");
                                         }
                                     }
-                                    PreferenceManager.getInstance().setCheckedResources(buffer.toString());
+                                    PreferenceManager.getInstance().put(PreferenceManager.CHECKED_RESOURCES_KEY, buffer.toString());
                                 }
                             }
                         }
@@ -174,7 +174,7 @@ public class ResourceTree {
 
             dialog.setResizable(true);
             dialog.pack();
-            dialog.setLocationRelativeTo(IGVMainFrame.getInstance());
+            dialog.setLocationRelativeTo(IGV.getMainFrame());
             dialog.setVisible(true);
 
             return  optionPane.isCanceled() ? null :  locators;
@@ -211,7 +211,7 @@ public class ResourceTree {
         tree.setCellEditor(new ResourceEditor(tree));
         tree.setEditable(true);
 
-        Set<ResourceLocator> loadedResources = IGVMainFrame.getInstance().getTrackManager().getDataResourceLocators();
+        Set<ResourceLocator> loadedResources = IGV.getInstance().getTrackManager().getDataResourceLocators();
         loadedResources.addAll(AttributeManager.getInstance().getLoadedResources());
 
         // Build and attach descentants of the root node to the tree
@@ -219,7 +219,7 @@ public class ResourceTree {
 
         // Get the previously checked leaf nodes.  The nodes are stored as
 
-        String paths = PreferenceManager.getInstance().getCheckedResources();
+        String paths = PreferenceManager.getInstance().get(PreferenceManager.CHECKED_RESOURCES_KEY);
         if (paths != null) {
             String[] selectedPaths = paths.split("\t");
             for (String path : selectedPaths) {
@@ -232,7 +232,6 @@ public class ResourceTree {
 
         expandTree(TreeExpansionFlag.EXPAND_SELECTED_ONLY, true);
         tree.updateUI(); // Tree state not always correct without this call
-        IGVMainFrame.getInstance().setCheckingSelectedResourceNodesAllowed(true);
         return tree;
     }
 
@@ -251,7 +250,13 @@ public class ResourceTree {
         }
 
         locator.setInfolink(getAttribute((Element) xmlNode, HYPERLINK.getText()));
-        locator.setSampleId(getAttribute((Element) xmlNode, SAMPLE_ID.getText()));
+
+        String sampleId = getAttribute((Element) xmlNode, SAMPLE_ID.getText());
+        if(sampleId == null) {
+            // legacy option
+            sampleId = getAttribute((Element) xmlNode, ID.getText());
+        }
+        locator.setSampleId(sampleId);
         locator.setUrl(getAttribute((Element) xmlNode, URL.getText()));
         locator.setDescription(getAttribute((Element) xmlNode, DESCRIPTION.getText()));
         locator.setName(name);
@@ -263,13 +268,23 @@ public class ResourceTree {
         }
         locator.setTrackLine(getAttribute((Element) xmlNode, TRACK_LINE.getText()));
 
+        if (coverage != null) {
+            locator.setCoverage(coverage);
+        }
+        locator.setTrackLine(getAttribute((Element) xmlNode, TRACK_LINE.getText()));
+
+        if (coverage != null) {
+            locator.setCoverage(coverage);
+        }
+        locator.setTrackLine(getAttribute((Element) xmlNode, TRACK_LINE.getText()));
+
         if (coverage != null) {
             locator.setCoverage(coverage);
         }
         String colorString = getAttribute((Element) xmlNode, "color");
         if (colorString != null) {
             try {
-                Color c = ColorUtilities.getColorFromString(colorString);
+                Color c = ColorUtilities.stringToColor(colorString);
                 locator.setColor(c);
             } catch (Exception e) {
                 log.error("Error setting color: ", e);
@@ -355,9 +370,9 @@ public class ResourceTree {
         return leafResources;
     }
 
-    public HashSet<ResourceLocator> getSelectedResourceLocators() {
+    public LinkedHashSet<ResourceLocator> getSelectedResourceLocators() {
 
-        HashSet<ResourceLocator> resourceLocators = new HashSet();
+        LinkedHashSet<ResourceLocator> resourceLocators = new LinkedHashSet();
 
         for (CheckableResource resource : leafResources) {
 
@@ -380,11 +395,11 @@ public class ResourceTree {
     }
 
 
-    private HashSet<DefaultMutableTreeNode> findSelectedLeafNodes(
-            TreeModel model, TreeNode parentNode, HashSet<DefaultMutableTreeNode> allCheckedLeafNodes) {
+    private LinkedHashSet<DefaultMutableTreeNode> findSelectedLeafNodes(
+            TreeModel model, TreeNode parentNode, LinkedHashSet<DefaultMutableTreeNode> allCheckedLeafNodes) {
 
         if (allCheckedLeafNodes == null) {
-            allCheckedLeafNodes = new HashSet();
+            allCheckedLeafNodes = new LinkedHashSet();
         }
 
         int count = model.getChildCount(parentNode);
@@ -1186,9 +1201,6 @@ public class ResourceTree {
     private void manuallySelectNode(DefaultMutableTreeNode childNode,
                                     boolean skipSelectingEnabledNodes) {
 
-        if (!IGVMainFrame.getInstance().isCheckingSelectedResourceNodesAllowed()) {
-            return;
-        }
 
         Object userObject = childNode.getUserObject();
         if (userObject instanceof CheckableResource) {
@@ -1278,7 +1290,7 @@ public class ResourceTree {
 
     static private Set<ResourceLocator> getLoadedResources() {
         Set<ResourceLocator> loadedResources =
-                IGVMainFrame.getInstance().getTrackManager().getDataResourceLocators();
+                IGV.getInstance().getTrackManager().getDataResourceLocators();
         loadedResources.addAll(AttributeManager.getInstance().getLoadedResources());
         return loadedResources;
     }
