@@ -1,19 +1,19 @@
 /*
- * Copyright (c) 2007-2010 by The Broad Institute, Inc. and the Massachusetts Institute of Technology.
- * All Rights Reserved.
+ * Copyright (c) 2007-2011 by The Broad Institute of MIT and Harvard.  All Rights Reserved.
  *
- * This software is licensed under the terms of the GNU Lesser General Public License (LGPL), Version 2.1 which
- * is available at http://www.opensource.org/licenses/lgpl-2.1.php.
+ * This software is licensed under the terms of the GNU Lesser General Public License (LGPL),
+ * Version 2.1 which is available at http://www.opensource.org/licenses/lgpl-2.1.php.
  *
- * THE SOFTWARE IS PROVIDED "AS IS." THE BROAD AND MIT MAKE NO REPRESENTATIONS OR WARRANTIES OF
- * ANY KIND CONCERNING THE SOFTWARE, EXPRESS OR IMPLIED, INCLUDING, WITHOUT LIMITATION, WARRANTIES
- * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NONINFRINGEMENT, OR THE ABSENCE OF LATENT
- * OR OTHER DEFECTS, WHETHER OR NOT DISCOVERABLE.  IN NO EVENT SHALL THE BROAD OR MIT, OR THEIR
- * RESPECTIVE TRUSTEES, DIRECTORS, OFFICERS, EMPLOYEES, AND AFFILIATES BE LIABLE FOR ANY DAMAGES OF
- * ANY KIND, INCLUDING, WITHOUT LIMITATION, INCIDENTAL OR CONSEQUENTIAL DAMAGES, ECONOMIC
- * DAMAGES OR INJURY TO PROPERTY AND LOST PROFITS, REGARDLESS OF WHETHER THE BROAD OR MIT SHALL
- * BE ADVISED, SHALL HAVE OTHER REASON TO KNOW, OR IN FACT SHALL KNOW OF THE POSSIBILITY OF THE
- * FOREGOING.
+ * THE SOFTWARE IS PROVIDED "AS IS." THE BROAD AND MIT MAKE NO REPRESENTATIONS OR
+ * WARRANTES OF ANY KIND CONCERNING THE SOFTWARE, EXPRESS OR IMPLIED, INCLUDING,
+ * WITHOUT LIMITATION, WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
+ * PURPOSE, NONINFRINGEMENT, OR THE ABSENCE OF LATENT OR OTHER DEFECTS, WHETHER
+ * OR NOT DISCOVERABLE.  IN NO EVENT SHALL THE BROAD OR MIT, OR THEIR RESPECTIVE
+ * TRUSTEES, DIRECTORS, OFFICERS, EMPLOYEES, AND AFFILIATES BE LIABLE FOR ANY DAMAGES
+ * OF ANY KIND, INCLUDING, WITHOUT LIMITATION, INCIDENTAL OR CONSEQUENTIAL DAMAGES,
+ * ECONOMIC DAMAGES OR INJURY TO PROPERTY AND LOST PROFITS, REGARDLESS OF WHETHER
+ * THE BROAD OR MIT SHALL BE ADVISED, SHALL HAVE OTHER REASON TO KNOW, OR IN FACT
+ * SHALL KNOW OF THE POSSIBILITY OF THE FOREGOING.
  */
 /*
  * To change this template, choose Tools | Templates
@@ -25,17 +25,14 @@ package org.broad.igv.ui.action;
 
 import org.apache.log4j.Logger;
 import org.broad.igv.PreferenceManager;
-import org.broad.igv.session.ViewContext;
-import org.broad.igv.track.AttributeManager;
-import org.broad.igv.ui.IGVMainFrame;
+import org.broad.igv.ui.IGV;
 import org.broad.igv.ui.ResourceTree;
 import org.broad.igv.ui.UIConstants;
 import org.broad.igv.ui.util.MessageUtils;
-import org.broad.igv.ui.util.UIUtilities;
 import org.broad.igv.util.IGVHttpUtils;
+import org.broad.igv.util.ParsingUtils;
 import org.broad.igv.util.ResourceLocator;
 import org.broad.igv.util.Utilities;
-import org.broad.tribble.util.HttpUtils;
 import org.w3c.dom.Document;
 import org.w3c.dom.Element;
 import org.w3c.dom.NodeList;
@@ -45,8 +42,14 @@ import javax.swing.*;
 import javax.xml.parsers.DocumentBuilderFactory;
 import java.awt.event.ActionEvent;
 import java.io.*;
-import java.net.*;
-import java.util.*;
+import java.net.ConnectException;
+import java.net.HttpURLConnection;
+import java.net.URL;
+import java.net.UnknownHostException;
+import java.util.ArrayList;
+import java.util.HashSet;
+import java.util.LinkedHashSet;
+import java.util.List;
 
 /**
  * @author jrobinso
@@ -54,13 +57,13 @@ import java.util.*;
 public class LoadFromServerAction extends MenuAction {
 
     static Logger log = Logger.getLogger(LoadFromServerAction.class);
-    IGVMainFrame mainFrame;
+    IGV mainFrame;
 
     // Keep track of authorization failures so user isn't constantly harranged
     static HashSet<String> failedURLs = new HashSet();
 
 
-    public LoadFromServerAction(String label, int mnemonic, IGVMainFrame mainFrame) {
+    public LoadFromServerAction(String label, int mnemonic, IGV mainFrame) {
         super(label, null, mnemonic);
         this.mainFrame = mainFrame;
         setToolTipText(UIConstants.LOAD_SERVER_DATA_TOOLTIP);
@@ -73,57 +76,27 @@ public class LoadFromServerAction extends MenuAction {
         mainFrame.setStatusBarMessage("Loading ...");
 
         String urlString = PreferenceManager.getInstance().getDataServerURL();
-        String genomeId = ViewContext.getInstance().getGenomeId();
+        String genomeId = IGV.getInstance().getGenomeManager().getGenomeId();
         String genomeURL = urlString.replaceAll("\\$\\$", genomeId);
         try {
-
-
-            // Add genome parameter for users with hardcoded reference to registry servlet
-            if (genomeURL.startsWith("http:") || genomeURL.startsWith("https:"))
-                genomeURL += "?genome=" + genomeId;
-
-
             InputStream is = null;
-            HttpURLConnection conn = null;
-            LinkedHashSet<URL> urls = null;
+            LinkedHashSet<String> urls = null;
             try {
-                URL masterResourceFileURL = new URL(genomeURL);
-                if (genomeURL.startsWith("ftp:")) {
-                    MessageUtils.showMessage("FTP protocol not supported for data registry URL");
-                    return;
-                } else if (genomeURL.startsWith("http:") || genomeURL.startsWith("https:")) {
-                    conn = IGVHttpUtils.openConnection(masterResourceFileURL);
-                    conn.setReadTimeout(10000);
-                    conn.setConnectTimeout(10000);
-                    is = IGVHttpUtils.openHttpStream(masterResourceFileURL, conn);
-                } else {
-                    File file = new File(genomeURL.startsWith("file:") ? masterResourceFileURL.getFile() : genomeURL);
-                    if(!file.exists()) {
-                       MessageUtils.showMessage("ERROR: File does not exist: " + file.getAbsolutePath());
-                    return;
-                    }
-                    is = new FileInputStream(file);
-                }
-
+                is = ParsingUtils.openInputStream(new ResourceLocator(genomeURL));
                 BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(is));
                 urls = getResourceUrls(bufferedReader);
             } catch (IOException e) {
-                String msg = "";
-                if (conn.getResponseCode() == 401) {
-                    msg = "Authorization failure accessing resource: " + genomeURL;
-                } else {
-                    msg = "Error accessing dataset list: " + e.toString();
-                }
-                MessageUtils.showMessage(msg);
-                log.error("Error accessing URL: " + genomeURL, e);
+                MessageUtils.showMessage("Error loading the genome registry file: "  + e.getMessage());
+                log.error("Error loading genome registry file", e);
+                return;
+
             }
             finally {
                 if (is != null) is.close();
-                if (conn != null) conn.disconnect();
             }
 
             if (urls == null || urls.isEmpty()) {
-                JOptionPane.showMessageDialog(mainFrame,
+                JOptionPane.showMessageDialog(mainFrame.getMainFrame(),
                         "No datasets are available for the current genome (" + genomeId + ").");
             } else {
 
@@ -156,7 +129,7 @@ public class LoadFromServerAction extends MenuAction {
 
     }
 
-    public List<ResourceLocator> selectResources(final LinkedHashSet<URL> xmlUrls) {
+    public List<ResourceLocator> selectResources(final LinkedHashSet<String> xmlUrls) {
 
         if ((xmlUrls == null) || xmlUrls.isEmpty()) {
             log.error("No datasets are available from this server for the current genome (");
@@ -178,22 +151,18 @@ public class LoadFromServerAction extends MenuAction {
             masterDocument.appendChild(rootNode);
 
             // Merge all documents into one xml document for processing
-            for (URL url : xmlUrls) {
+            for (String url : xmlUrls) {
 
                 // Skip urls that have previously failed due to authorization
-                if (failedURLs.contains(url.toString())) {
+                if (failedURLs.contains(url)) {
                     continue;
                 }
 
                 try {
-                    HttpURLConnection connection = null;
                     InputStream is = null;
                     Document xmlDocument = null;
                     try {
-                        connection = IGVHttpUtils.openConnection(url);
-                        connection.setConnectTimeout(10000);
-                        connection.setReadTimeout(10000);
-                        is = IGVHttpUtils.openHttpStream(url, connection);
+                        is = ParsingUtils.openInputStream(new ResourceLocator(url));
                         xmlDocument = Utilities.createDOMDocumentFromXmlStream(is);
                     }
                     catch (java.net.SocketTimeoutException e) {
@@ -227,19 +196,13 @@ public class LoadFromServerAction extends MenuAction {
                         continue;
                     }
                     catch (IOException e) {
-                        String msg = "";
-                        if (connection.getResponseCode() == 401) {
-                            failedURLs.add(url.toString());
-                            msg = "Authorization failure accessing resource: " + url;
-                        } else {
-                            msg = "Error accessing dataset list: " + e.toString();
-                        }
+                        String msg = "Error accessing dataset list: " + e.toString();
+
                         MessageUtils.showMessage(msg);
                         log.error("Error accessing URL: " + url, e);
                     }
                     finally {
                         if (is != null) is.close();
-                        if (connection != null) connection.disconnect();
                     }
 
                     if (xmlDocument != null) {
@@ -263,26 +226,24 @@ public class LoadFromServerAction extends MenuAction {
 
             }
             if (xmlParsingError) {
-                JOptionPane.showMessageDialog(mainFrame, buffer.toString());
+                JOptionPane.showMessageDialog(mainFrame.getMainFrame(), buffer.toString());
             }
 
 
             /**
              * Resource Tree
              */
-            HashSet<ResourceLocator> selectedLocators =
-                    ResourceTree.getInstance().showResourceTreeDialog(mainFrame, masterDocument, "Available Datasets");
+            LinkedHashSet<ResourceLocator> selectedLocators =
+                    ResourceTree.getInstance().showResourceTreeDialog(mainFrame.getMainFrame(),
+                            masterDocument, "Available Datasets");
 
             List<ResourceLocator> newLoadList = new ArrayList();
 
-            Set<ResourceLocator> loadedResources = IGVMainFrame.getInstance().getTrackManager().getDataResourceLocators();
-            loadedResources.addAll(AttributeManager.getInstance().getLoadedResources());
-
             if (selectedLocators != null) {
                 for (ResourceLocator locator : selectedLocators) {
 
                     // Don't reload data that is already loaded
-                    if (loadedResources.contains(locator)) {
+                    if (IGV.getInstance().getTrackManager().getDataResourceLocators().contains(locator)) {
                         continue;
                     }
 
@@ -310,19 +271,17 @@ public class LoadFromServerAction extends MenuAction {
      * @throws java.net.MalformedURLException
      * @throws java.io.IOException
      */
-    private LinkedHashSet<URL> getResourceUrls
-            (BufferedReader
-                    bufferedReader)
+    private LinkedHashSet<String> getResourceUrls(BufferedReader bufferedReader)
             throws IOException {
 
-        LinkedHashSet<URL> xmlFileUrls = new LinkedHashSet();
+        LinkedHashSet<String> xmlFileUrls = new LinkedHashSet();
         while (true) {
             String xmlFileUrl = bufferedReader.readLine();
             if ((xmlFileUrl == null) || (xmlFileUrl.trim().length() == 0)) {
                 break;
             }
             xmlFileUrl = xmlFileUrl.trim();
-            xmlFileUrls.add(new URL(xmlFileUrl));
+            xmlFileUrls.add(xmlFileUrl);
         }
 
         return xmlFileUrls;
