@@ -1,19 +1,19 @@
 /*
- * Copyright (c) 2007-2010 by The Broad Institute, Inc. and the Massachusetts Institute of Technology.
- * All Rights Reserved.
+ * Copyright (c) 2007-2011 by The Broad Institute of MIT and Harvard.  All Rights Reserved.
  *
- * This software is licensed under the terms of the GNU Lesser General Public License (LGPL), Version 2.1 which
- * is available at http://www.opensource.org/licenses/lgpl-2.1.php.
+ * This software is licensed under the terms of the GNU Lesser General Public License (LGPL),
+ * Version 2.1 which is available at http://www.opensource.org/licenses/lgpl-2.1.php.
  *
- * THE SOFTWARE IS PROVIDED "AS IS." THE BROAD AND MIT MAKE NO REPRESENTATIONS OR WARRANTIES OF
- * ANY KIND CONCERNING THE SOFTWARE, EXPRESS OR IMPLIED, INCLUDING, WITHOUT LIMITATION, WARRANTIES
- * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NONINFRINGEMENT, OR THE ABSENCE OF LATENT
- * OR OTHER DEFECTS, WHETHER OR NOT DISCOVERABLE.  IN NO EVENT SHALL THE BROAD OR MIT, OR THEIR
- * RESPECTIVE TRUSTEES, DIRECTORS, OFFICERS, EMPLOYEES, AND AFFILIATES BE LIABLE FOR ANY DAMAGES OF
- * ANY KIND, INCLUDING, WITHOUT LIMITATION, INCIDENTAL OR CONSEQUENTIAL DAMAGES, ECONOMIC
- * DAMAGES OR INJURY TO PROPERTY AND LOST PROFITS, REGARDLESS OF WHETHER THE BROAD OR MIT SHALL
- * BE ADVISED, SHALL HAVE OTHER REASON TO KNOW, OR IN FACT SHALL KNOW OF THE POSSIBILITY OF THE
- * FOREGOING.
+ * THE SOFTWARE IS PROVIDED "AS IS." THE BROAD AND MIT MAKE NO REPRESENTATIONS OR
+ * WARRANTES OF ANY KIND CONCERNING THE SOFTWARE, EXPRESS OR IMPLIED, INCLUDING,
+ * WITHOUT LIMITATION, WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
+ * PURPOSE, NONINFRINGEMENT, OR THE ABSENCE OF LATENT OR OTHER DEFECTS, WHETHER
+ * OR NOT DISCOVERABLE.  IN NO EVENT SHALL THE BROAD OR MIT, OR THEIR RESPECTIVE
+ * TRUSTEES, DIRECTORS, OFFICERS, EMPLOYEES, AND AFFILIATES BE LIABLE FOR ANY DAMAGES
+ * OF ANY KIND, INCLUDING, WITHOUT LIMITATION, INCIDENTAL OR CONSEQUENTIAL DAMAGES,
+ * ECONOMIC DAMAGES OR INJURY TO PROPERTY AND LOST PROFITS, REGARDLESS OF WHETHER
+ * THE BROAD OR MIT SHALL BE ADVISED, SHALL HAVE OTHER REASON TO KNOW, OR IN FACT
+ * SHALL KNOW OF THE POSSIBILITY OF THE FOREGOING.
  */
 /*
  * TrackPanel.java
@@ -31,10 +31,9 @@ import org.apache.log4j.Logger;
 import org.broad.igv.PreferenceManager;
 import org.broad.igv.track.AttributeManager;
 import org.broad.igv.track.Track;
+import org.broad.igv.track.TrackClickEvent;
 import org.broad.igv.track.TrackGroup;
-import org.broad.igv.ui.IGVMainFrame;
-
-import static org.broad.igv.ui.IGVMainFrame.getInstance;
+import org.broad.igv.ui.IGV;
 
 import org.broad.igv.ui.UIConstants;
 import org.broad.igv.ui.util.Packable;
@@ -42,21 +41,17 @@ import org.broad.igv.ui.util.Packable;
 import javax.swing.event.MouseInputAdapter;
 import java.awt.*;
 import java.awt.event.MouseEvent;
-import java.beans.PropertyChangeEvent;
-import java.beans.PropertyChangeListener;
 import java.util.*;
 import java.util.List;
 
 /**
  * @author jrobinso
  */
-public class AttributePanel extends TrackPanelComponent implements Packable {
+public class AttributePanel extends TrackPanelComponent implements Packable, Paintable {
 
     private static Logger log = Logger.getLogger(AttributePanel.class);
 
-    private static Map<String, Color> colorMap = new Hashtable();
-
-
+ 
     /**
      * Constructs ...
      */
@@ -71,17 +66,31 @@ public class AttributePanel extends TrackPanelComponent implements Packable {
     protected void paintComponent(Graphics g) {
 
         super.paintComponent(g);
+        Rectangle visibleRect = getVisibleRect();
         removeMousableRegions();
+        paintOffscreen((Graphics2D) g, visibleRect);
 
-        List<String> keys = AttributeManager.getInstance().getAttributeKeys();
-        keys.removeAll(AttributeManager.getInstance().getHiddenAttributes());
+    }
 
-        if (keys == null) {
+
+    public void paintOffscreen(Graphics2D g, Rectangle rect) {
+
+        paintImpl(g, rect);
+        super.paintBorder(g);
+    }
+
+    public void paintImpl(Graphics2D g, Rectangle visibleRect) {
+
+        List<String> names = AttributeManager.getInstance().getAttributeNames();
+
+        if (names == null) {
             return;
         }
 
-        if (keys.size() > 0) {
-            Rectangle visibleRect = getVisibleRect();
+        final Set<String> hiddenAttributes = IGV.getInstance().getSession().getHiddenAttributes();
+        if (hiddenAttributes != null) names.removeAll(hiddenAttributes);
+
+        if (names.size() > 0) {
 
 
             // Get the current tracks
@@ -96,7 +105,7 @@ public class AttributePanel extends TrackPanelComponent implements Packable {
                 graphics2D.setColor(Color.BLACK);
 
                 final Graphics2D greyGraphics = (Graphics2D) g.create();
-                greyGraphics.setColor(UIConstants.VERY_LIGHT_GRAY);
+                greyGraphics.setColor(UIConstants.ZOOMED_OUT_COLOR);
 
                 int regionX = 3;
                 int regionY = 0;
@@ -120,7 +129,7 @@ public class AttributePanel extends TrackPanelComponent implements Packable {
                         }
 
                         for (Track track : group.getTracks()) {
-
+                            if (track == null) continue;
                             int trackHeight = track.getHeight();
                             if (regionY > visibleRect.y + visibleRect.height) {
                                 break;
@@ -128,7 +137,7 @@ public class AttributePanel extends TrackPanelComponent implements Packable {
 
                             if (track.isVisible()) {
                                 if (regionY + trackHeight >= visibleRect.y) {
-                                    regionY = draw(keys, track, regionX, regionY, attributeColumnWidth, track.getHeight(), graphics2D);
+                                    regionY = draw(names, track, regionX, regionY, attributeColumnWidth, track.getHeight(), graphics2D);
                                 } else {
                                     regionY += trackHeight;
                                 }
@@ -144,57 +153,20 @@ public class AttributePanel extends TrackPanelComponent implements Packable {
         }
     }
 
-    private Color getColor(String attKey, String attValue) {
 
-        if (attValue == null || attValue.length() == 0) {
-            return Color.white;
-        }
 
-        String key = (attKey + "_" + attValue).toLowerCase();
-        Color c = colorMap.get(key);
-        if (c == null) {
-
-            Integer cnt = colorCounter.get(attKey);
-            if (cnt == null) {
-                cnt = 0;
-            }
-            cnt++;
-            colorCounter.put(attKey, cnt);
-            float hue = (float) (.4 + 0.2 * Math.random());
-
-            // int index = colorMap.size() + 1;
-            c = randomColor(cnt);
-            colorMap.put(key, c);
-        }
-        return c;
-    }
-
-    Map<String, Integer> colorCounter = new HashMap();
-
-    /**
-     * Method description
-     *
-     * @param idx
-     * @return
-     */
-    public static Color randomColor(int idx) {
-        float hue = (float) Math.random();
-        float sat = (float) (0.8 * Math.random());
-        float bri = (float) (0.6 + 0.4 * Math.random());
-        return Color.getHSBColor(hue, sat, bri);
-    }
-
-    private int draw(List<String> keys, Track track, int trackX, int trackY, int trackWidth, int trackHeight,
+    private int draw(List<String> names, Track track, int trackX, int trackY, int trackWidth, int trackHeight,
                      Graphics2D graphics) {
 
-        for (String key : keys) {
+        for (String name : names) {
 
+            String key = name.toUpperCase();
             String attributeValue = track.getAttributeValue(key);
 
             if (attributeValue != null) {
 
                 Rectangle trackRectangle = new Rectangle(trackX, trackY, trackWidth, trackHeight);
-                graphics.setColor(getColor(key, attributeValue));
+                graphics.setColor(AttributeManager.getInstance().getColor(key, attributeValue));
                 graphics.fill(trackRectangle);
                 addMousableRegion(new MouseableRegion(trackRectangle, key, attributeValue));
             }
@@ -208,23 +180,13 @@ public class AttributePanel extends TrackPanelComponent implements Packable {
     private void init() {
 
         setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
-        if (!PreferenceManager.getInstance().getShowAttributeView()) {
+        if (!PreferenceManager.getInstance().getAsBoolean(PreferenceManager.SHOW_ATTRIBUTE_VIEWS_KEY)) {
             setSize(0, getHeight());
         }
         setBackground(new java.awt.Color(255, 255, 255));
         setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0)));
         setPreferredSize(new java.awt.Dimension(0, 0));
         setVerifyInputWhenFocusTarget(false);
-        org.jdesktop.layout.GroupLayout dataTrackAttributePanelLayout = new org.jdesktop.layout.GroupLayout(
-                this);
-        setLayout(dataTrackAttributePanelLayout);
-        dataTrackAttributePanelLayout.setHorizontalGroup(
-                dataTrackAttributePanelLayout.createParallelGroup(
-                        org.jdesktop.layout.GroupLayout.LEADING).add(0, 118, Short.MAX_VALUE));
-        dataTrackAttributePanelLayout.setVerticalGroup(
-                dataTrackAttributePanelLayout.createParallelGroup(
-                        org.jdesktop.layout.GroupLayout.LEADING).add(0, 528, Short.MAX_VALUE));
-
 
         MouseInputAdapter mouseAdapter = new AttributePanelMouseAdapter();
         addMouseMotionListener(mouseAdapter);
@@ -284,41 +246,16 @@ public class AttributePanel extends TrackPanelComponent implements Packable {
 
     // Packable interface
 
-    /**
-     * Method description
-     *
-     * @return
-     */
-    public boolean isHorizontallyPackable() {
-        return true;
-    }
-
-
-    // Code below is used to pack the attribute pane
-
-    /**
-     * Method description
-     */
-    public void packHorizontally() {
-
-        int newWidth = calculatePackWidth();
-
-        Dimension dimension = getSize();
-        dimension = new Dimension(newWidth, dimension.height);
-        setMinimumSize(dimension);
-        setMaximumSize(dimension);
-        setSize(dimension);
-        setPreferredSize(dimension);
-    }
 
     private int calculatePackWidth() {
 
-        if (!PreferenceManager.getInstance().getShowAttributeView()) {
+        if (!PreferenceManager.getInstance().getAsBoolean(PreferenceManager.SHOW_ATTRIBUTE_VIEWS_KEY)) {
             return 0;
         }
 
-        HashSet<String> attributeKeys = new HashSet(AttributeManager.getInstance().getAttributeKeys());
-        attributeKeys.removeAll(AttributeManager.getInstance().getHiddenAttributes());
+        HashSet<String> attributeKeys = new HashSet(AttributeManager.getInstance().getAttributeNames());
+        final Set<String> hiddenAttributes = IGV.getInstance().getSession().getHiddenAttributes();
+        if (hiddenAttributes != null) attributeKeys.removeAll(hiddenAttributes);
 
         int attributeCount = attributeKeys.size();
         int packWidth = (attributeCount) * (AttributeHeaderPanel.ATTRIBUTE_COLUMN_WIDTH +
@@ -366,17 +303,19 @@ public class AttributePanel extends TrackPanelComponent implements Packable {
             clearTrackSelections();
             selectTracks(e);
             if (e.isPopupTrigger()) {
-                openPopupMenu(e);
+                TrackClickEvent te = new TrackClickEvent(e, null);
+                openPopupMenu(te);
 
             }
-            IGVMainFrame.getInstance().repaintNamePanels();
+            IGV.getInstance().repaintNamePanels();
         }
 
         public void mouseReleased(MouseEvent e) {
             // Show Popup Menu.  The track selection is cleared afterwards.
             // Note: clearing after this operation is "non standard", at least on the mac
             if (e.isPopupTrigger()) {
-                openPopupMenu(e);
+                TrackClickEvent te = new TrackClickEvent(e, null);
+                openPopupMenu(te);
                 clearTrackSelections();
             }
 
