@@ -1,19 +1,19 @@
 /*
- * Copyright (c) 2007-2010 by The Broad Institute, Inc. and the Massachusetts Institute of Technology.
- * All Rights Reserved.
+ * Copyright (c) 2007-2011 by The Broad Institute of MIT and Harvard.  All Rights Reserved.
  *
- * This software is licensed under the terms of the GNU Lesser General Public License (LGPL), Version 2.1 which
- * is available at http://www.opensource.org/licenses/lgpl-2.1.php.
+ * This software is licensed under the terms of the GNU Lesser General Public License (LGPL),
+ * Version 2.1 which is available at http://www.opensource.org/licenses/lgpl-2.1.php.
  *
- * THE SOFTWARE IS PROVIDED "AS IS." THE BROAD AND MIT MAKE NO REPRESENTATIONS OR WARRANTIES OF
- * ANY KIND CONCERNING THE SOFTWARE, EXPRESS OR IMPLIED, INCLUDING, WITHOUT LIMITATION, WARRANTIES
- * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NONINFRINGEMENT, OR THE ABSENCE OF LATENT
- * OR OTHER DEFECTS, WHETHER OR NOT DISCOVERABLE.  IN NO EVENT SHALL THE BROAD OR MIT, OR THEIR
- * RESPECTIVE TRUSTEES, DIRECTORS, OFFICERS, EMPLOYEES, AND AFFILIATES BE LIABLE FOR ANY DAMAGES OF
- * ANY KIND, INCLUDING, WITHOUT LIMITATION, INCIDENTAL OR CONSEQUENTIAL DAMAGES, ECONOMIC
- * DAMAGES OR INJURY TO PROPERTY AND LOST PROFITS, REGARDLESS OF WHETHER THE BROAD OR MIT SHALL
- * BE ADVISED, SHALL HAVE OTHER REASON TO KNOW, OR IN FACT SHALL KNOW OF THE POSSIBILITY OF THE
- * FOREGOING.
+ * THE SOFTWARE IS PROVIDED "AS IS." THE BROAD AND MIT MAKE NO REPRESENTATIONS OR
+ * WARRANTES OF ANY KIND CONCERNING THE SOFTWARE, EXPRESS OR IMPLIED, INCLUDING,
+ * WITHOUT LIMITATION, WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
+ * PURPOSE, NONINFRINGEMENT, OR THE ABSENCE OF LATENT OR OTHER DEFECTS, WHETHER
+ * OR NOT DISCOVERABLE.  IN NO EVENT SHALL THE BROAD OR MIT, OR THEIR RESPECTIVE
+ * TRUSTEES, DIRECTORS, OFFICERS, EMPLOYEES, AND AFFILIATES BE LIABLE FOR ANY DAMAGES
+ * OF ANY KIND, INCLUDING, WITHOUT LIMITATION, INCIDENTAL OR CONSEQUENTIAL DAMAGES,
+ * ECONOMIC DAMAGES OR INJURY TO PROPERTY AND LOST PROFITS, REGARDLESS OF WHETHER
+ * THE BROAD OR MIT SHALL BE ADVISED, SHALL HAVE OTHER REASON TO KNOW, OR IN FACT
+ * SHALL KNOW OF THE POSSIBILITY OF THE FOREGOING.
  */
 
 /*
@@ -26,21 +26,23 @@
  */
 package org.broad.igv.ui.panel;
 
-import com.jidesoft.utils.SwingWorker;
 import org.apache.log4j.Logger;
 import org.broad.igv.Globals;
 import org.broad.igv.PreferenceManager;
 import org.broad.igv.feature.Chromosome;
-import org.broad.igv.feature.Genome;
-import org.broad.igv.session.ViewContext;
+import org.broad.igv.feature.genome.Genome;
 import org.broad.igv.ui.FontManager;
-import org.broad.igv.ui.IGVMainFrame;
+import org.broad.igv.ui.IGV;
+import org.broad.igv.ui.UIConstants;
 import org.broad.igv.ui.WaitCursorManager;
+import org.broad.igv.util.LongRunningTask;
+import org.broad.igv.util.NamedRunnable;
 
 import javax.swing.*;
 import javax.swing.event.MouseInputAdapter;
 import java.awt.*;
 import java.awt.event.MouseEvent;
+import java.awt.geom.Rectangle2D;
 import java.text.DecimalFormat;
 import java.util.ArrayList;
 import java.util.List;
@@ -64,9 +66,8 @@ public class RulerPanel extends JPanel {
     // TODO -- get from preferences
     boolean drawSpan = true;
     boolean drawEllipsis = false;
-    private Font tickFont = FontManager.getScalableFont(Font.BOLD, 9);
-    private Font spanFont = FontManager.getScalableFont(Font.BOLD, 12);
-    private Font chrFont = FontManager.getScalableFont(10);
+    private Font tickFont = FontManager.getFont(Font.BOLD, 9);
+    private Font spanFont = FontManager.getFont(Font.BOLD, 12);
     private List<ClickLink> chromosomeRects = new ArrayList();
 
     private static Color dragColor = new Color(.5f, .5f, 1f, .3f);
@@ -79,12 +80,15 @@ public class RulerPanel extends JPanel {
             "<br>or click and drag to zoom in.";
     public static final String CHROM_TOOLTIP = "Click and drag to zoom in.";
 
-    public RulerPanel() {
-        initialize();
+    ReferenceFrame frame;
+
+    public RulerPanel(ReferenceFrame frame) {
+        this.frame = frame;
+        init();
     }
 
     private boolean isWholeGenomeView() {
-        return getViewContext().getChrName().equals("All");
+        return getViewContext().getChrName().equals(Globals.CHR_ALL);
     }
 
     @Override
@@ -92,7 +96,6 @@ public class RulerPanel extends JPanel {
 
         super.paintComponent(g);
 
-
         g.setColor(Color.black);
 
         if (isWholeGenomeView()) {
@@ -123,8 +126,8 @@ public class RulerPanel extends JPanel {
 
     }
 
-    private ViewContext getViewContext() {
-        return ViewContext.getInstance();
+    private ReferenceFrame getViewContext() {
+        return frame;
     }
 
     private void drawSpan(Graphics g) {
@@ -138,7 +141,7 @@ public class RulerPanel extends JPanel {
         // Positions are 1/2 open, subtract 1 to compensate
         int range = (int) (getViewContext().getScale() * w) + 1;
 
-        TickSpacing ts = findSpacing(range);
+        TickSpacing ts = findSpacing(getViewContext(), range);
         String rangeString = formatNumber((double) range / ts.getUnitMultiplier()) + " " + ts.getMajorUnit();
         int strWidth = g.getFontMetrics().stringWidth(rangeString);
         int strHeight = g.getFontMetrics().getAscent();
@@ -177,21 +180,24 @@ public class RulerPanel extends JPanel {
     private void drawTicks(Graphics g) {
 
         int w = getWidth();
+        if (w < 200) {
+            return;
+        }
 
         g.setFont(tickFont);
 
         int range = (int) (w * getViewContext().getScale());
-        TickSpacing ts = findSpacing(range);
+        TickSpacing ts = findSpacing(getViewContext(), range);
         double spacing = ts.getMajorTick();
 
         // Find starting point closest to the current origin
         int nTick = (int) (getViewContext().getOrigin() / spacing) - 1;
         int l = (int) (nTick * spacing);
-        int x = getViewContext().getPixelPosition(l - 1);    // 0 vs 1 based coordinates
+        int x = getViewContext().getScreenPosition(l - 1);    // 0 vs 1 based coordinates
         //int strEnd = Integer.MIN_VALUE;
         while (x < getWidth()) {
             l = (int) (nTick * spacing);
-            x = getViewContext().getPixelPosition(l - 1);
+            x = getViewContext().getScreenPosition(l - 1);
             String chrPosition = formatNumber((double) l / ts.getUnitMultiplier()) +
                     " " + ts.getMajorUnit();
             int strWidth = g.getFontMetrics().stringWidth(chrPosition);
@@ -210,6 +216,7 @@ public class RulerPanel extends JPanel {
 
     private void drawChromosomeTicks(Graphics g) {
 
+        Font chrFont = FontManager.getFont(10);
         //this.removeAll();
         this.setLayout(null);
 
@@ -218,9 +225,9 @@ public class RulerPanel extends JPanel {
 
         g.setFont(chrFont);
 
-        Genome genome = getViewContext().getGenome();
+        Genome genome = IGV.getInstance().getGenomeManager().getCurrentGenome();
         if (genome == null) {
-            log.info("No genome found with id: " + getViewContext().getGenomeId());
+            log.info("No genome found with id: " + genome.getId());
             PreferenceManager.getInstance().remove(PreferenceManager.DEFAULT_GENOME_KEY);
         }
 
@@ -236,35 +243,49 @@ public class RulerPanel extends JPanel {
             return;
         }
 
-        for (String chrName : genome.getChromosomeNames()) {
+        final FontMetrics fontMetrics = g.getFontMetrics();
+        for (String chrName : chrNames) {
             Chromosome c = genome.getChromosome(chrName);
             int chrLength = c.getLength();
 
             int x = (int) (offset / (locationUnit * getViewContext().getScale()));
-
-            g.drawLine(x, getHeight() - 10, x, getHeight() - 2);
-
             int dw = (int) (chrLength / (locationUnit * getViewContext().getScale()));
-            int center = x + dw / 2;
-            String chrNumber = chrName.replace("chr", "");
-            int strWidth = g.getFontMetrics().stringWidth(chrNumber);
-            int strPosition = center - strWidth / 2;
+
+            // Dont draw very small chromosome & contigs in whole genome view
+            if (dw > 1) {
+
+                g.drawLine(x, getHeight() - 10, x, getHeight() - 2);
+
+                // Don't label chromosome if its width is < 5 pixels
+                if (dw > 5) {
+                    int center = x + dw / 2;
+
+                    String displayName = null;
+                    if (chrName.startsWith("gi|")) {
+                        displayName = Genome.getNCBIName(chrName);
+                    } else {
+                        displayName = chrName.replace("chr", "");
+                    }
+                     int strWidth = fontMetrics.stringWidth(displayName);
+                    int strPosition = center - strWidth / 2;
 
 
-            int y = (even ? getHeight() - 35 : getHeight() - 25);
-            g.drawString(chrNumber, strPosition, y);
+                    int y = (even ? getHeight() - 35 : getHeight() - 25);
+                    g.drawString(displayName, strPosition, y);
+                    int sw = (int) fontMetrics.getStringBounds(displayName, g).getWidth();
+                    Rectangle clickRect = new Rectangle(strPosition, y - 15, sw, 15);
+                    String tooltipText = "Jump to chromosome: " + chrName;
+                    chromosomeRects.add(new ClickLink(clickRect, chrName, tooltipText));
 
-            Rectangle clickRect = new Rectangle(strPosition, y - 15, 15, 15);
-            String tooltipText = "Jump to chromosome: " + chrName;
-            chromosomeRects.add(new ClickLink(clickRect, chrName, tooltipText));
-
-            even = !even;
+                    even = !even;
+                }
+            }
 
             offset += chrLength;
         }
     }
 
-    private String formatNumber(double position) {
+    public static String formatNumber(double position) {
 
         //NumberFormatter f = new NumberFormatter();
         DecimalFormat formatter = new DecimalFormat();
@@ -273,13 +294,13 @@ public class RulerPanel extends JPanel {
 
     }
 
-    TickSpacing findSpacing(long maxValue) {
+    public static TickSpacing findSpacing(ReferenceFrame frame, long maxValue) {
 
         if (maxValue < 10) {
             return new TickSpacing(1, "bp", 1);
         }
         // TODO -- hack, assumes location unit for whole genome is kilo-base
-        boolean scaleInKB = getViewContext().getChrName().equals(Globals.CHR_ALL);
+        boolean scaleInKB = frame.getChrName().equals(Globals.CHR_ALL);
 
 
         // Now man zeroes?
@@ -306,9 +327,9 @@ public class RulerPanel extends JPanel {
         }
     }
 
-    private void initialize() {
+    private void init() {
 
-        setBorder(BorderFactory.createLineBorder(new Color(0f, 0f, 0f, 0.2f)));
+        setBorder(BorderFactory.createLineBorder(UIConstants.TRACK_BORDER_GRAY));
 
         setCursor(Cursor.getDefaultCursor());
         if (isWholeGenomeView()) {
@@ -324,29 +345,42 @@ public class RulerPanel extends JPanel {
             @Override
             public void mouseClicked(MouseEvent evt) {
                 final MouseEvent e = evt;
+                setCursor(Cursor.getDefaultCursor());                
                 WaitCursorManager.CursorToken token = WaitCursorManager.showWaitCursor();
                 try {
 
                     if (!isWholeGenomeView()) {
-                        double newLocation = ViewContext.getInstance().getChromosomePosition(e.getX());
+                        double newLocation = frame.getChromosomePosition(e.getX());
                         getViewContext().centerOnLocation(newLocation);
                     } else {
-                        for (ClickLink link : chromosomeRects) {
+                        for (final ClickLink link : chromosomeRects) {
                             if (link.region.contains(e.getPoint())) {
-                                getViewContext().setChrName(link.value);
-                                getViewContext().recordHistory();
-                                // TODO -- get rid of this ugly reference to IGVMainFrame.theInstance
-                                IGVMainFrame.getInstance().chromosomeChangeEvent();
+                                NamedRunnable runnable = new NamedRunnable() {
+                                    final String chrName = link.value;
+
+                                    public void run() {
+
+                                        getViewContext().setChrName(chrName);
+                                        getViewContext().recordHistory();
+                                        // TODO -- get rid of this ugly reference to IGV.theInstance
+                                        IGV.getInstance().chromosomeChangeEvent(chrName);
+                                    }
+
+                                    public String getName() {
+                                        return "Select chromosome: " + chrName;
+                                    }
+                                };
+
+                                LongRunningTask.submit(runnable);
 
                                 return;
                             }
-                            setCursor(Cursor.getDefaultCursor());
-
                         }
                     }
                 } finally {
                     WaitCursorManager.removeWaitCursor(token);
                 }
+
             }
 
             @Override
@@ -413,15 +447,10 @@ public class RulerPanel extends JPanel {
 
     private void zoom() {
 
-        SwingWorker worker = new SwingWorker() {
 
-            WaitCursorManager.CursorToken token;
-
-            @Override
-            protected Object doInBackground() throws Exception {
-
-                token = WaitCursorManager.showWaitCursor();
-                final ViewContext vc = getViewContext();
+        NamedRunnable runnable = new NamedRunnable() {
+            public void run() {
+                final ReferenceFrame vc = getViewContext();
                 double s = vc.getChromosomePosition(dragStart);
                 double e = vc.getChromosomePosition(dragEnd);
                 if (e < s) {
@@ -437,7 +466,7 @@ public class RulerPanel extends JPanel {
 
                 String chr = null;
                 if (isWholeGenomeView()) {
-                    Genome genome = vc.getGenome();
+                    Genome genome = IGV.getInstance().getGenomeManager().getCurrentGenome();
                     Genome.ChromosomeCoordinate start = genome.getChromosomeCoordinate((int) s);
                     Genome.ChromosomeCoordinate end = genome.getChromosomeCoordinate((int) e);
 
@@ -453,19 +482,19 @@ public class RulerPanel extends JPanel {
 
                 vc.jumpTo(chr, (int) Math.min(s, e), (int) Math.max(s, e));
                 vc.recordHistory();
-                return null;
             }
 
-            @Override
-            protected void done() {
-                WaitCursorManager.removeWaitCursor(token);
+            public String getName() {
+                return "Rule panel zoom";
             }
         };
 
-        worker.execute();
+        LongRunningTask.submit(runnable);
+
     }
 
-    class TickSpacing {
+
+    public static class TickSpacing {
 
         private double majorTick;
         private double minorTick;
@@ -504,7 +533,7 @@ public class RulerPanel extends JPanel {
         }
     }
 
-    // TODO -- possibly generalize?
+// TODO -- possibly generalize?
 
     class ClickLink {
 
