@@ -1,19 +1,19 @@
 /*
- * Copyright (c) 2007-2010 by The Broad Institute, Inc. and the Massachusetts Institute of Technology.
- * All Rights Reserved.
+ * Copyright (c) 2007-2011 by The Broad Institute of MIT and Harvard.  All Rights Reserved.
  *
- * This software is licensed under the terms of the GNU Lesser General Public License (LGPL), Version 2.1 which
- * is available at http://www.opensource.org/licenses/lgpl-2.1.php.
+ * This software is licensed under the terms of the GNU Lesser General Public License (LGPL),
+ * Version 2.1 which is available at http://www.opensource.org/licenses/lgpl-2.1.php.
  *
- * THE SOFTWARE IS PROVIDED "AS IS." THE BROAD AND MIT MAKE NO REPRESENTATIONS OR WARRANTIES OF
- * ANY KIND CONCERNING THE SOFTWARE, EXPRESS OR IMPLIED, INCLUDING, WITHOUT LIMITATION, WARRANTIES
- * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NONINFRINGEMENT, OR THE ABSENCE OF LATENT
- * OR OTHER DEFECTS, WHETHER OR NOT DISCOVERABLE.  IN NO EVENT SHALL THE BROAD OR MIT, OR THEIR
- * RESPECTIVE TRUSTEES, DIRECTORS, OFFICERS, EMPLOYEES, AND AFFILIATES BE LIABLE FOR ANY DAMAGES OF
- * ANY KIND, INCLUDING, WITHOUT LIMITATION, INCIDENTAL OR CONSEQUENTIAL DAMAGES, ECONOMIC
- * DAMAGES OR INJURY TO PROPERTY AND LOST PROFITS, REGARDLESS OF WHETHER THE BROAD OR MIT SHALL
- * BE ADVISED, SHALL HAVE OTHER REASON TO KNOW, OR IN FACT SHALL KNOW OF THE POSSIBILITY OF THE
- * FOREGOING.
+ * THE SOFTWARE IS PROVIDED "AS IS." THE BROAD AND MIT MAKE NO REPRESENTATIONS OR
+ * WARRANTES OF ANY KIND CONCERNING THE SOFTWARE, EXPRESS OR IMPLIED, INCLUDING,
+ * WITHOUT LIMITATION, WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
+ * PURPOSE, NONINFRINGEMENT, OR THE ABSENCE OF LATENT OR OTHER DEFECTS, WHETHER
+ * OR NOT DISCOVERABLE.  IN NO EVENT SHALL THE BROAD OR MIT, OR THEIR RESPECTIVE
+ * TRUSTEES, DIRECTORS, OFFICERS, EMPLOYEES, AND AFFILIATES BE LIABLE FOR ANY DAMAGES
+ * OF ANY KIND, INCLUDING, WITHOUT LIMITATION, INCIDENTAL OR CONSEQUENTIAL DAMAGES,
+ * ECONOMIC DAMAGES OR INJURY TO PROPERTY AND LOST PROFITS, REGARDLESS OF WHETHER
+ * THE BROAD OR MIT SHALL BE ADVISED, SHALL HAVE OTHER REASON TO KNOW, OR IN FACT
+ * SHALL KNOW OF THE POSSIBILITY OF THE FOREGOING.
  */
 /*
  * TrackPanel.java
@@ -27,10 +27,12 @@ package org.broad.igv.ui.panel;
 
 
 import org.apache.log4j.Logger;
+import org.broad.igv.PreferenceManager;
 import org.broad.igv.track.Track;
+import org.broad.igv.track.TrackClickEvent;
 import org.broad.igv.track.TrackGroup;
-import org.broad.igv.track.TrackMenuUtils;
-import org.broad.igv.ui.IGVMainFrame;
+import org.broad.igv.ui.FontManager;
+import org.broad.igv.ui.IGV;
 import org.broad.igv.ui.UIConstants;
 import org.broad.igv.ui.dnd.AbstractGhostDropManager;
 import org.broad.igv.ui.dnd.GhostDropEvent;
@@ -44,48 +46,21 @@ import javax.swing.event.MouseInputAdapter;
 import java.awt.*;
 import java.awt.event.*;
 import java.awt.image.BufferedImage;
-import java.util.ArrayList;
-import java.util.Collection;
-import java.util.Iterator;
+import java.util.*;
 import java.util.List;
 
 /**
  * @author jrobinso
  */
-public class TrackNamePanel extends TrackPanelComponent implements AdjustmentListener {
+public class TrackNamePanel extends TrackPanelComponent implements Paintable {
 
     private static Logger log = Logger.getLogger(TrackNamePanel.class);
 
 
-    // TODO -- this use of a static is really bad,  bugs and memory leaks waiting to happen.  Redesign this.
-    static List<DropListener> dropListeners = new ArrayList();
-
-
-    static void addGhostDropListener(DropListener listener) {
-        if (listener != null) {
-            dropListeners.add(listener);
-        }
-    }
-
-    public static void removeDropListenerFor(TrackNamePanel panel) {
-        List<DropListener> removeThese = new ArrayList();
-        for (DropListener dl : dropListeners) {
-            if (dl.panel == panel) {
-                removeThese.add(dl);
-            }
-        }
-        dropListeners.removeAll(removeThese);
-    }
-
-
     List<GroupExtent> groupExtents = new ArrayList();
-
     BufferedImage dndImage = null;
-
     TrackGroup selectedGroup = null;
-
     boolean showGroupNames = true;
-
     boolean showSampleNamesWhenGrouped = false;
 
 
@@ -95,13 +70,11 @@ public class TrackNamePanel extends TrackPanelComponent implements AdjustmentLis
     }
 
     Collection<TrackGroup> getGroups() {
-        TrackPanel dataTrackView = (TrackPanel) getParent();
-        return dataTrackView.getGroups();
+        return getTrackPanel().getGroups();
     }
 
     private boolean isGrouped() {
-        Collection<TrackGroup> groups = getGroups();
-        return groups.size() > 1;
+        return getGroups().size() > 1;
     }
 
 
@@ -109,28 +82,46 @@ public class TrackNamePanel extends TrackPanelComponent implements AdjustmentLis
     public void paintComponent(Graphics g) {
 
         super.paintComponent(g);
-
         removeMousableRegions();
+        Rectangle visibleRect = getVisibleRect();
+        paintImpl(g, visibleRect);
+    }
 
+
+    public void paintOffscreen(Graphics2D g, Rectangle rect) {
+        g.setColor(Color.white);
+        g.fill(rect);
+        paintImpl(g, rect);
+        super.paintBorder(g);
+    }
+
+
+    private void paintImpl(Graphics g, Rectangle visibleRect) {
         // Get available tracks
         Collection<TrackGroup> groups = getGroups();
         boolean isGrouped = groups.size() > 1;
 
-        Rectangle visibleRect = getVisibleRect();
 
         if (!groups.isEmpty()) {
             final Graphics2D graphics2D = (Graphics2D) g.create();
             graphics2D.setColor(Color.BLACK);
 
             final Graphics2D greyGraphics = (Graphics2D) g.create();
-            greyGraphics.setColor(UIConstants.VERY_LIGHT_GRAY);
+            greyGraphics.setColor(UIConstants.ZOOMED_OUT_COLOR);
 
             int regionY = 0;
 
             groupExtents.clear();
+
+            Rectangle clipRect = g.getClipBounds();
+
             for (Iterator<TrackGroup> groupIter = groups.iterator(); groupIter.hasNext();) {
                 TrackGroup group = groupIter.next();
 
+                if (regionY > clipRect.getMaxY()) {
+                    break;
+                }
+
                 if (group.isVisible()) {
                     if (isGrouped) {
                         if (regionY + UIConstants.groupGap >= visibleRect.y && regionY < visibleRect.getMaxY()) {
@@ -145,7 +136,7 @@ public class TrackNamePanel extends TrackPanelComponent implements AdjustmentLis
                     }
 
                     int y = regionY;
-                    regionY = printTrackNames(group, visibleRect, graphics2D, 0, regionY);
+                    regionY = printTrackNames(group, visibleRect, clipRect, graphics2D, 0, regionY);
 
                     if (isGrouped) {
                         int h = group.getHeight();
@@ -180,15 +171,22 @@ public class TrackNamePanel extends TrackPanelComponent implements AdjustmentLis
 
     }
 
-    private int printTrackNames(TrackGroup group, Rectangle visibleRect, Graphics2D graphics2D, int regionX, int regionY) {
+    private int printTrackNames(TrackGroup group, Rectangle visibleRect, Rectangle clipRect,
+                                Graphics2D graphics2D, int regionX, int regionY) {
+
 
         List<Track> tmp = new ArrayList(group.getTracks());
+        final Color backgroundColor = PreferenceManager.getInstance().getAsColor(PreferenceManager.BACKGROUND_COLOR);
+        graphics2D.setBackground(backgroundColor);
+        graphics2D.clearRect(visibleRect.x, visibleRect.y, visibleRect.width, visibleRect.height);
+
         for (Track track : tmp) {
+            if (track == null) continue;
             track.setTop(regionY);
             int trackHeight = track.getHeight();
             if (track.isVisible()) {
 
-                if (regionY + trackHeight >= visibleRect.y && regionY < visibleRect.getMaxY()) {
+                if (regionY + trackHeight >= clipRect.y && regionY < clipRect.getMaxY()) {
                     int width = getWidth();
                     int height = track.getHeight();
 
@@ -197,6 +195,16 @@ public class TrackNamePanel extends TrackPanelComponent implements AdjustmentLis
 
                     if (!isGrouped() || showSampleNamesWhenGrouped) {
                         Rectangle rect = new Rectangle(regionX, regionY, width, height);
+                        //Graphics2D g2D = graphics; //(Graphics2D) graphics.create();
+                        if (track.isSelected()) {
+                            graphics2D.setBackground(Color.LIGHT_GRAY);
+                            graphics2D.clearRect(rect.x, rect.y, rect.width, rect.height);
+                        }
+                        else {
+                           graphics2D.setBackground(backgroundColor);
+                        }
+                        graphics2D.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,
+                                RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
                         track.renderName(graphics2D, rect, visibleRect);
                     }
 
@@ -229,58 +237,36 @@ public class TrackNamePanel extends TrackPanelComponent implements AdjustmentLis
 
 
     @Override
-    protected void openPopupMenu(MouseEvent e) {
-
-        // If there is a single selected track give it a chance to handle the click
-        Collection<Track> selectedTracs = getSelectedTracks();
-        String title = getPopupMenuTitle(0, 0);
-
-        JPopupMenu menu = TrackMenuUtils.getEmptyPopup(title);
+    protected void openPopupMenu(TrackClickEvent te) {
 
+        ArrayList<Component> extraItems = null;
         if (isGrouped()) {
+            extraItems = new ArrayList();
+
             final JMenuItem item = new JCheckBoxMenuItem("Show group names");
             item.setSelected(showGroupNames);
             item.addActionListener(new ActionListener() {
                 public void actionPerformed(ActionEvent e) {
                     showGroupNames = item.isSelected();
-                    clearTrackSelections();
                     repaint();
                 }
             });
-            menu.add(item);
+            extraItems.add(item);
 
             final JMenuItem item2 = new JCheckBoxMenuItem("Show sample names");
             item2.setSelected(showSampleNamesWhenGrouped);
             item2.addActionListener(new ActionListener() {
                 public void actionPerformed(ActionEvent e) {
                     showSampleNamesWhenGrouped = item2.isSelected();
-                    clearTrackSelections();
                     repaint();
                 }
             });
-            menu.add(item2);
-            menu.addSeparator();
+            extraItems.add(item2);
         }
 
-        TrackMenuUtils.addStandardItems(menu, selectedTracs, e);
-
-        menu.show(e.getComponent(), e.getX(), e.getY());
-
-    }
+        super.openPopupMenu(te, extraItems);
 
 
-    public String getPopupMenuTitle(int x, int y) {
-
-        Collection<Track> tracks = getSelectedTracks();
-
-        String popupTitle;
-        if (tracks.size() == 1) {
-            popupTitle = tracks.iterator().next().getName();
-        } else {
-            popupTitle = "Total Tracks Selected: " + tracks.size();
-        }
-
-        return popupTitle;
     }
 
 
@@ -296,19 +282,6 @@ public class TrackNamePanel extends TrackPanelComponent implements AdjustmentLis
         return "";
     }
 
-    /**
-     * Listener for scroll pane events
-     *
-     * @param evt
-     */
-    public void adjustmentValueChanged(AdjustmentEvent evt) {
-        if (evt.getValueIsAdjusting()) {
-            // The user is dragging the knob
-            return;
-        }
-        repaint();
-
-    }
 
     private synchronized void createDnDImage() {
         dndImage = new BufferedImage(getWidth(), 2, BufferedImage.TYPE_INT_ARGB);
@@ -325,7 +298,7 @@ public class TrackNamePanel extends TrackPanelComponent implements AdjustmentLis
                 Collection<Track> clickedTracks = mouseRegion.getTracks();
                 if (clickedTracks != null && clickedTracks.size() > 0) {
                     Track t = clickedTracks.iterator().next();
-                    IGVMainFrame.getInstance().getTrackManager().shiftSelectTracks(t);
+                    IGV.getInstance().getTrackManager().shiftSelectTracks(t);
                 }
                 return;
             }
@@ -342,6 +315,7 @@ public class TrackNamePanel extends TrackPanelComponent implements AdjustmentLis
         return null;
     }
 
+
     /**
      * Mouse adapter for the track name panel.  Supports multiple selection,
      * popup menu, and drag & drop within or between name panels.
@@ -371,12 +345,19 @@ public class TrackNamePanel extends TrackPanelComponent implements AdjustmentLis
 
             if (e.isPopupTrigger()) {
                 if (isGrouped) {
-
+                    clearTrackSelections();
+                    TrackGroup g = getGroup(e.getY());
+                    if (g == selectedGroup) {
+                        selectedGroup = null;
+                    } else {
+                        selectGroup(getGroup(e.getY()));
+                    }
                 } else if (!isTrackSelected(e)) {
                     clearTrackSelections();
                     selectTracks(e);
                 }
-                openPopupMenu(e);
+                TrackClickEvent te = new TrackClickEvent(e, null);
+                openPopupMenu(te);
             } // meta (mac) or control,  toggle selection]
             else if (e.getButton() == MouseEvent.BUTTON1) {
 
@@ -408,7 +389,7 @@ public class TrackNamePanel extends TrackPanelComponent implements AdjustmentLis
             }
 
 
-            IGVMainFrame.getInstance().repaintNamePanels();
+            IGV.getInstance().repaintNamePanels();
 
         }
 
@@ -423,8 +404,8 @@ public class TrackNamePanel extends TrackPanelComponent implements AdjustmentLis
 
                 Component c = e.getComponent();
 
-                IGVMainFrame.getInstance().endDnD();
-                GhostGlassPane glassPane = IGVMainFrame.getInstance().getDnDGlassPane();
+                IGV.getInstance().endDnD();
+                GhostGlassPane glassPane = IGV.getInstance().getDnDGlassPane();
 
                 Point p = (Point) e.getPoint().clone();
                 SwingUtilities.convertPointToScreen(p, c);
@@ -450,13 +431,14 @@ public class TrackNamePanel extends TrackPanelComponent implements AdjustmentLis
             }
 
             if (e.isPopupTrigger()) {
-                openPopupMenu(e);
+                TrackClickEvent te = new TrackClickEvent(e, null);
+                openPopupMenu(te);
             } else {
                 if (!isDragging && !e.isMetaDown() && !e.isControlDown() &&
                         !e.isShiftDown()) {
                     clearTrackSelections();
                     selectTracks(e);
-                    IGVMainFrame.getInstance().repaintNamePanels();
+                    IGV.getInstance().repaintNamePanels();
                 }
             }
 
@@ -476,20 +458,23 @@ public class TrackNamePanel extends TrackPanelComponent implements AdjustmentLis
             }
             if (!isDragging) {
 
-                if(e.getPoint().distance(dragStart) < 5) {
+                if (dragStart == null) {
+                    dragStart = e.getPoint();
+                    return;
+                } else if (e.getPoint().distance(dragStart) < 5) {
                     return;
                 }
 
                 dragStart.x = getWidth() / 2;
-                IGVMainFrame.getInstance().startDnD();
+                IGV.getInstance().startDnD();
 
                 if (dndImage == null) {
                     createDnDImage();
                 }
-                IGVMainFrame.getInstance().getDnDGlassPane().setImage(dndImage);
+                IGV.getInstance().getDnDGlassPane().setImage(dndImage);
                 isDragging = true;
                 dragTracks.clear();
-                dragTracks.addAll(IGVMainFrame.getInstance().getTrackManager().getSelectedTracks());
+                dragTracks.addAll(IGV.getInstance().getTrackManager().getSelectedTracks());
 
 
                 if (getGroups().size() > 0) {
@@ -503,7 +488,7 @@ public class TrackNamePanel extends TrackPanelComponent implements AdjustmentLis
             }
             if (isDragging) {
 
-                final GhostGlassPane glassPane = IGVMainFrame.getInstance().getDnDGlassPane();
+                final GhostGlassPane glassPane = IGV.getInstance().getDnDGlassPane();
 
                 Point p = (Point) e.getPoint().clone();
                 p.x = getWidth() / 2;
@@ -523,12 +508,47 @@ public class TrackNamePanel extends TrackPanelComponent implements AdjustmentLis
             }
         }
 
+        @Override
         public void mouseMoved(MouseEvent e) {
             int x = e.getX();
             int y = e.getY();
             setToolTipText(getTooltipTextForLocation(x, y));
         }
 
+
+        /**
+         * Mouse was clicked.  Delegate single-click action to the track(s) clicked on.   We won't know if this
+         * is a double click or not until the double-click interval has passed, so defer the action with a
+         * TimerTask.  If a second click arrives it will be canceled.
+         *
+         * @param e
+         */
+        @Override
+        public void mouseClicked(final MouseEvent e) {
+
+            // If this is the second click of a double click, cancel the scheduled single click task.
+            if (e.getClickCount() > 1) {
+                clickScheduler.cancelClickTask();
+                return;
+            }
+
+            TimerTask clickTask = new TimerTask() {
+
+                @Override
+                public void run() {
+                    for (MouseableRegion mouseRegion : trackRegions) {
+                        if (mouseRegion.containsPoint(e.getX(), e.getY())) {
+                            for (Track t : mouseRegion.getTracks()) {
+                                t.handleNameClick(e);
+                            }
+                            return;
+                        }
+                    }//To change body of implemented methods use File | Settings | File Templates.
+                }
+            };
+            clickScheduler.scheduleClickTask(clickTask);
+        }
+
         protected void fireGhostDropEvent(GhostDropEvent evt) {
             Iterator it = TrackNamePanel.dropListeners.iterator();
             while (it.hasNext()) {
@@ -545,6 +565,7 @@ public class TrackNamePanel extends TrackPanelComponent implements AdjustmentLis
         public DropListener(TrackNamePanel target) {
             super(target);
             this.panel = target;
+
         }
 
         public void ghostDropped(GhostDropEvent e) {
@@ -607,7 +628,6 @@ public class TrackNamePanel extends TrackPanelComponent implements AdjustmentLis
             }
             view.moveSelectedTracksTo(tracks, dropTrack, before);
 
-            IGVMainFrame.getInstance().doResizeTrackPanels();
 
         }
     }
@@ -650,4 +670,26 @@ public class TrackNamePanel extends TrackPanelComponent implements AdjustmentLis
     }
 
 
+    // Track D&D support follows
+    // TODO -- this use of a static is really bad,  bugs and memory leaks waiting to happen.  Redesign this.
+    static List<DropListener> dropListeners = new ArrayList();
+
+
+    private static void addGhostDropListener(DropListener listener) {
+        if (listener != null) {
+            dropListeners.add(listener);
+        }
+    }
+
+    public static void removeDropListenerFor(TrackNamePanel panel) {
+        List<DropListener> removeThese = new ArrayList();
+        for (DropListener dl : dropListeners) {
+            if (dl.panel == panel) {
+                removeThese.add(dl);
+            }
+        }
+        dropListeners.removeAll(removeThese);
+    }
+
+
 }
