@@ -1,48 +1,49 @@
 /*
- * Copyright (c) 2007-2010 by The Broad Institute, Inc. and the Massachusetts Institute of Technology.
- * All Rights Reserved.
+ * Copyright (c) 2007-2011 by The Broad Institute of MIT and Harvard.  All Rights Reserved.
  *
- * This software is licensed under the terms of the GNU Lesser General Public License (LGPL), Version 2.1 which
- * is available at http://www.opensource.org/licenses/lgpl-2.1.php.
+ * This software is licensed under the terms of the GNU Lesser General Public License (LGPL),
+ * Version 2.1 which is available at http://www.opensource.org/licenses/lgpl-2.1.php.
  *
- * THE SOFTWARE IS PROVIDED "AS IS." THE BROAD AND MIT MAKE NO REPRESENTATIONS OR WARRANTIES OF
- * ANY KIND CONCERNING THE SOFTWARE, EXPRESS OR IMPLIED, INCLUDING, WITHOUT LIMITATION, WARRANTIES
- * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NONINFRINGEMENT, OR THE ABSENCE OF LATENT
- * OR OTHER DEFECTS, WHETHER OR NOT DISCOVERABLE.  IN NO EVENT SHALL THE BROAD OR MIT, OR THEIR
- * RESPECTIVE TRUSTEES, DIRECTORS, OFFICERS, EMPLOYEES, AND AFFILIATES BE LIABLE FOR ANY DAMAGES OF
- * ANY KIND, INCLUDING, WITHOUT LIMITATION, INCIDENTAL OR CONSEQUENTIAL DAMAGES, ECONOMIC
- * DAMAGES OR INJURY TO PROPERTY AND LOST PROFITS, REGARDLESS OF WHETHER THE BROAD OR MIT SHALL
- * BE ADVISED, SHALL HAVE OTHER REASON TO KNOW, OR IN FACT SHALL KNOW OF THE POSSIBILITY OF THE
- * FOREGOING.
+ * THE SOFTWARE IS PROVIDED "AS IS." THE BROAD AND MIT MAKE NO REPRESENTATIONS OR
+ * WARRANTES OF ANY KIND CONCERNING THE SOFTWARE, EXPRESS OR IMPLIED, INCLUDING,
+ * WITHOUT LIMITATION, WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
+ * PURPOSE, NONINFRINGEMENT, OR THE ABSENCE OF LATENT OR OTHER DEFECTS, WHETHER
+ * OR NOT DISCOVERABLE.  IN NO EVENT SHALL THE BROAD OR MIT, OR THEIR RESPECTIVE
+ * TRUSTEES, DIRECTORS, OFFICERS, EMPLOYEES, AND AFFILIATES BE LIABLE FOR ANY DAMAGES
+ * OF ANY KIND, INCLUDING, WITHOUT LIMITATION, INCIDENTAL OR CONSEQUENTIAL DAMAGES,
+ * ECONOMIC DAMAGES OR INJURY TO PROPERTY AND LOST PROFITS, REGARDLESS OF WHETHER
+ * THE BROAD OR MIT SHALL BE ADVISED, SHALL HAVE OTHER REASON TO KNOW, OR IN FACT
+ * SHALL KNOW OF THE POSSIBILITY OF THE FOREGOING.
  */
 package org.broad.igv.ui.panel;
 
 
+import org.apache.log4j.Logger;
 import org.broad.igv.PreferenceManager;
-import org.broad.igv.session.ViewContext;
+import org.broad.igv.feature.RegionOfInterest;
 import org.broad.igv.track.RegionScoreType;
 import org.broad.igv.track.Track;
 import org.broad.igv.track.TrackGroup;
-import org.broad.igv.ui.IGVMainFrame;
-import org.broad.igv.ui.RegionOfInterest;
+import org.broad.igv.ui.IGV;
 import org.broad.igv.ui.UIConstants;
-import org.netbeans.lib.awtextra.AbsoluteConstraints;
-import org.netbeans.lib.awtextra.AbsoluteLayout;
+import org.broad.igv.ui.util.UIUtilities;
 
+import java.awt.*;
 import java.util.*;
+import java.util.List;
 
 /**
  * @author eflakes
  */
 public class TrackPanel extends IGVPanel {
 
-    public static int X_SPACING_BETWEEN_COMPONENTS = 10;
-
+    private static Logger log = Logger.getLogger(TrackPanel.class);
 
     private String name = null;
+    //private JPanel grabPanel;
     private TrackNamePanel namePanel;
     private AttributePanel attributePanel;
-    private DataPanel dataPanel;
+    private DataPanelContainer dataPanelContainer;
     private String groupAttribute;
     int trackCountEstimate = 0;  // <= used to size array list, not neccesarily precise
 
@@ -56,7 +57,8 @@ public class TrackPanel extends IGVPanel {
      *
      * @param name
      */
-    public TrackPanel(String name) {
+    public TrackPanel(String name, MainPanel mainPanel) {
+        super(mainPanel);
         this.name = name;
         TrackGroup nullGroup = new TrackGroup();
         nullGroup.setDrawBorder(false);
@@ -67,35 +69,46 @@ public class TrackPanel extends IGVPanel {
 
 
     private void init() {
-        setLayout(new AbsoluteLayout());
+
         namePanel = new TrackNamePanel(this);
         attributePanel = new AttributePanel(this);
-        dataPanel = new DataPanel(this);
-        add(namePanel, new AbsoluteConstraints(0, 0, 150, 230));
-        add(attributePanel, new AbsoluteConstraints(160, 0, 120, 230));
-        add(dataPanel, new AbsoluteConstraints(290, 0, 710, 230));
+        dataPanelContainer = new DataPanelContainer(this);
+
+        //add(grabPanel);
+        add(namePanel);
+        add(attributePanel);
+        add(dataPanelContainer);
+
+    }
+
+
+    public void createDataPanels() {
+        dataPanelContainer.createDataPanels();
+    }
+
+    @Override
+    public void setBackground(Color color) {
+        super.setBackground(color);
+        if (namePanel != null) {
+            namePanel.setBackground(color);
+            attributePanel.setBackground(color);
+            dataPanelContainer.setBackground(color);
+        }
     }
 
 
-    /**
-     * @return the namePanel
-     */
     public TrackNamePanel getNamePanel() {
         return namePanel;
     }
 
-    /**
-     * @return the attributePanel
-     */
+
     public AttributePanel getAttributePanel() {
         return attributePanel;
     }
 
-    /**
-     * @return the dataPanel
-     */
-    public DataPanel getDataPanel() {
-        return dataPanel;
+
+    public DataPanelContainer getDataPanelContainer() {
+        return dataPanelContainer;
     }
 
 
@@ -130,7 +143,7 @@ public class TrackPanel extends IGVPanel {
         int count = 0;
         for (TrackGroup tg : trackGroups) {
             for (Track t : tg.getTracks()) {
-                if (t.isVisible()) {
+                if (t != null && t.isVisible()) {
                     count++;
                 }
             }
@@ -187,12 +200,12 @@ public class TrackPanel extends IGVPanel {
 
     public void moveGroup(TrackGroup group, int index) {
 
-        if(index > trackGroups.indexOf(group)) {
+        if (index > trackGroups.indexOf(group)) {
             index--;
         }
         trackGroups.remove(group);
         if (index >= trackGroups.size()) {
-           trackGroups.add(group);
+            trackGroups.add(group);
         } else {
             trackGroups.add(index, group);
         }
@@ -268,20 +281,21 @@ public class TrackPanel extends IGVPanel {
      * @param region
      * @param type
      */
-    public void sortByRegionsScore(final RegionOfInterest region, final RegionScoreType type) {
+    public void sortByRegionsScore(final RegionOfInterest region, final RegionScoreType type,
+                                   final ReferenceFrame frame) {
 
-        boolean useLinkedSorting = PreferenceManager.getInstance().isLinkedSortingEnabled();
-        String linkingAtt = IGVMainFrame.getInstance().getSession().getOverlayAttribute();
+        boolean useLinkedSorting = PreferenceManager.getInstance().getAsBoolean(PreferenceManager.ENABLE_LINKED_SORTING);
+        String linkingAtt = IGV.getInstance().getSession().getOverlayAttribute();
 
-        sortGroupsByRegionScore(trackGroups, region, type);
+        sortGroupsByRegionScore(trackGroups, region, type, frame);
 
         for (TrackGroup group : trackGroups) {
             // If there is a non-null linking attribute
             // Segregate tracks into 2 sub-groups, those matching the score type and those that do not
             if (linkingAtt != null && !useLinkedSorting) {
-                group.sortByRegionScore(region, type);
+                group.sortByRegionScore(region, type, frame);
             } else {
-                group.sortGroup(region, linkingAtt, type);
+                group.sortGroup(region, linkingAtt, type, frame);
             }
 
         }
@@ -296,17 +310,18 @@ public class TrackPanel extends IGVPanel {
      */
     private void sortGroupsByRegionScore(List<TrackGroup> groups,
                                          final RegionOfInterest region,
-                                         final RegionScoreType type) {
+                                         final RegionScoreType type,
+                                         final ReferenceFrame frame) {
         if ((groups != null) && (region != null) && !groups.isEmpty()) {
-            final int zoom = Math.max(0, ViewContext.getInstance().getZoom());
+            final int zoom = Math.max(0, frame.getZoom());
             final String chr = region.getChr();
             final int start = region.getStart();
             final int end = region.getEnd();
             Comparator<TrackGroup> c = new Comparator<TrackGroup>() {
 
                 public int compare(TrackGroup group1, TrackGroup group2) {
-                    float s1 = group1.getRegionScore(chr, start, end, zoom, type);
-                    float s2 = group2.getRegionScore(chr, start, end, zoom, type);
+                    float s1 = group1.getRegionScore(chr, start, end, zoom, type, frame);
+                    float s2 = group2.getRegionScore(chr, start, end, zoom, type, frame);
 
                     if (s2 > s1) {
                         return 1;
@@ -326,28 +341,28 @@ public class TrackPanel extends IGVPanel {
 
 
     /**
-     * Replace current gene track with new one.  This is called upon switching genomes.
+     * This is called upon switching genomes to replace the gene and sequence tracks
      *
-     * @param newGeneTrack
+     * @param newTrack
      * @return true if gene track is found.
      */
-    public boolean setGeneTrack(Track geneTrack, Track newGeneTrack) {
+    public boolean replaceTrack(Track oldTrack, Track newTrack) {
 
-        boolean foundGeneTrack = false;
+        boolean foundTrack = false;
 
         for (TrackGroup g : trackGroups) {
-            if (g.contains(geneTrack)) {
-                int geneTrackIndex = g.indexOf(geneTrack);
-                g.remove(geneTrack);
-                geneTrackIndex = Math.min(g.size(), geneTrackIndex);
-                g.add(geneTrackIndex, newGeneTrack);
-                foundGeneTrack = true;
+            if (g.contains(oldTrack)) {
+                int idx = g.indexOf(oldTrack);
+                g.remove(oldTrack);
+                idx = Math.min(g.size(), idx);
+                if (newTrack != null) {
+                    g.add(idx, newTrack);
+                }
+                foundTrack = true;
             }
         }
 
-
-        return foundGeneTrack;
-
+        return foundTrack;
     }
 
     public void removeTracks(Collection<Track> tracksToRemove) {
@@ -383,24 +398,64 @@ public class TrackPanel extends IGVPanel {
     public int getPreferredPanelHeight() {
         int height = 0;
 
-        // Copy to prevent concurrent modification exception.  Should we synchronize on the
-        // groups collection instead?
         Collection<TrackGroup> groups = getGroups();
 
         if (groups.size() > 1) {
             height += UIConstants.groupGap;
         }
 
-        for (Iterator<TrackGroup> groupIter = groups.iterator(); groupIter.hasNext();) {
-            TrackGroup group = groupIter.next();
-            if (group != null && group.isVisible()) {
-                if (groups.size() > 1) {
-                    height += UIConstants.groupGap;
+        synchronized (groups) {
+
+            for (Iterator<TrackGroup> groupIter = groups.iterator(); groupIter.hasNext();) {
+                TrackGroup group = groupIter.next();
+                if (group != null && group.isVisible()) {
+                    if (groups.size() > 1) {
+                        height += UIConstants.groupGap;
+                    }
+                    height += group.getHeight();
                 }
-                height += group.getPreferredHeight();
             }
         }
 
         return Math.max(20, height);
     }
+
+    @Override
+    public Dimension getPreferredSize() {
+        Dimension dim = super.getPreferredSize();
+        dim.height = getPreferredPanelHeight();
+        return dim;
+    }
+
+    @Override
+    public void paintOffscreen(Graphics2D g, Rectangle rect) {
+
+        int h = rect.height;
+
+        Component[] children = getComponents();
+        // name panel starts at offset=0
+
+        g.translate(mainPanel.getNamePanelX(), 0);
+
+        Rectangle nameRect = new Rectangle(children[0].getBounds());
+        nameRect.height = h;
+        g.setClip(nameRect);
+        ((Paintable) children[0]).paintOffscreen(g, nameRect);
+
+        int dx = mainPanel.getAttributePanelX() - mainPanel.getNamePanelX();
+        g.translate(dx, 0);
+        Rectangle attRect = new Rectangle(0, 0, children[1].getWidth(), h);
+        g.setClip(attRect);
+        ((Paintable) children[1]).paintOffscreen(g, attRect);
+
+        dx = mainPanel.getDataPanelX() - mainPanel.getAttributePanelX();
+        g.translate(dx, 0);
+        Rectangle dataRect = new Rectangle(0, 0, mainPanel.getDataPanelWidth(), h);
+        g.setClip(dataRect);
+        ((Paintable) children[2]).paintOffscreen(g, dataRect);
+
+
+        super.paintBorder(g);
+
+    }
 }
