@@ -1,19 +1,19 @@
 /*
- * Copyright (c) 2007-2010 by The Broad Institute, Inc. and the Massachusetts Institute of Technology.
- * All Rights Reserved.
+ * Copyright (c) 2007-2011 by The Broad Institute of MIT and Harvard.  All Rights Reserved.
  *
- * This software is licensed under the terms of the GNU Lesser General Public License (LGPL), Version 2.1 which
- * is available at http://www.opensource.org/licenses/lgpl-2.1.php.
+ * This software is licensed under the terms of the GNU Lesser General Public License (LGPL),
+ * Version 2.1 which is available at http://www.opensource.org/licenses/lgpl-2.1.php.
  *
- * THE SOFTWARE IS PROVIDED "AS IS." THE BROAD AND MIT MAKE NO REPRESENTATIONS OR WARRANTIES OF
- * ANY KIND CONCERNING THE SOFTWARE, EXPRESS OR IMPLIED, INCLUDING, WITHOUT LIMITATION, WARRANTIES
- * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NONINFRINGEMENT, OR THE ABSENCE OF LATENT
- * OR OTHER DEFECTS, WHETHER OR NOT DISCOVERABLE.  IN NO EVENT SHALL THE BROAD OR MIT, OR THEIR
- * RESPECTIVE TRUSTEES, DIRECTORS, OFFICERS, EMPLOYEES, AND AFFILIATES BE LIABLE FOR ANY DAMAGES OF
- * ANY KIND, INCLUDING, WITHOUT LIMITATION, INCIDENTAL OR CONSEQUENTIAL DAMAGES, ECONOMIC
- * DAMAGES OR INJURY TO PROPERTY AND LOST PROFITS, REGARDLESS OF WHETHER THE BROAD OR MIT SHALL
- * BE ADVISED, SHALL HAVE OTHER REASON TO KNOW, OR IN FACT SHALL KNOW OF THE POSSIBILITY OF THE
- * FOREGOING.
+ * THE SOFTWARE IS PROVIDED "AS IS." THE BROAD AND MIT MAKE NO REPRESENTATIONS OR
+ * WARRANTES OF ANY KIND CONCERNING THE SOFTWARE, EXPRESS OR IMPLIED, INCLUDING,
+ * WITHOUT LIMITATION, WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
+ * PURPOSE, NONINFRINGEMENT, OR THE ABSENCE OF LATENT OR OTHER DEFECTS, WHETHER
+ * OR NOT DISCOVERABLE.  IN NO EVENT SHALL THE BROAD OR MIT, OR THEIR RESPECTIVE
+ * TRUSTEES, DIRECTORS, OFFICERS, EMPLOYEES, AND AFFILIATES BE LIABLE FOR ANY DAMAGES
+ * OF ANY KIND, INCLUDING, WITHOUT LIMITATION, INCIDENTAL OR CONSEQUENTIAL DAMAGES,
+ * ECONOMIC DAMAGES OR INJURY TO PROPERTY AND LOST PROFITS, REGARDLESS OF WHETHER
+ * THE BROAD OR MIT SHALL BE ADVISED, SHALL HAVE OTHER REASON TO KNOW, OR IN FACT
+ * SHALL KNOW OF THE POSSIBILITY OF THE FOREGOING.
  */
 package org.broad.igv.util;
 
@@ -21,15 +21,13 @@ package org.broad.igv.util;
 
 import org.apache.log4j.Logger;
 import org.broad.igv.feature.Exon;
-import org.broad.igv.feature.Feature;
+import org.broad.igv.feature.IGVFeature;
 import org.broad.igv.feature.Strand;
-import org.broad.igv.renderer.BarChartRenderer;
-import org.broad.igv.renderer.HeatmapRenderer;
-import org.broad.igv.renderer.LineplotRenderer;
-import org.broad.igv.renderer.ScatterplotRenderer;
+import org.broad.igv.renderer.*;
 import org.broad.igv.track.TrackProperties;
 import org.broad.igv.track.WindowFunction;
 import org.broad.igv.ui.util.MessageUtils;
+import org.broad.tribble.readers.AsciiLineReader;
 
 import java.awt.*;
 import java.io.*;
@@ -50,10 +48,9 @@ public class ParsingUtils {
             throws FileNotFoundException, IOException {
         BufferedReader reader = null;
 
-        if (pathOrUrl.startsWith("http:") || pathOrUrl.startsWith("https:") ||
-                pathOrUrl.startsWith("file:")) {
+        if (IGVHttpClientUtils.isURL(pathOrUrl)) {
             URL url = new URL(pathOrUrl);
-            reader = new BufferedReader(new InputStreamReader(IGVHttpUtils.openConnectionStream(url)));
+            reader = new BufferedReader(new InputStreamReader(IGVHttpClientUtils.openConnectionStream(url)));
         } else {
             File file = new File(pathOrUrl);
 
@@ -70,21 +67,55 @@ public class ParsingUtils {
     }
 
 
-    public static int estimateLineCount(String filename) {
+    public static int estimateLineCount(File file) {
+        if (file.isDirectory()) {
+            int lineCount = 0;
+            for (File f : file.listFiles()) {
+                // Don't recurse
+                if (!f.isDirectory()) {
+                    lineCount += estimateLineCount(f.getAbsolutePath());
+                }
+            }
+            return lineCount;
+
+        } else {
+            return estimateLineCount(file.getAbsolutePath());
+        }
+
+    }
+
+    public static long getContentLength(String path) {
+        try {
+            long contentLength = -1;
+            if (path.startsWith("http:") || path.startsWith("https:")) {
+                URL url = new URL(path);
+                contentLength = IGVHttpClientUtils.getContentLength(url);
+
+            } else if (path.startsWith("ftp:")) {
+                // Use JDK url
+                URL url = new URL(path);
+                contentLength = url.openConnection().getContentLength();
+            } else {
+                contentLength = (new File(path)).length();
+            }
+            return contentLength;
+        } catch (IOException e) {
+            log.error("Error getting content length for: " + path, e);
+            return -1;
+        }
+    }
+
+    public static int estimateLineCount(String path) {
 
         AsciiLineReader reader = null;
         try {
-            long fileLength = 0;
-            //TODO -- ftp
-            if (filename.startsWith("http:") || filename.startsWith("https:")) {
-                URL url = new URL(filename);
-                fileLength = Long.parseLong(IGVHttpUtils.getHeaderField(url, "Content-length"));
-            } else {
-                fileLength = (new File(filename)).length();
+            final int defaultLength = 100000;
+            long fileLength = getContentLength(path);
+            if (fileLength <= 0) {
+                return defaultLength;
             }
 
-
-            reader = openAsciiReader(new ResourceLocator(filename));
+            reader = openAsciiReader(new ResourceLocator(path));
             String nextLine;
             int lines = 0;
             // Skip the first 10 lines (headers, etc)
@@ -98,7 +129,7 @@ public class ParsingUtils {
             }
 
             if (lines == 0) {
-                return 1000;
+                return defaultLength;
             }
 
             double bytesPerLine = (double) ((reader.getPosition() - startPos) / lines);
@@ -109,7 +140,11 @@ public class ParsingUtils {
             log.error("Error estimating line count", e);
             return 1000;
         } finally {
-            reader.close();
+            try {
+                reader.close();
+            } catch (Exception e) {
+                // Ignore errors closing reader
+            }
         }
 
     }
@@ -142,7 +177,7 @@ public class ParsingUtils {
 
         if (locator.getServerURL() != null) {
             URL url = new URL(locator.getServerURL() + "?method=getContents&file=" + locator.getPath());
-            InputStream is = IGVHttpUtils.openConnectionStream(url);
+            InputStream is = IGVHttpClientUtils.openConnectionStream(url);
 
             // Note -- assumption that url stream is compressed!
             try {
@@ -158,12 +193,15 @@ public class ParsingUtils {
         } else {
 
             InputStream inputStream = null;
-            if (locator.getPath().startsWith("http:") || locator.getPath().startsWith("https:") ||
-                    locator.getPath().startsWith("ftp:") || locator.getPath().startsWith("file:")) {
+            if (IGVHttpClientUtils.isURL(locator.getPath())) {
                 URL url = new URL(locator.getPath());
-                inputStream = IGVHttpUtils.openConnectionStream(url);
+                inputStream = IGVHttpClientUtils.openConnectionStream(url);
             } else {
-                File file = new File(locator.getPath());
+                String path = locator.getPath();
+                if (path.startsWith("file://")) {
+                    path = path.substring(7);
+                }
+                File file = new File(path);
                 inputStream = new FileInputStream(file);
             }
 
@@ -191,6 +229,15 @@ public class ParsingUtils {
         int nTokens = 0;
         int start = 0;
         int end = aString.indexOf(delim);
+        if(end == 0) {
+            if(aString.length() > 1) {
+                start = 1;
+                end = aString.indexOf(delim, start);
+            }
+            else {
+                return 0;
+            }
+        }
         if (end < 0) {
             tokens[nTokens++] = aString;
             return nTokens;
@@ -211,6 +258,48 @@ public class ParsingUtils {
         return nTokens;
     }
 
+    /**
+     * Split the string into tokens separated by one or more space.  This method
+     * was added so support PLINK files.
+     *
+     * @param aString the string to split
+     * @param tokens  an array to hold the parsed tokens
+     * @return the number of tokens parsed
+     */
+    public static int splitSpaces(String aString, String[] tokens) {
+
+        aString = aString.trim();
+        int maxTokens = tokens.length;
+        int nTokens = 0;
+        int start = 0;
+        int end = aString.indexOf(' ');
+        if (end < 0) {
+            tokens[nTokens++] = aString;
+            return nTokens;
+        }
+        while ((end > 0) && (nTokens < maxTokens)) {
+
+            String t = aString.substring(start, end);
+            if (t.length() > 0) {
+                tokens[nTokens++] = t;
+            }
+            start = end + 1;
+
+            end = aString.indexOf(' ', start);
+
+        }
+
+        // Add the trailing string,  if there is room and if it is not empty.
+        if (nTokens < maxTokens) {
+            String trailingString = aString.substring(start);
+            if (trailingString.length() > 0) {
+                tokens[nTokens++] = trailingString;
+            }
+        }
+        return nTokens;
+    }
+
+
     /**
      * Split the string into tokesn separated by tab or space(s).  This method
      * was added so support wig and bed files, which apparently accept
@@ -228,7 +317,7 @@ public class ParsingUtils {
         int tabEnd = aString.indexOf('\t');
         int spaceEnd = aString.indexOf(' ');
         int end = tabEnd < 0 ? spaceEnd : spaceEnd < 0 ? tabEnd : Math.min(spaceEnd, tabEnd);
-        while  ((end > 0) && (nTokens < maxTokens)) {
+        while ((end > 0) && (nTokens < maxTokens)) {
             //tokens[nTokens++] = new String(aString.toCharArray(), start, end-start); //  aString.substring(start, end);
             tokens[nTokens++] = aString.substring(start, end);
 
@@ -345,7 +434,7 @@ public class ParsingUtils {
      *
      * @param gene
      */
-    public static void computeReadingShifts(Feature gene) {
+    public static void computeReadingShifts(IGVFeature gene) {
         List<Exon> exons = gene.getExons();
         if (exons.size() == 0) {
             return;
@@ -403,20 +492,26 @@ public class ParsingUtils {
                     }
                     if (key.equals("name")) {
                         trackProperties.setName(value);
+                        //dhmay adding name check for TopHat junctions files. graphType is also checked.
+                        if (value.equals("junctions")) {
+                            trackProperties.setRendererClass(SpliceJunctionRenderer.class);
+                            trackProperties.setHeight(60);
+                        }
                     } else if (key.equals("description")) {
                         trackProperties.setDescription(value);
                     } else if (key.equals("itemrgb")) {
-                        trackProperties.setItemRGB(value.toLowerCase().equals("on"));
+                        trackProperties.setItemRGB(value.toLowerCase().equals("on") ||
+                                value.equals("1"));
                     } else if (key.equals("usescore")) {
                         trackProperties.setUseScore(value.equals("1"));
                     } else if (key.equals("color")) {
-                        Color color = ColorUtilities.convertRGBStringToColor(value);
+                        Color color = ColorUtilities.stringToColor(value);
                         trackProperties.setColor(color);
                     } else if (key.equals("altcolor")) {
-                        Color color = ColorUtilities.convertRGBStringToColor(value);
+                        Color color = ColorUtilities.stringToColor(value);
                         trackProperties.setAltColor(color);
                     } else if (key.equals("midcolor")) {
-                        Color color = ColorUtilities.convertRGBStringToColor(value);
+                        Color color = ColorUtilities.stringToColor(value);
                         trackProperties.setMidColor(color);
                     } else if (key.equals("autoscale")) {
                         boolean autoscale = value.equals("on");
@@ -425,8 +520,8 @@ public class ParsingUtils {
 
                         // Ignore the min and max
                         String[] maxDefMin = value.split(":");
-                        trackProperties.setHeight(Integer.parseInt(maxDefMin[1].trim()));
-                        trackProperties.setMinHeight(Integer.parseInt(maxDefMin[2].trim()));
+                        trackProperties.setMinHeight(Integer.parseInt(maxDefMin[1].trim()));
+                        trackProperties.setHeight(Integer.parseInt(maxDefMin[0].trim()));
 
                     } else if (key.equals("url")) {
                         trackProperties.setUrl(value);
@@ -436,10 +531,17 @@ public class ParsingUtils {
                             trackProperties.setRendererClass(BarChartRenderer.class);
                         } else if (value.equals("points")) {
                             trackProperties.setRendererClass(ScatterplotRenderer.class);
+                            trackProperties.setWindowingFunction(WindowFunction.none);
                         } else if (value.equals("line")) {
                             trackProperties.setRendererClass(LineplotRenderer.class);
                         } else if (value.equals("heatmap")) {
                             trackProperties.setRendererClass(HeatmapRenderer.class);
+                        } else if (value.equals("junctions")) {
+                            //dhmay adding check for graphType=junction.  name is also checked
+                            trackProperties.setRendererClass(SpliceJunctionRenderer.class);
+                        } else if (value.equals("genotype")) {
+                            //dhmay adding check for graphType=junction.  name is also checked
+                            trackProperties.setRendererClass(GenotypeRenderer.class);
                         }
                     } else if (key.toLowerCase().equals("viewlimits")) {
                         String[] limits = value.split(":");
@@ -467,13 +569,13 @@ public class ParsingUtils {
                         }
                     } else if (key.equals("ylinemark")) {
                         try {
-                            float midValue = Float.parseFloat(value);
-                            trackProperties.setMidValue(midValue);
+                            float yLine = Float.parseFloat(value);
+                            trackProperties.setyLine(yLine);
                         } catch (NumberFormatException e) {
                             log.error("Number format exception in track line (ylinemark): " + nextLine);
                         }
                     } else if (key.equals("ylineonoff")) {
-                        trackProperties.setDrawMidValue(value.equals("on"));
+                        trackProperties.setDrawYLine(value.equals("on"));
                     } else if (key.equals("windowingfunction")) {
                         if (value.equals("maximum")) {
                             trackProperties.setWindowingFunction(WindowFunction.max);
@@ -491,32 +593,32 @@ public class ParsingUtils {
 
                         } else if (value.equals("percentile90")) {
                             trackProperties.setWindowingFunction(WindowFunction.percentile90);
-
+                        } else if (value.equals("none")) {
+                            trackProperties.setWindowingFunction(WindowFunction.none);
                         }
-                    } else if(key.equals("maxfeaturewindow") || key.equals("featurevisibilitywindow")) {
-                        // These options are deprecated.  Use visibilityWindow
+                    } else if (key.equals("maxfeaturewindow") || key.equals("featurevisibilitywindow") ||
+                            key.equals("visibilitywindow")) {
                         try {
                             int windowSize = Integer.parseInt(value);
                             trackProperties.setFeatureVisibilityWindow(windowSize);
                         } catch (NumberFormatException e) {
-                            log.error("featureVisibilityWindow must be numeric: " + nextLine);
+                            log.error(key + " must be numeric: " + nextLine);
+
                         }
 
-                    } else if(key.equals("visibilitywindow")) {
-                        try {
-                            int windowSize = Integer.parseInt(value) * 1000;
-                            trackProperties.setFeatureVisibilityWindow(windowSize);
-                        } catch (NumberFormatException e) {
-                            log.error("featureVisibilityWindow must be an integer: " + nextLine);
-                        }
-
-                    }
-                    else if(key.equals("scaletype")) {
-                        if(value.equals("log")) {
+                    } else if (key.equals("scaletype")) {
+                        if (value.equals("log")) {
                             trackProperties.setLogScale(true);
                         }
+                    } else if (key.equals("gfftags")) {
+                        // Any value other than 0 or off => on
+                        boolean gffTags = !(value.equals("0") || (value.toLowerCase().equals("off")));
+                        trackProperties.setGffTags(gffTags);
+                    } else if (key.equals("sortable")) {
+                        // Any value other than 0 or off => on
+                        boolean sortable = (value.equals("1") || (value.toLowerCase().equals("true")));
+                        trackProperties.setSortable(sortable);
                     }
-
                 }
             }
 
@@ -534,11 +636,10 @@ public class ParsingUtils {
 
     }
 
-    public static  boolean pathExists(String covPath)  {
+    public static boolean pathExists(String covPath) {
         try {
             return (new File(covPath)).exists() ||
-                    ((covPath.startsWith("http:") || covPath.startsWith("https:")) &&
-                            IGVHttpUtils.resourceAvailable(new URL(covPath)));
+                    (IGVHttpClientUtils.isURL(covPath) && IGVHttpClientUtils.resourceAvailable(new URL(covPath)));
         } catch (MalformedURLException e) {
             // todo -- log
             return false;
