@@ -1,19 +1,19 @@
 /*
- * Copyright (c) 2007-2010 by The Broad Institute, Inc. and the Massachusetts Institute of Technology.
- * All Rights Reserved.
+ * Copyright (c) 2007-2011 by The Broad Institute of MIT and Harvard.  All Rights Reserved.
  *
- * This software is licensed under the terms of the GNU Lesser General Public License (LGPL), Version 2.1 which
- * is available at http://www.opensource.org/licenses/lgpl-2.1.php.
+ * This software is licensed under the terms of the GNU Lesser General Public License (LGPL),
+ * Version 2.1 which is available at http://www.opensource.org/licenses/lgpl-2.1.php.
  *
- * THE SOFTWARE IS PROVIDED "AS IS." THE BROAD AND MIT MAKE NO REPRESENTATIONS OR WARRANTIES OF
- * ANY KIND CONCERNING THE SOFTWARE, EXPRESS OR IMPLIED, INCLUDING, WITHOUT LIMITATION, WARRANTIES
- * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NONINFRINGEMENT, OR THE ABSENCE OF LATENT
- * OR OTHER DEFECTS, WHETHER OR NOT DISCOVERABLE.  IN NO EVENT SHALL THE BROAD OR MIT, OR THEIR
- * RESPECTIVE TRUSTEES, DIRECTORS, OFFICERS, EMPLOYEES, AND AFFILIATES BE LIABLE FOR ANY DAMAGES OF
- * ANY KIND, INCLUDING, WITHOUT LIMITATION, INCIDENTAL OR CONSEQUENTIAL DAMAGES, ECONOMIC
- * DAMAGES OR INJURY TO PROPERTY AND LOST PROFITS, REGARDLESS OF WHETHER THE BROAD OR MIT SHALL
- * BE ADVISED, SHALL HAVE OTHER REASON TO KNOW, OR IN FACT SHALL KNOW OF THE POSSIBILITY OF THE
- * FOREGOING.
+ * THE SOFTWARE IS PROVIDED "AS IS." THE BROAD AND MIT MAKE NO REPRESENTATIONS OR
+ * WARRANTES OF ANY KIND CONCERNING THE SOFTWARE, EXPRESS OR IMPLIED, INCLUDING,
+ * WITHOUT LIMITATION, WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
+ * PURPOSE, NONINFRINGEMENT, OR THE ABSENCE OF LATENT OR OTHER DEFECTS, WHETHER
+ * OR NOT DISCOVERABLE.  IN NO EVENT SHALL THE BROAD OR MIT, OR THEIR RESPECTIVE
+ * TRUSTEES, DIRECTORS, OFFICERS, EMPLOYEES, AND AFFILIATES BE LIABLE FOR ANY DAMAGES
+ * OF ANY KIND, INCLUDING, WITHOUT LIMITATION, INCIDENTAL OR CONSEQUENTIAL DAMAGES,
+ * ECONOMIC DAMAGES OR INJURY TO PROPERTY AND LOST PROFITS, REGARDLESS OF WHETHER
+ * THE BROAD OR MIT SHALL BE ADVISED, SHALL HAVE OTHER REASON TO KNOW, OR IN FACT
+ * SHALL KNOW OF THE POSSIBILITY OF THE FOREGOING.
  */
 package org.broad.igv.util;
 
@@ -29,6 +29,7 @@ public class ResourceLocator {
 
     String serverURL; //URL for the remote data server.  Null for local files
     String path; //The path for the file or resource.
+    String indexPath; // Option path to an associated index file
     String name;  // Display name.  Also used as a key for the sample info file
     String infolink; // A hyperlink to general information about the track.
     String url; //A URL pattern (UCSC convention) to a specific URL applicable to each feature
@@ -133,8 +134,7 @@ public class ResourceLocator {
     }
 
     public boolean isLocal() {
-        return serverURL == null && !(path.toLowerCase().startsWith("http:") ||
-                path.toLowerCase().startsWith("https:") || path.toLowerCase().startsWith("ftp:"));
+        return serverURL == null && !(FileUtils.isRemote(path));
     }
 
     public void setInfolink(String infolink) {
@@ -183,6 +183,7 @@ public class ResourceLocator {
     }
 
 
+
     public String getUrl() {
         return url;
     }
@@ -191,6 +192,10 @@ public class ResourceLocator {
         this.url = url;
     }
 
+    public void setPath(String path) {
+        this.path = path;
+    }
+
     public String getTrackLine() {
         return trackLine;
     }
@@ -207,6 +212,14 @@ public class ResourceLocator {
         this.sampleId = sampleId;
     }
 
+    public String getIndexPath() {
+        return indexPath;
+    }
+
+    public void setIndexPath(String indexPath) {
+        this.indexPath = indexPath;
+    }
+
 
     /**
      * FOR LOAD FROM SERVER (LEGACY)
@@ -217,7 +230,8 @@ public class ResourceLocator {
         PATH("path"),
         DESCRIPTION("description"),
         HYPERLINK("hyperlink"),
-        SAMPLE_ID("sampleID"),
+        ID("id"),
+        SAMPLE_ID("sampleId"),
         NAME("name"),
         URL("url"),
         RESOURCE_TYPE("resourceType"),
