@@ -1,10 +1,5 @@
 package org.broad.tribble;
 
-import org.broad.tribble.util.LineReader;
-
-import java.io.IOException;
-import java.io.File;
-
 /**
  * Created by IntelliJ IDEA.
  * User: jrobinso
