@@ -34,7 +34,9 @@ public class Block {
      * This method is used to aid in consolidating blocks
      */
     public void setEndPosition(long endPosition) {
-        assert(endPosition > startPosition);
+        if(endPosition < startPosition)
+            throw new IllegalArgumentException("Attempting to set block end position to " +
+                                                                           endPosition + " which is before the start of " + startPosition);
         size = (int) (endPosition - startPosition);
 
     }
@@ -45,4 +47,11 @@ public class Block {
     public int getSize() {
         return size;
     }
+
+    public boolean equals(Object obj) {
+        if ( this == obj ) return true;
+        if ( ! (obj instanceof Block) ) return false;
+        Block otherBlock = (Block)obj;
+        return this.startPosition == otherBlock.startPosition && this.size == otherBlock.size;
+    }
 }
