@@ -32,8 +32,6 @@ import java.util.*;
  * Date: Jul 9, 2010
  */
 public class IntervalTreeIndex extends AbstractIndex {
-
-
     /**
      * Default constructor -- used by factory methods.  Do not remove.
      */
@@ -50,9 +48,9 @@ public class IntervalTreeIndex extends AbstractIndex {
         return ChrIndex.class;
     }
 
-        @Override
+    @Override
     protected int getType() {
-        return IndexFactory.INTERVAL_TREE;
+        return IndexFactory.IndexType.INTERVAL_TREE.getHeaderValue();
     }
 
     public void insert(String chr, Interval interval) {
@@ -64,6 +62,11 @@ public class IntervalTreeIndex extends AbstractIndex {
         chrIdx.insert(interval);
     }
 
+    protected void setChrIndex(List<ChrIndex> indicies) {
+        for (ChrIndex index : indicies) {
+            chrIndeces.put(index.getName(),index);
+        }
+    }
 
     public void printTree() {
 
@@ -75,8 +78,6 @@ public class IntervalTreeIndex extends AbstractIndex {
         }
     }
 
-
-
     public static class ChrIndex implements org.broad.tribble.index.ChrIndex {
 
         IntervalTree tree;
@@ -101,6 +102,11 @@ public class IntervalTreeIndex extends AbstractIndex {
         public void insert(Interval iv) {
             tree.insert(iv);
         }
+        
+        public List<Block> getBlocks() {
+            return null;
+        }
+
 
         public List<Block> getBlocks(int start, int end) {
 
@@ -119,7 +125,8 @@ public class IntervalTreeIndex extends AbstractIndex {
             // Sort blocks by start position
             Arrays.sort(blocks, new Comparator<Block>() {
                 public int compare(Block b1, Block b2) {
-                    return (int) (b1.getStartPosition() - b2.getStartPosition());
+                    // this is a little cryptic because the normal method (b1.getStartPosition() - b2.getStartPosition()) wraps in int space and we incorrectly sort the blocks in extreme cases
+                    return b1.getStartPosition() - b2.getStartPosition() < 1 ? -1 : (b1.getStartPosition() - b2.getStartPosition() > 1 ? 1 : 0);
                 }
             });
 
